!function($, window, document, _undefined)
{
    "use strict";

    XF.XFARMMPPurchaseFormContainer = XF.Element.newHandler({
        coupon: null,

        totalPrice: 0.0,

        options: {
            physical: false,
            resourcePrice: 0.0,
            resourceCurrency: "",
            requireCgvValidation: false,
            hasQuantity: false,
            hasShipping: false,
            couponValidateUrl: null
        },

        init: function ()
        {
            // CGV agreement handling (if any)
            if (this.options.requireCgvValidation)
            {
                this.$target.find('input[name="xfa_agree"]').on('change', XF.proxy(this, 'cgvAgreementCheckboxChanged'));
            }

            // Add price update on quantity change if any
            if (this.options.hasQuantity)
            {
                this.$target.find('input[name="quantity"]').on('change', XF.proxy(this, 'updatePrice'));
            }

            // Add price update on shipping change if any
            if (this.options.hasShipping)
            {
                this.$target.find('select[name="shipping"]').on('change', XF.proxy(this, 'updatePrice'));
            }

            // Coupon code validate push
            this.$target.find('button[name="coupon-check-button"]').on('click', XF.proxy(this, 'validateCoupon'));

            // Submit event
            this.$target.on('submit', XF.proxy(this, 'submit'));

            // Update price
            this.updatePrice();
        },

        cgvAgreementCheckboxChanged: function(event)
        {
            // Activate button if checkbox is checked
            if (this.$target.find('input[name="xfa_agree"]').is(':checked'))
            {
                this.$target.find(':submit').prop('disabled', false);
                this.$target.find(':submit').removeClass('is-disabled');
            }
            // Deactivate it otherwise
            else
            {
                this.$target.find(':submit').prop('disabled', true);
                this.$target.find(':submit').addClass('is-disabled');
            }
        },

        validateCoupon: function (event)
        {
            // Skip if no coupon code
            var couponCode = this.$target.find('input[name="coupon_code"]').val();
            if (!couponCode)
            {
                return;
            }

            XF.ajax('get', this.options.couponValidateUrl, {coupon: couponCode}, XF.proxy(this, 'handleCouponCodeValidationAjax'));
        },

        handleCouponCodeValidationAjax: function (data)
        {
            this.$target.find('.coupon_validation_phrase').html(data.validationPhrase);
            this.$target.find('.coupon_validation_phrase').css('margin-top', '10px');

            if (data.usable)
            {
                this.coupon = data.coupon;
                this.coupon['usages'] = data.usages;
            }
            else
            {
                this.coupon = null;
            }

            this.updatePrice();
        },

        getResourcePriceQtyBased: function()
        {
            var resourcePrice = 0.0;

            if (this.options.hasQuantity)
            {
                resourcePrice = parseInt(this.$target.find ('input[name="quantity"]').val()) * this.options.resourcePrice;
            }
            else
            {
                resourcePrice = this.options.resourcePrice;
            }

            return resourcePrice;
        },

        updatePrice: function(event)
        {
            // Get resource price total
            var resourcePrice = 0.0;

            if (this.coupon)
            {
                switch (this.coupon.coupon_type)
                {
                    // DISCOUNT
                    case 'discount':
                        resourcePrice = this.getResourcePriceQtyBased() - parseFloat(this.coupon.coupon_amount);
                        break;
                    // AMOUNT
                    case 'percentage':
                        var initialResourcePrice = this.getResourcePriceQtyBased();
                        resourcePrice = initialResourcePrice - ((initialResourcePrice * parseFloat(this.coupon.coupon_amount)) / 100);
                        break;
                    // Free product
                    default:
                        if (this.options.hasQuantity)
                        {
                            resourcePrice = (parseInt(this.$target.find ('input[name="quantity"]').val()) - 1) * this.options.resourcePrice;
                        }
                        else
                        {
                            resourcePrice = 0.0;
                        }
                        break;
                }

                resourcePrice = Math.max(resourcePrice, 0.0);
            }
            else
            {
                resourcePrice = this.getResourcePriceQtyBased();
            }

            // Get shipping
            var shippingCost = 0.0;
            if (this.options.physical && this.options.hasShipping)
            {
                shippingCost = parseFloat(this.$target.find('select[name="shipping"]').val());
            }

            // Compute total
            this.totalPrice = resourcePrice + shippingCost;

            this.$target.find('#price_row dd').html(this.totalPrice.toFixed(2) + ' '  + this.$target.data('resource-currency'));
        },

        submit: function(event)
        {
            this.updatePrice();

            if (this.totalPrice == 0.0)
            {
                this.$target.attr('action', this.$target.data('free-purchase-url'));
            }

            if (this.$target.find('input[name="payment_profile_id"]:checked').val() == "manual")
            {
                this.$target.attr('action', this.$target.data('manual-purchase-url'));
            }
        }
    });

    XF.Element.register('xfa-rmmp-purchase-form-container', 'XF.XFARMMPPurchaseFormContainer');
}
(jQuery, window, document);