!function($, window, document, _undefined)
{
    "use strict";

    XF.XFARMMPPaymentProviderContainerRenew = XF.extend(XF.PaymentProviderContainer, {
        __backup: {
            'init': '_init'
        },

        coupon: null,

        totalPrice: 0.0,

        options: $.extend({}, XF.PaymentProviderContainer.prototype.options, {
            renewalPrice: 0.0,
            renewalCurrency: "",
            requireCgvValidation: false,
            couponValidateUrl: null
        }),

        init: function ()
        {
            this._init();

            // CGV agreement handling (if any)
            if (this.options.requireCgvValidation)
            {
                this.$target.find('input[name="xfa_agree"]').on('change', XF.proxy(this, 'cgvAgreementCheckboxChanged'));
            }

            // Coupon code validate push
            this.$target.find('button[name="coupon-check-button"]').on('click', XF.proxy(this, 'validateCoupon'));

            // Submit event
            this.$target.on('submit', XF.proxy(this, 'submit'));

            // Update price
            this.updatePrice();
        },

        cgvAgreementCheckboxChanged: function(event)
        {
            // Activate button if checkbox is checked
            if (this.$target.find('input[name="xfa_agree"]').is(':checked'))
            {
                this.$target.find(':submit').prop('disabled', false);
                this.$target.find(':submit').removeClass('is-disabled');
            }
            // Deactivate it otherwise
            else
            {
                this.$target.find(':submit').prop('disabled', true);
                this.$target.find(':submit').addClass('is-disabled');
            }
        },

        validateCoupon: function (event)
        {
            // Skip if no coupon code
            var couponCode = this.$target.find('input[name="coupon_code"]').val();
            if (!couponCode)
            {
                return;
            }

            XF.ajax('get', this.options.couponValidateUrl, {coupon: couponCode}, XF.proxy(this, 'handleCouponCodeValidationAjax'));
        },

        handleCouponCodeValidationAjax: function (data)
        {
            this.$target.find('.coupon_validation_phrase').html(data.validationPhrase);
            this.$target.find('.coupon_validation_phrase').css('margin-top', '10px');

            if (data.usable)
            {
                this.coupon = data.coupon;
                this.coupon['usages'] = data.usages;
            }
            else
            {
                this.coupon = null;
            }

            this.updatePrice();
        },

        updatePrice: function(event)
        {
            // Get renewal price total
            var renewalPrice = 0.0;

            if (this.coupon)
            {
                switch (this.coupon.coupon_type)
                {
                    // DISCOUNT
                    case 'discount':
                        renewalPrice = this.options.renewalPrice - parseFloat(this.coupon.coupon_amount);
                        break;
                    // AMOUNT
                    case 'percentage':
                        var initialRenewalPrice = this.options.renewalPrice;
                        renewalPrice = initialRenewalPrice - ((initialRenewalPrice * parseFloat(this.coupon.coupon_amount)) / 100);
                        break;
                    // Free product
                    default:
                        renewalPrice = 0.0;
                        break;
                }

                renewalPrice = Math.max(renewalPrice, 0.0);
            }
            else
            {
                renewalPrice = this.options.renewalPrice;
            }

            // Compute total
            this.totalPrice = renewalPrice;

            this.$target.find('#price_row dd').html(this.totalPrice.toFixed(2) + ' '  + this.options.renewalCurrency);
        },

        submit: function(event)
        {
            this.updatePrice();

            if (this.totalPrice == 0.0)
            {
                this.$target.attr('action', this.$target.data('free-purchase-url'));
            }
        }
    });

    XF.Element.register('xfa-rmmp-payment-provider-container-renew', 'XF.XFARMMPPaymentProviderContainerRenew');
}
(jQuery, window, document);