<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Admin\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class FeaturingDuration extends \XF\Admin\Controller\AbstractController
{
    public function actionIndex()
    {
        $durations = $this->finder('XFA\RMMarketplace:FeaturingDuration')->order(['display_order'])->fetch();

        $viewParams = [
            'totalDurations'    => $durations->count(),
            'durations'         => $durations
        ];
        return $this->view('', 'xfa_rmmp_featuring_duration_list', $viewParams);
    }

    protected function _addEdit(\XFA\RMMarketplace\Entity\FeaturingDuration $duration)
    {
        $viewParams = [
            'duration' => $duration
        ];
        return $this->view('', 'xfa_rmmp_featuring_duration_edit', $viewParams);
    }

    public function actionEdit(ParameterBag $params)
    {
        $duration = $this->assertDurationExists($params['duration_id']);
        return $this->_addEdit($duration);
    }

    public function actionAdd()
    {
        $duration = $this->em()->create('XFA\RMMarketplace:FeaturingDuration');
        return $this->_addEdit($duration);
    }

    protected function saveProcess(\XFA\RMMarketplace\Entity\FeaturingDuration $duration)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'value'         => 'uint',
            'type'          => 'str',
            'amount'        => 'float',
            'active'        => 'bool',
            'display_order' => 'uint'
        ]);
        $form->basicEntitySave($duration, $input);

        return $form;
    }

    public function actionSave(ParameterBag $params)
    {
        $this->assertPostOnly();

        if ($params['duration_id'])
        {
            $duration = $this->assertDurationExists($params['duration_id']);
        }
        else
        {
            $duration = $this->em()->create('XFA\RMMarketplace:FeaturingDuration');
        }

        $this->saveProcess($duration)->run();

        return $this->redirect($this->buildLink('rm-marketplace/featuring-duration') . $this->buildLinkHash($duration->duration_id));
    }

    public function actionDelete(ParameterBag $params)
    {
        $duration = $this->assertDurationExists($params['duration_id']);
        if (!$duration->preDelete())
        {
            return $this->error($duration->getErrors());
        }

        if ($this->isPost())
        {
            $duration->delete();

            return $this->redirect($this->buildLink('rm-marketplace/featuring-duration'));
        }
        else
        {
            $viewParams = [
                'duration' => $duration
            ];
            return $this->view('', 'xfa_rmmp_featuring_duration_delete', $viewParams);
        }
    }

    public function actionToggle()
    {
        /** @var \XF\ControllerPlugin\Toggle $plugin */
        $plugin = $this->plugin('XF:Toggle');
        return $plugin->actionToggle('XFA\RMMarketplace:FeaturingDuration');
    }

    protected function assertDurationExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('XFA\RMMarketplace:FeaturingDuration', $id, $with, $phraseKey);
    }
}