<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Admin\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class FeaturingPurchase extends \XF\Admin\Controller\AbstractController
{
    public function actionIndex()
    {
        $page = $this->filterPage();
        $perPage = 20;

        /** @var \XFA\RMMarketplace\Repository\FeaturingPurchase $featuringPurchaseRepo */
        $featuringPurchaseRepo = $this->repository('XFA\RMMarketplace:FeaturingPurchase');

        $purchases = $featuringPurchaseRepo->findLogsForList()->limitByPage($page, $perPage);

        $linkFilters = [];
        if ($status = $this->filter('status', 'str'))
        {
            $linkFilters['status'] = $status;

            if ($status == 'wait')
            {
                $purchases->where('status', 'validated');
                $purchases->where('validated', 0);
            }
            else
            {
                $purchases->where('status', $status);
            }
        }

        if ($userId = $this->filter('user_id', 'uint'))
        {
            $linkFilters['user_id'] = $userId;
            $purchases->where('user_id', $userId);
        }

        if ($this->isPost())
        {
            // redirect to give a linkable page
            return $this->redirect($this->buildLink('rm-marketplace/featuring-purchase', null, $linkFilters));
        }

        $viewParams = [
            'entries'       => $purchases->fetch(),
            'users'         => $featuringPurchaseRepo->getUsersWhoPurchased(),
            'page'          => $page,
            'perPage'       => $perPage,
            'total'         => $purchases->total(),
            'linkFilters'   => $linkFilters
        ];
        return $this->view('', 'xfa_rmmp_featuring_purchase_list', $viewParams);
    }

    public function actionAccept(ParameterBag $params)
    {
        $purchase = $this->assertPurchaseExists($params['purchase_id']);

        if ($this->isPost())
        {
            /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Accept $acceptor */
            $acceptor = $this->service('XFA\RMMarketplace:FeaturingPurchase\Accept', $purchase);
            $acceptor->accept();

            return $this->redirect($this->buildLink('rm-marketplace/featuring-purchase'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];
            return $this->view('', 'xfa_rmmp_featuring_purchase_accept', $viewParams);
        }
    }

    public function actionRefund(ParameterBag $params)
    {
        $purchase = $this->assertPurchaseExists($params['purchase_id']);

        if ($this->isPost())
        {
            /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Accept $refunder */
            $refunder = $this->service('XFA\RMMarketplace:FeaturingPurchase\Refund', $purchase);
            $refunder->refund();

            return $this->redirect($this->buildLink('rm-marketplace/featuring-purchase'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];
            return $this->view('', 'xfa_rmmp_featuring_purchase_refund', $viewParams);
        }
    }

    protected function assertPurchaseExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('XFA\RMMarketplace:FeaturingPurchase', $id, $with, $phraseKey);
    }
}