<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Admin\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class Invoice extends \XF\Admin\Controller\AbstractController
{
    public function actionIndex()
    {
	    if (\XF::options()->xfa_rmmp_mode != 'xf')
	    {
			throw $this->exception(
				$this->notFound(\XF::phrase('xfa_rmmp_custom_payment_processing_method_selected'))
			);
	    }
	    
        $page = $this->filterPage();
        $perPage = 20;

        /** @var \XFA\RMMarketplace\Repository\MonthlyInvoice $invoiceRepo */
        $invoiceRepo = $this->repository('XFA\RMMarketplace:MonthlyInvoice');
        
        $invoices = $invoiceRepo->findInvoicesForList()->limitByPage($page, $perPage);

        $linkFilters = [];
        if ($status = $this->filter('status', 'str'))
        {
            $linkFilters['status'] = $status;

            $invoices->where('status', $status);
        }

        if ($userId = $this->filter('user_id', 'uint'))
        {
            $linkFilters['user_id'] = $userId;
            $invoices->where('user_id', $userId);
        }

        if ($this->isPost())
        {
            // redirect to give a linkable page
            return $this->redirect($this->buildLink('rm-marketplace/invoices', null, $linkFilters));
        }

        $viewParams = [
            'entries'       => $invoices->fetch(),
            'users'         => $invoiceRepo->getUsersWithInvoices(),
            'page'          => $page,
            'perPage'       => $perPage,
            'total'         => $invoices->total(),
            'linkFilters'   => $linkFilters
        ];
        return $this->view('', 'xfa_rmmp_invoices_list', $viewParams);
    }
    
    public function actionSendReminder(ParameterBag $params)
    {
	    if (\XF::options()->xfa_rmmp_mode != 'xf')
	    {
			throw $this->exception(
				$this->notFound(\XF::phrase('xfa_rmmp_custom_payment_processing_method_selected'))
			);
	    }
	    
	    $invoice = $this->finder('XFA\RMMarketplace:MonthlyInvoice')->with('Seller')->whereId($params->invoice_id)->fetchOne();
	    if (!$invoice)
	    {
			throw $this->exception(
				$this->notFound(\XF::phrase('xfa_rmmp_invoice_not_found'))
			);		    
	    }
	    
	    if ($this->isPost())
        {
	        // Notify seller     
	        /** @var \XFA\RMMarketplace\Service\PosInvoice\Notify $notifier */
	        $notifier = \XF::service('XFA\RMMarketplace:PosInvoice\Notify', $invoice);
	        
	        $notifier->emailSeller('xfa_rmmp_invoice_pending');
	
	        $notifier->alertSeller('xfa_rmmp_invoice_pending');   
	        
			return $this->redirect($this->buildLink('rm-marketplace/invoices') . $this->buildLinkHash($invoice->invoice_id));
        }
        else
        {
            $viewParams = [
                'invoice' => $invoice
            ];
            return $this->view('', 'xfa_rmmp_invoice_send_reminder', $viewParams);
        }	    
    }
}