<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2023
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class Coupon extends AbstractPlugin
{
    public function getListData($type = 'all')
    {
        /** @var \XFA\RMMarketplace\Finder\Coupon $couponFinder */
        $couponFinder = $this->finder('XFA\RMMarketplace:Coupon');

        $filters = $this->getCouponFilterInput();
        $this->applyCouponFilters($couponFinder, $filters, $type);

        $totalCoupons = $couponFinder->total();

        $page       = $this->filterPage();
        $perPage    = $this->options()->xfrmResourcesPerPage;

        $couponFinder->limitByPage($page, $perPage);
        $coupons = $couponFinder->fetch();

        return [
            'coupons'   => $coupons,
            'filters'   => $filters,
            'total'     => $totalCoupons,
            'page'      => $page,
            'perPage'   => $perPage
        ];
    }

    public function applyCouponFilters(\XFA\RMMarketplace\Finder\Coupon $couponFinder, array $filters, $type)
    {
        if (!empty($filters['status']))
        {
            $time = \XF::$time;

            switch ($filters['status'])
            {
                case 'ongoing':
                    $couponFinder->where('from_date', '<', $time);
                    $couponFinder->where('to_date', '>', $time);
                    break;

                case 'upcoming':
                    $couponFinder->where('from_date', '>', $time);
                    break;

                case 'past':
                    $couponFinder->where('to_date', '<', $time);
                    break;
            }
        }

        if (!empty($filters['coupon_type']))
        {
            $couponFinder->where('coupon_type', $filters['coupon_type']);
        }

        if (!empty($filters['apply_to']))
        {
            $couponFinder->where('apply_to', $filters['apply_to']);
        }

        if (!empty($filters['order']))
        {
            $couponFinder->order($filters['order'], $filters['direction']);
        }
        else
        {
            $couponFinder->order('coupon_id', 'desc');
        }

        if ($type != 'all')
        {
            $couponFinder->where('creator_id', \XF::visitor()->user_id);
        }
    }

    public function getCouponFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'status'        => 'str',
            'coupon_type'   => 'str',
            'apply_to'      => 'str',
            'order'         => 'str',
            'direction'     => 'str'
        ]);

        if ($input['status'] && \in_array($input['status'], ['ongoing', 'upcoming', 'past']))
        {
            $filters['status'] = $input['status'];
        }

        if ($input['coupon_type'] && \in_array($input['coupon_type'], ['free', 'percentage', 'fixed_amount']))
        {
            $filters['coupon_type'] = $input['coupon_type'];
        }

        if ($input['apply_to'] && \in_array($input['apply_to'], ['purchase', 'renewal', 'both']))
        {
            $filters['apply_to'] = $input['apply_to'];
        }

        if ($input['order'] && \in_array($input['order'], ['coupon_id', 'from_date', 'to_date', 'total_usages']))
        {
            if (!\in_array($input['direction'], ['asc', 'desc']))
            {
                $input['direction'] = 'desc';
            }

            $defaultOrder   = 'coupon_id';
            $defaultDir     = 'desc';

            if ($input['order'] != $defaultOrder || $input['direction'] != $defaultDir)
            {
                $filters['order']       = $input['order'];
                $filters['direction']   = $input['direction'];
            }
        }

        return $filters;
    }

    public function actionFilters($type)
    {
        $filters = $this->getCouponFilterInput();

        if ($this->filter('apply', 'bool'))
        {
            return $this->redirect($this->buildLink(
                'resources/market-place-dashboard/coupon',
                null,
                $filters
            ));
        }

        $viewParams = [
            'filters' => $filters,
        ];

        return $this->view('', 'xfa_rmmp_coupon_filters', $viewParams);
    }
}