<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2022
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class Discount extends AbstractPlugin
{
    public function getListData($type = 'all')
    {
        /** @var \XFA\RMMarketplace\Finder\Discount $discountFinder */
        $discountFinder = $this->finder('XFA\RMMarketplace:Discount');

        $filters = $this->getDiscountFilterInput();
        $this->applyDiscountFilters($discountFinder, $filters, $type);

        $totalDiscounts = $discountFinder->total();

        $page       = $this->filterPage();
        $perPage    = $this->options()->xfrmResourcesPerPage;

        $discountFinder->limitByPage($page, $perPage);
        $discounts = $discountFinder->fetch();

        return [
            'discounts'         => $discounts,
            'filters'           => $filters,
            'total'             => $totalDiscounts,
            'page'              => $page,
            'perPage'           => $perPage
        ];
    }

    public function applyDiscountFilters(\XFA\RMMarketplace\Finder\Discount $discountFinder, array $filters, $type)
    {
        if (!empty($filters['status']))
        {
            $time = \XF::$time;

            switch ($filters['status'])
            {
                case 'ongoing':
                    $discountFinder->where('from_date', '<', $time);
                    $discountFinder->where('to_date', '>', $time);
                    break;

                case 'upcoming':
                    $discountFinder->where('from_date', '>', $time);
                    break;

                case 'past':
                    $discountFinder->where('to_date', '<', $time);
                    break;
            }
        }

        if (!empty($filters['order']))
        {
            $discountFinder->order($filters['order'], $filters['direction']);
        }
        else
        {
            $discountFinder->order('discount_id', 'desc');
        }

        if ($type != 'all')
        {
            $discountFinder->where('creator_id', \XF::visitor()->user_id);
        }
    }

    public function getDiscountFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'status'    => 'str',
            'order'     => 'str',
            'direction' => 'str'
        ]);

        if ($input['status'] && \in_array($input['status'], ['ongoing', 'upcoming', 'past']))
        {
            $filters['status'] = $input['status'];
        }

        if ($input['order'] && \in_array($input['order'], ['discount_id', 'from_date', 'to_date']))
        {
            if (!\in_array($input['direction'], ['asc', 'desc']))
            {
                $input['direction'] = 'desc';
            }

            $defaultOrder   = 'discount_id';
            $defaultDir     = 'desc';

            if ($input['order'] != $defaultOrder || $input['direction'] != $defaultDir)
            {
                $filters['order']       = $input['order'];
                $filters['direction']   = $input['direction'];
            }
        }

        return $filters;
    }

    public function actionFilters($type)
    {
        $filters = $this->getDiscountFilterInput();

        if ($this->filter('apply', 'bool'))
        {
            return $this->redirect($this->buildLink(
                'resources/market-place-dashboard/discount',
                null,
                $filters
            ));
        }

        $viewParams = [
            'filters' => $filters,
        ];

        return $this->view('', 'xfa_rmmp_discount_filters', $viewParams);
    }
}