<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2022
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Discount extends Entity
{
    public function getTypePhrase()
    {
        switch ($this->type)
        {
            case 'all':
                $phraseName = 'xfa_rmmp_all_products';
                break;
            case 'all_own':
                $phraseName = 'xfa_rmmp_all_my_products';
                break;
            default:
                $phraseName = 'xfa_rmmp_specific_resources';
                break;
        }

        return \XF::phrase($phraseName)->render();
    }

    public function getDiscountAmountPhrase()
    {
        if ($this->is_fixed)
        {
            $phraseName = 'xfa_rmmp_x_amount';
        }
        else
        {
            $phraseName = 'xfa_rmmp_x_percent';
        }
        return \XF::phrase($phraseName, ['amount' => $this->discount_amount])->render();
    }

    public function applyDiscount()
    {
        $discountApplierParams = [
            'discount_ids'  => [$this->discount_id],
            'discount_id'   => $this->discount_id
        ];

        \XF::app()->jobManager()->enqueue('XFA\RMMarketplace:DiscountsApplier', $discountApplierParams);
    }

    public function removeDiscount()
    {
        $discountRemoverParams = [
            'discount_ids'  => [$this->discount_id],
            'discount_id'   => $this->discount_id
        ];

        \XF::app()->jobManager()->enqueue('XFA\RMMarketplace:DiscountsRemover', $discountRemoverParams);
    }

    public function _postSave()
    {
        parent::_postSave();

        // Apply the discount to resources on insert or update if previous not in past and new past
        if ($this->isInsert()
            && $this->from_date < \XF::$time)
        {
            $this->applyDiscount();
        }

        if ($this->isUpdate()
                && $this->isChanged('from_date')
                && $this->getExistingValue('from_date') > \XF::$time
                && $this->from_date < \XF::$time)
        {
            $this->applyDiscount();
        }
    }

    public function _postDelete()
    {
        parent::_postDelete();

        // Remove the discount from resources
        $this->removeDiscount();
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_discount';
        $structure->shortName = 'XFA\RMMarketplace:Discount';
        $structure->primaryKey = 'discount_id';
        $structure->columns = [
            'discount_id'       => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'name'              => ['type' => self::STR, 'default' => '', 'maxLength' => 100],
            'creator_id'        => ['type' => self::UINT, 'required' => true],
            'discount_amount'   => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'is_fixed'          => ['type' => self::UINT, 'default' => 0],
            'applyto_renewal'   => ['type' => self::UINT, 'default' => 0],
            'type'              => ['type' => self::STR, 'required' => true, 'default' => 'all_own',
                'allowedValues' => ['all', 'all_own', 'specifics']
            ],
            'resource_ids'      => ['type' => self::LIST_COMMA, 'default' => []],
            'from_date'         => ['type' => self::UINT, 'default' => \XF::$time, 'required' => true],
            'to_date'           => ['type' => self::UINT, 'default' => (\XF::$time + 86400), 'required' => true],
            'applied'           => ['type' => self::UINT, 'default' => 0]
        ];
        $structure->getters = [];
        $structure->relations = [];

        return $structure;
    }
}