<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class FeaturingPurchase extends Entity
{
    public function getFeatureExpiration()
    {
        return (new \DateTime())->setTimestamp($this->Resource->Featured->feature_date)->modify('+' . $this->duration)->getTimestamp();
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_featuring_purchase';
        $structure->shortName = 'XFA\RMMarketplace:FeaturingPurchase';
        $structure->primaryKey = 'purchase_id';
        $structure->columns = [
            'purchase_id'       => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'resource_id'       => ['type' => self::UINT, 'required' => true],
            'transaction_id'    => ['type' => self::UINT],
            'status'            => ['type' => self::STR, 'default' => 'pending',
                'allowedValues' => ['pending', 'validated', 'refunded', 'expired']
            ],
            'user_id'           => ['type' => self::UINT, 'required' => true],
            'username'          => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'price'             => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'currency'          => ['type' => self::STR, 'default' => '', 'maxLength' => 3],
            'duration'          => ['type' => self::STR, 'default' => ''],
            'purchase_date'     => ['type' => self::UINT, 'default' => \XF::$time],
            'validated'         => ['type' => self::UINT, 'default' => 0],
            'validation_date'   => ['type' => self::UINT, 'default' => \XF::$time],
            'refund_txn_id'     => ['type' => self::STR, 'default' => '']
        ];
        $structure->getters = [];
        $structure->relations = [
            'Resource' => [
                'entity'        => 'XFRM:ResourceItem',
                'type'          => Entity::TO_ONE,
                'conditions'    => 'resource_id'
            ],
            'Buyer' => [
                'entity'        => 'XF:User',
                'type'          => self::TO_ONE,
                'conditions'    => 'user_id',
                'primary'       => true
            ],
            'Transaction' => [
                'entity'        => 'XFA\RMMarketplace:Transaction',
                'type'          => self::TO_ONE,
                'conditions'    => 'transaction_id'
            ],
        ];
        $structure->defaultWith = ['Resource'];

        return $structure;
    }
}