<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class License extends Entity
{
    public function getResourceTitle()
    {
        return $this->Resource ? $this->Resource->title : '';
    }

    public function isExpired()
    {
        if ($this->end_date < \XF::$time)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function hasNoUrlSet()
    {
        if ($this->siteurl == '')
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function getRemainingDays()
    {
        if ($this->end_date > \XF::$time)
        {
            return ceil(($this->end_date  - \XF::$time) / (24 * 60 * 60));
        }
        else
        {
            return 0;
        }
    }

    public function canBeRenewed()
    {
        $remainingDays = $this->getRemainingDays();

        // Expired and not allowd to renew after expiration
        if ($remainingDays == 0 && !$this->Resource->DigitalProduct->allow_expired_renewal)
        {
            return false;
        }

        // Check if renewal is already allowed
        if ($remainingDays <= $this->Resource->DigitalProduct->delay_renewal)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function isOwner()
    {
        return ($this->user_id == \XF::visitor()->user_id);
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_license';
        $structure->shortName = 'XFA\RMMarketplace:License';
        $structure->primaryKey = 'license_id';
        $structure->columns = [
            'license_id'        => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'resource_id'       => ['type' => self::UINT, 'required' => true],
            'purchase_id'       => ['type' => self::UINT],
            'user_id'           => ['type' => self::UINT, 'required' => true],
            'username'          => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'validated'         => ['type' => self::BOOL, 'required' => true],
            'active'            => ['type' => self::BOOL, 'required' => true],
            'siteurl'           => ['type' => self::STR, 'default' => ''],
            'start_date'        => ['type' => self::UINT, 'default' => \XF::$time, 'required' => true],

            'end_date'          => ['type' => self::UINT, 'default' => \XF::$time, 'required' => true],
            'previous_end_date' => ['type' => self::UINT],
            'expiration_sent'   => ['type' => self::UINT, 'default' => 0],
            'lic_key'           => ['type' => self::STR]
        ];
        $structure->getters = [
            'resource_title' => true
        ];
        $structure->relations = [
            'Resource' => [
                'entity'        => 'XFRM:ResourceItem',
                'type'          => self::TO_ONE,
                'conditions'    => 'resource_id',
                'primary'       => true
            ],
            'Purchase' => [
                'entity'        => 'XFA\RMMarketplace:Purchase',
                'type'          => self::TO_ONE,
                'conditions'    => 'purchase_id'
            ],
            'Holder' => [
                'entity'        => 'XF:User',
                'type'          => self::TO_ONE,
                'conditions'    => 'user_id',
                'primary'       => true
            ],
        ];

        return $structure;
    }
}