<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class MonthlyInvoice extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_invoices';
        $structure->shortName = 'XFA\RMMarketplace:MonthlyInvoice';
        $structure->primaryKey = 'invoice_id';
        $structure->columns = [
            'invoice_id'        => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'user_id'           => ['type' => self::UINT, 'required' => true],
            'username'          => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'price'             => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'currency'          => ['type' => self::STR, 'default' => '', 'maxLength' => 3],
            'status'            => ['type' => self::STR, 'default' => 'pending',
                'allowedValues' => ['pending', 'validated', 'refunded', 'expired']
            ],
            'invoice_date'      => ['type' => self::UINT, 'default' => \XF::$time, 'required' => true],
            'warned'            => ['type' => self::BOOL]
		];
        $structure->getters = [];
        $structure->relations = [
            'Seller' => [
                'entity'        => 'XF:User',
                'type'          => self::TO_ONE,
                'conditions'    => 'user_id',
                'primary'       => true
            ],
        ];

        return $structure;
    }
}