<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Purchase extends Entity
{
    public function getResourceTitle()
    {
        return $this->Resource ? $this->Resource->title : '';
    }

    public function canView()
    {
        if (!\XF::visitor()->user_id
            || !(
                \XF::visitor()->user_id == $this->user_id
                    || \XF::visitor()->user_id == $this->Resource->user_id
                    || \XF::visitor()->canViewAllSales()))
        {
            return false;
        }

        return true;
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_purchase';
        $structure->shortName = 'XFA\RMMarketplace:Purchase';
        $structure->primaryKey = 'purchase_id';
        $structure->columns = [
            'purchase_id'       => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'resource_id'       => ['type' => self::UINT, 'required' => true],
            'transaction_id'    => ['type' => self::UINT],
            'license_id'        => ['type' => self::UINT, 'default' => 0],
            'quantity'          => ['type' => self::UINT, 'default' => 1],
            'user_id'           => ['type' => self::UINT, 'required' => true],
            'username'          => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'for_user_id'       => ['type' => self::UINT, 'default' => 0],
            'for_username'      => ['type' => self::STR, 'maxLength' => 50, 'default' => ''],
            'price'             => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'currency'          => ['type' => self::STR, 'default' => '', 'maxLength' => 3],
            'type'              => ['type' => self::STR, 'default' => 'purchase',
                'allowedValues' => ['purchase', 'renewal']
            ],
            'status'            => ['type' => self::STR, 'default' => 'pending',
                'allowedValues' => ['pending', 'validated', 'refunded']
            ],
            'shipment_status'   => ['type' => self::STR, 'default' => 'pending',
                'allowedValues' => ['pending', 'in_preparation', 'sent', 'delivered']
            ],
            'shipment_data'     => ['type' => self::STR],
            'purchase_date'     => ['type' => self::UINT, 'default' => \XF::$time],
            'validation_date'   => ['type' => self::UINT, 'default' => \XF::$time]
        ];
        $structure->getters = [
            'resource_title' => true
        ];
        $structure->relations = [
            'Resource' => [
                'entity'        => 'XFRM:ResourceItem',
                'type'          => self::TO_ONE,
                'conditions'    => 'resource_id',
                'primary'       => true
            ],
            'License' => [
                'entity'        => 'XFA\RMMarketplace:License',
                'type'          => self::TO_ONE,
                'conditions'    => 'license_id',
                'primary'       => true
            ],
            'PurchaseInfo' => [
                'entity'        => 'XFA\RMMarketplace:PurchaseInfo',
                'type'          => self::TO_ONE,
                'conditions'    => 'purchase_id',
                'primary'       => true
            ],
            'Transaction' => [
                'entity'        => 'XFA\RMMarketplace:Transaction',
                'type'          => self::TO_ONE,
                'conditions'    => 'transaction_id'
            ],
            'Buyer' => [
                'entity'        => 'XF:User',
                'type'          => self::TO_ONE,
                'conditions'    => 'user_id',
                'primary'       => true
            ],
            'ForUser' => [
	            'entity'        => 'XF:User',
	            'type'          => Entity::TO_ONE,
	            'conditions'    => [
	                ['user_id', '=', '$for_user_id']
	            ]
	        ],
            'CouponUsage' => [
                'entity'        => 'XFA\RMMarketplace:CouponUsage',
                'type'          => self::TO_ONE,
                'conditions'    => 'purchase_id'
            ]
        ];

        return $structure;
    }
}