<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Seller extends Entity
{
    protected $_splitAddress = null;

    public function offsetGet($key)
    {
        $data = parent::offsetGet($key);

        switch ($key)
        {
            case 'comm_title_purchase':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_thank_you_x_for_your_purchase_of_y')->render();
                }
                break;
            case 'comm_text_purchase':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_thank_you_x_for_your_purchase_of_y_text')->render();
                }
                break;
            case 'comm_title_purchase_refund':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_x_purchase_of_y_refund')->render();
                }
                break;
            case 'comm_text_purchase_refund':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_x_purchase_of_y_refund_text')->render();
                }
                break;
            case 'comm_title_renewal':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_thank_you_x_for_your_renewal_of_y')->render();
                }
                break;
            case 'comm_text_renewal':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_thank_you_x_for_your_renewal_of_y_text')->render();
                }
                break;
            case 'comm_title_renewal_refund':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_x_purchase_renewal_of_y_refund')->render();
                }
                break;
            case 'comm_text_renewal_refund':
                if (empty($data) || !$this->User->hasPermission('general', 'customizeMarketplaceComm'))
                {
                    $data = \XF::phrase('xfa_rmmp_x_purchase_renewal_of_y_refund_text')->render();
                }
                break;
        }

        return $data;
    }

    public function hasLogo()
    {
        return $this->logo_date;
    }

    public function getLogoUrl($withDate = true)
    {
        if ($withDate)
        {
            return $this->app()->applyExternalDataUrl(sprintf('xfa/rmmarketplace/logo_images/%d/%d.png?%d',
                floor($this->user_id / 1000),
                $this->user_id,
                $this->logo_date
            ), true);
        }
        else
        {
            return $this->app()->applyExternalDataUrl(sprintf('xfa/rmmarketplace/logo_images/%d/%d.png',
                floor($this->user_id / 1000),
                $this->user_id
            ), true);

        }
    }

    public function getAddressRow($idx)
    {
        if (!$this->_splitAddress)
        {
            $this->_splitAddress = explode('\n', $this->address);
        }

        if (isset($this->_splitAddress[$idx]))
        {
            return $this->_splitAddress[$idx];
        }
        else
        {
            return '';
        }
    }

    public function getCommunicationDataFromType($type)
    {
        switch ($type)
        {
            case 'purchase':
                return [$this->offsetGet('comm_title_purchase'), $this->offsetGet('comm_text_purchase')];
            case 'purchase_refund':
                return [$this->offsetGet('comm_title_purchase_refund'), $this->offsetGet('comm_text_purchase_refund')];
            case 'renewal':
                return [$this->offsetGet('comm_title_renewal'), $this->offsetGet('comm_text_renewal')];
            case 'renewal_refund':
                return [$this->offsetGet('comm_title_renewal_refund'), $this->offsetGet('comm_text_renewal_refund')];
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_seller_info';
        $structure->shortName = 'XFA\RMMarketplace:Seller';
        $structure->primaryKey = 'user_id';
        $structure->columns = [
            'user_id'                       => ['type' => self::UINT, 'required' => true],
            'activate_invoicing'            => ['type' => self::BOOL, 'default' => false],
            'name'                          => ['type' => self::STR],
            'invoice_prefix'                => ['type' => self::STR],
            'address'                       => ['type' => self::STR],
            'additional_info'               => ['type' => self::STR],
            'logo_date'                     => ['type' => self::UINT, 'default' => 0],
            'comm_type'                     => ['type' => self::STR, 'default' => 'email',
                'allowedValues' => ['email', 'pm', 'both']
            ],
            'comm_title_purchase'           => ['type' => self::STR],
            'comm_text_purchase'            => ['type' => self::STR],
            'comm_title_renewal'            => ['type' => self::STR],
            'comm_text_renewal'             => ['type' => self::STR],
            'comm_title_purchase_refund'    => ['type' => self::STR],
            'comm_text_purchase_refund'     => ['type' => self::STR],
            'comm_title_renewal_refund'     => ['type' => self::STR],
            'comm_text_renewal_refund'      => ['type' => self::STR]
        ];
        $structure->getters = [];
        $structure->relations = [
            'User' => [
                'entity'        => 'XF:User',
                'type'          => self::TO_ONE,
                'conditions'    => 'user_id',
                'primary'       => true
            ]
        ];

        return $structure;
    }
}