<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Transaction extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_transaction';
        $structure->shortName = 'XFA\RMMarketplace:Transaction';
        $structure->primaryKey = 'transaction_id';
        $structure->columns = [
            'transaction_id'    => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'business'          => ['type' => self::STR, 'default' => '', 'maxLength' => 250],
            'txn_id'            => ['type' => self::STR, 'maxLength' => 30],
            'txn_id_pos'        => ['type' => self::STR, 'maxLength' => 30],
            'shipping'          => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'fees'              => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'pos'               => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'token'             => ['type' => self::STR, 'maxLength' => 20],
            'status'            => ['type' => self::STR, 'default' => 'completed',
                'allowedValues' => ['initialized', 'pending', 'completed', 'pending_manual']
            ],
            'pos_paid'          => ['type' => self::STR, 'default' => 'na',
                'allowedValues' => ['na', 'pending', 'completed']
            ],
            'pos_invoice_id'	=> ['type' => self::UINT, 'default' => 0]
        ];
        $structure->getters = [];
        $structure->relations = [
            'Purchase' => [
                'entity'        => 'XFA\RMMarketplace:Purchase',
                'type'          => self::TO_ONE,
                'conditions'    => 'transaction_id'
            ],
            'FeaturingPurchase' => [
                'entity'        => 'XFA\RMMarketplace:FeaturingPurchase',
                'type'          => self::TO_ONE,
                'conditions'    => 'transaction_id'
            ],
        ];

        return $structure;
    }
}