<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2022
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Job;

use XF\Job\AbstractJob;

class DiscountsApplier extends AbstractJob
{
	protected $defaultData = [
		'steps'	=> 0,
		'start'	=> 0,
		'batch' => 100,
	];

	public function run($maxRunTime)
	{
		$db = $this->app->db();
		$em = $this->app->em();

        $continue = false;

		$this->data['steps']++;

		// Get discount
        $discount = \XF::finder('XFA\RMMarketplace:Discount')->whereId($this->data['discount_id'])->fetchOne();

        if ($discount)
        {
            // Get current discount resources ids
            switch ($discount->type)
            {
                case 'all':
                    $resourceIds = $db->fetchAllColumn($db->limit("
                            SELECT resource_id
                            FROM xf_rm_resource
                            WHERE 
                                resource_id > ? AND
                                xfa_rmmp_discount_id = 0 AND
                                xfa_rmmp_type <> 'none'
                            ORDER BY resource_id ASC
                        ", $this->data['batch']
                    ), $this->data['start']);
                    break;
                case 'all_own':
                    $resourceIds = $db->fetchAllColumn($db->limit("
                            SELECT resource_id
                            FROM xf_rm_resource
                            WHERE 
                                resource_id > ? AND
                                user_id = ? AND
                                xfa_rmmp_discount_id = 0 AND
                                xfa_rmmp_type <> 'none'
                            ORDER BY resource_id ASC
                        ", $this->data['batch']
                    ), [$this->data['start'], $discount->creator_id]);
                    break;
                case 'specifics':
                    $resourceIds = $db->fetchAllColumn($db->limit("
                            SELECT resource_id
                            FROM xf_rm_resource
                            WHERE 
                                resource_id IN (" . $db->quote($discount->resource_ids) . ") AND
                                resource_id > ? AND
                                xfa_rmmp_discount_id = 0 AND
                                xfa_rmmp_type <> 'none'
                            ORDER BY resource_id ASC
                        ", $this->data['batch']
                    ), $this->data['start']);
                    break;
            }

            // Get resources and apply discount
            if ($resourceIds)
            {
                $resources = \XF::finder('XFRM:ResourceItem')->whereId($resourceIds);

                $start = microtime(true);

                foreach ($resources AS $resource)
                {
                    $this->data['start'] = $resource->resource_id;

                    $resource->applyDiscount($discount);
                    $resource->save();

                    if ($maxRunTime && microtime(true) - $start > $maxRunTime)
                    {
                        $continue = true;
                        break;
                    }
                }
            }
        }

        // Discount applied
		if (!$continue)
        {
            if ($discount)
            {
                $discount->fastUpdate('applied', 1);
            }

            // Remove discount from list
            array_shift($this->data['discount_ids']);

            // Finished !
            if (count($this->data['discount_ids']) == 0)
            {
                return $this->complete();
            }

            // Prepare for next discount
            $this->data['discount_id']  = current($this->data['discount_ids']);
            $this->data['start']        = 0;
        }

		return $this->resume();
	}

	public function getStatusMessage()
	{
		$actionPhrase = \XF::phrase('xfa_rmmp_applying_discounts');
		return sprintf('%s... %s', $actionPhrase, str_repeat('. ', $this->data['steps']));
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return false;
	}
}