<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Job\Invoices;

use XF\Job\AbstractJob;

class BlockUnpaid extends AbstractJob
{
	protected $defaultData = [
		'steps'	=> 0,
		'start'	=> 0,
		'batch' => 500,
	];

	public function run($maxRunTime)
	{
		$db = $this->app->db();
		$em = $this->app->em();

		$this->data['steps']++;

		$invoiceTimeThreshold = \XF::$time - (\XF::options()->xfa_rmmp_pendingInvoiceBlockSalesDays * 86400);

		$invoices = $db->fetchAllColumn($db->limit("
				SELECT invoice_id
				FROM xf_xfa_rmmp_invoices
				WHERE invoice_date < ? 
				    AND status = 'pending'
				    AND warned = '0'
				    AND invoice_id > ?
				ORDER BY invoice_id ASC
			", $this->data['batch']
		), [$invoiceTimeThreshold, $this->data['start']]);
		
		if (!$invoices)
		{
			return $this->complete();
		}

        $invoices = \XF::finder('XFA\RMMarketplace:MonthlyInvoice')->whereId($invoices)->with('Seller');
		
		$start = microtime(true);

		foreach ($invoices AS $invoice)
		{
			$this->data['start'] = $invoice->invoice_id;
			
			$fromTime = \XF::$time;

            /** @var \XFA\RMMarketplace\Service\PosInvoice\BlockUnpaid $blocker */
            $blocker = \XF::service('XFA\RMMarketplace:PosInvoice\BlockUnpaid', $invoice);
            $blocker->block();
			
            if ($maxRunTime && microtime(true) - $start > $maxRunTime)
            {
                $continue = true;
                break;
            }			
		}

		return $this->resume();
	}

	public function getStatusMessage()
	{
		$actionPhrase = \XF::phrase('xfa_rmmp_creating');
		$typePhrase = \XF::phrase('xfa_rmmp_monhtly_invoices');
		return sprintf('%s... %s %s', $actionPhrase, $typePhrase, str_repeat('. ', $this->data['steps']));
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return false;
	}
}