<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Job\Invoices;

use XF\Job\AbstractJob;

class Generate extends AbstractJob
{
	protected $defaultData = [
		'steps'	=> 0,
		'start'	=> 0,
		'batch' => 500,
	];

	public function run($maxRunTime)
	{
		$db = $this->app->db();
		$em = $this->app->em();

		$this->data['steps']++;

		$sellers = $db->fetchAllColumn($db->limit("
				SELECT user_id
				FROM xf_rm_resource
				WHERE xfa_rmmp_type != 'none' AND user_id > ?
				GROUP BY user_id
				ORDER BY user_id ASC
			", $this->data['batch']
		), $this->data['start']);
		
		if (!$sellers)
		{
			return $this->complete();
		}		
		
		$sellers = \XF::finder('XF:User')->whereId($sellers);
		
		$start = microtime(true);

		foreach ($sellers AS $seller)
		{
			$this->data['start'] = $seller->user_id;
			
			$fromTime = \XF::$time;
			
			// Select the sales to compute the percentages
			$unpaidPos = $db->fetchAll("
				SELECT SUM(pos) AS total_pos,p.currency
				FROM xf_xfa_rmmp_purchase AS p
				LEFT JOIN xf_rm_resource AS r ON (p.resource_id = r.resource_id)
				LEFT JOIN xf_xfa_rmmp_transaction AS t ON (t.transaction_id = p.transaction_id)
				WHERE p.purchase_date < ? AND r.user_id = ? AND p.status = 'validated' AND t.pos_paid = 'pending' AND t.pos_invoice_id = 0
				GROUP BY p.currency
			", [$fromTime, $seller['user_id']]);
		
			// Create the invoices
			foreach ($unpaidPos AS $unpaid)
			{
		        /** @var \XFA\RMMarketplace\Service\PosInvoice\Create $creator */
		        $creator = \XF::service('XFA\RMMarketplace:PosInvoice\Create', $seller);
		        $creator->setAmount($unpaid['total_pos'], $unpaid['currency']);
		        $invoice = $creator->save();
		        
		        $db->query("
		        	UPDATE xf_xfa_rmmp_purchase AS p
					LEFT JOIN xf_rm_resource AS r ON (p.resource_id = r.resource_id)
					LEFT JOIN xf_xfa_rmmp_transaction AS t ON (t.transaction_id = p.transaction_id)
		        	SET t.pos_invoice_id = ?
		        	WHERE p.purchase_date < ? AND r.user_id = ? AND p.currency = ? AND p.status = 'validated' AND t.pos_paid = 'pending' AND t.pos_invoice_id = 0
		        ", [$invoice->invoice_id, $fromTime, $seller->user_id, $unpaid['currency']]);
			}
			
            if ($maxRunTime && microtime(true) - $start > $maxRunTime)
            {
                $continue = true;
                break;
            }			
		}

		return $this->resume();
	}

	public function getStatusMessage()
	{
		$actionPhrase = \XF::phrase('xfa_rmmp_creating');
		$typePhrase = \XF::phrase('xfa_rmmp_monhtly_invoices');
		return sprintf('%s... %s %s', $actionPhrase, $typePhrase, str_repeat('. ', $this->data['steps']));
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return false;
	}
}