<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace;

use XF\Mvc\Entity\Entity;

class Listener
{
    public static function categoryEntityStructure(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
    {
        $structure->columns['xfa_rmmp_conf']                    = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_types']                   = ['type' => Entity::SERIALIZED_ARRAY];
        $structure->columns['xfa_rmmp_payment_data']            = ['type' => Entity::STR,   'default' => ''];
        $structure->columns['xfa_rmmp_sale_percentage']         = ['type' => Entity::FLOAT, 'default' => 0.00];
        $structure->columns['xfa_rmmp_sale_percentage_paypal']  = ['type' => Entity::STR];
        $structure->columns['xfa_rmmp_alt_display']             = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_restrict_support']        = ['type' => Entity::UINT,  'default' => 0];
    }

    public static function resourceItemEntityStructure(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
    {
        $structure->columns['xfa_rmmp_parent_id']               = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_type']                    = ['type' => Entity::STR,   'default' => 'none'];
        $structure->columns['xfa_rmmp_max_qty']                 = ['type' => Entity::UINT,  'default' => 1];
        $structure->columns['xfa_rmmp_paypal']                  = ['type' => Entity::STR,   'default' => ''];
        $structure->columns['xfa_rmmp_accept_other']            = ['type' => Entity::BOOL,  'default' => '0'];
        $structure->columns['xfa_rmmp_amount']                  = ['type' => Entity::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0];
        $structure->columns['xfa_rmmp_base_amount']             = ['type' => Entity::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0];
        $structure->columns['xfa_rmmp_discount_id']             = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_currency']                = ['type' => Entity::STR,   'default' => 'USD'];
        $structure->columns['xfa_rmmp_purchases']               = ['type' => Entity::UINT,  'default' => '0'];
        $structure->columns['xfa_rmmp_img_date']                = ['type' => Entity::UINT,  'default' => '0'];
        $structure->columns['xfa_rmmp_cgv']                     = ['type' => Entity::STR,   'default' => ''];
        $structure->columns['xfa_rmmp_sales_disabled']          = ['type' => Entity::BOOL,  'default' => '0'];

        $structure->relations['PhysicalProduct'] = [
            'entity'        => 'XFA\RMMarketplace:PhysicalProduct',
            'type'          => Entity::TO_ONE,
            'conditions'    => 'resource_id'
        ];

        $structure->relations['DigitalProduct'] = [
            'entity'        => 'XFA\RMMarketplace:DigitalProduct',
            'type'          => Entity::TO_ONE,
            'conditions'    => 'resource_id'
        ];

        $structure->relations['ValidPurchase'] = [
            'entity'        => 'XFA\RMMarketplace:Purchase',
            'type'          => Entity::TO_ONE,
            'conditions'    => [
                ['resource_id', '=', '$resource_id'],
                ['status', '=', 'validated']
            ]
        ];

        $structure->relations['ActiveFeaturing'] = [
            'entity'        => 'XFA\RMMarketplace:FeaturingPurchase',
            'type'          => Entity::TO_ONE,
            'conditions'    => [
                ['resource_id', '=', '$resource_id'],
                ['status', '=', 'validated']
            ]
        ];

        $structure->relations['Seller'] = [
            'entity'        => 'XFA\RMMarketplace:Seller',
            'type'          => Entity::TO_ONE,
            'conditions'    => 'user_id'
        ];

        $structure->relations['Discount'] = [
            'entity'        => 'XFA\RMMarketplace:Discount',
            'type'          => Entity::TO_ONE,
            'conditions'    => [['discount_id', '=', '$xfa_rmmp_discount_id']]
        ];

        $structure->relations['ParentResource'] = [
            'entity'        => 'XFRM:ResourceItem',
            'type'          => Entity::TO_ONE,
            'conditions'    => [['resource_id', '=', '$xfa_rmmp_parent_id']]
        ];

        $structure->relations['ValidParentPurchase'] = [
            'entity'        => 'XFA\RMMarketplace:Purchase',
            'type'          => Entity::TO_ONE,
            'conditions'    => [
                ['resource_id', '=', '$xfa_rmmp_parent_id'],
                ['status', '=', 'validated']
            ]
        ];
    }

    public static function userEntityStructure(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
    {
        $structure->columns['xfa_rmmp_user_purchases']      = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_user_sales']          = ['type' => Entity::UINT,  'default' => 0];
        $structure->columns['xfa_rmmp_user_sales_amount']   = ['type' => Entity::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0];
        $structure->columns['xfa_rmmp_user_cgv']            = ['type' => Entity::STR,   'default' => ''];
        $structure->columns['xfa_rmmp_user_paypal']         = ['type' => Entity::STR,   'default' => ''];
        $structure->columns['xfa_rmmp_user_blocked']        = ['type' => Entity::BOOL,  'default' => 0];

        $structure->relations['SellerInfo'] = [
            'entity'        => 'XFA\RMMarketplace:Seller',
            'type'          => Entity::TO_ONE,
            'conditions'    => 'user_id'
        ];
    }
    
    public static function paymentProfileEntityStructure(\XF\Mvc\Entity\Manager $em, \XF\Mvc\Entity\Structure &$structure)
    {
        $structure->columns['xfa_rmmp_user_id'] = ['type' => Entity::UINT,  'default' => 0];
    }

    public static function templaterSetup(\XF\Container $container, \XF\Template\Templater &$templater)
    {
        $templater->addFunction('xfa_rmmp_resource_image', [__CLASS__, 'templaterFnResourceImage']);
    }

    public static function templaterFnResourceImage(
        $templater, &$escape, \XFRM\Entity\ResourceItem $resource, $href = ''
    )
    {
        $escape = false;

        if ($href)
        {
            $tag = 'a';
            $hrefAttr = 'href="' . htmlspecialchars($href) . '"';
        }
        else
        {
            $tag = 'span';
            $hrefAttr = '';
        }

        if ($resource->xfa_rmmp_img_date == 0)
        {
	        $pather = \XF::app()->container('request.pather');
	        
	        $url = 'styles/default/xfa/rmmarketplace/resource_image.png';
	        $url = $pather($url, 'base');
	        
            return "<{$tag} {$hrefAttr}>"
                . '<img src="' . $url . '" alt="' . htmlspecialchars($resource->title) . '" />'
                . "</{$tag}>";

        }
        else
        {
            $src = $resource->getImageUrl();

            return "<{$tag} {$hrefAttr}>"
                . '<img src="' . htmlspecialchars($src) . '" alt="' . htmlspecialchars($resource->title) . '" />'
                . "</{$tag}>";
        }
    }

    public static function criteriaUser($rule, array $data, \XF\Entity\User $user, &$returnValue)
    {
        switch ($rule)
        {
            case 'xfa_rmmp_purchases':
                if (isset($user->xfa_rmmp_user_purchases) && $user->xfa_rmmp_user_purchases >= $data['purchases'])
                {
                    $returnValue = true;
                }
                break;
            case 'xfa_rmmp_sales':
                if (isset($user->xfa_rmmp_user_sales) && $user->xfa_rmmp_user_sales >= $data['sales'])
                {
                    $returnValue = true;
                }
                break;
            case 'xfa_rmmp_sales_amount':
                if (isset($user->xfa_rmmp_user_sales_amount) && $user->xfa_rmmp_user_sales_amount >= $data['amount'])
                {
                    $returnValue = true;
                }
                break;
        }
    }

    public static function userSearcherOrders(\XF\Searcher\User $userSearcher, array &$sortOrders)
    {
        $sortOrders['xfa_rmmp_user_purchases']      = \XF::phrase('xfa_rmmp_purchases');
        $sortOrders['xfa_rmmp_user_sales']          = \XF::phrase('xfa_rmmp_sales');
        $sortOrders['xfa_rmmp_user_sales_amount']   = \XF::phrase('xfa_rmmp_sales_amount');
    }
    
    public static function templateMacroPreRender(\XF\Template\Templater $templater, &$type, &$template, &$name, array &$arguments, array &$globalVars)
    {
        if (isset($arguments['group']) && $arguments['group']->group_id == 'xfa_rmmp')
        {
            // Override template name
            $template = 'xfa_rmmp_options_macros';
           
            // Or use 'option_form_block_tabs' for tabs
            $name = 'option_form_block_tabs';
           
            // Your block header configurations
            $arguments['headers'] = [
                'generalOptions'      	=> [
                    'label'           => \XF::phrase('general_options'),
                    'minDisplayOrder' => 0,
                    'maxDisplayOrder' => 1000,
                    'active'          => true // Only used for tabs, indicates default active tab
                ],
                'paymentOptions'        => [
                    'label'           => \XF::phrase('xfa_rmmp_payment'),
                    'minDisplayOrder' => 1000,
                    'maxDisplayOrder' => 2000
                ],
                'licensingOptions'   	=> [
                    'label'           => \XF::phrase('xfa_rmmp_licenses'),
                    'minDisplayOrder' => 2000,
                    'maxDisplayOrder' => 3000,
                ],
                'invoicingOptions'    	=> [
                    'label'           => \XF::phrase('xfa_rmmp_invoicing'),
                    'minDisplayOrder' => 3000,
                    'maxDisplayOrder' => 4000
                ],
                'statsOptions'        	=> [
                    'label'           => \XF::phrase('xfa_rmmp_statistics'),
                    'minDisplayOrder' => 4000,
                    'maxDisplayOrder' => 5000
                ],
                'featuringOptions'     => [
                    'label'           => \XF::phrase('xfa_rmmp_resource_featuring'),
                    'minDisplayOrder' => 5000,
                    'maxDisplayOrder' => 6000
                ],
                'prefixesOptions'		=> [
                    'label'           => \XF::phrase('xfa_rmmp_resource_prefixes'),
                    'minDisplayOrder' => 6000,
                    'maxDisplayOrder' => -1 // This allows for any higher display order value
                ],
            ];
        }	    
    }
}