<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Mvc\FormAction;
use XF\Pub\Controller\AbstractController;
use OneSheet\Writer;

class Dashboard extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewResources($error)
            || !($visitor->isASeller()
                    || $visitor->canViewAllSales()
                    || $visitor->canManageAllLicenses()))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    public function actionIndex()
    {
        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if ($visitor->isASeller())
        {
            return $this->actionStats();
        }
        else if ($visitor->canViewAllSales())
        {
            return $this->actionViewSales();
        }
        else
        {
            return $this->actionManageLicenses();
        }
    }

    public function actionConfiguration()
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $visitor->set('xfa_rmmp_user_cgv', $this->plugin('XF:Editor')->fromInput('cgv'));
            $visitor->save();

            if (\XF::options()->xfa_rmmp_activateInvoicing
                    || $visitor->hasPermission('general', 'confMarketplaceCommType')
                    || $visitor->hasPermission('general', 'customizeMarketplaceComm'))
            {
                if (!$visitor->SellerInfo)
                {
                    $sellerInfo = $this->setupSellerInfoCreate($visitor)->save();
                }
                else
                {
                    $sellerInfo = $this->setupSellerInfoEdit($visitor->SellerInfo)->save();
                }
            }

            return $this->redirect($this->buildLink('resources/market-place-dashboard/configuration'));
        }
        else
        {
            return $this->view('', 'xfa_rmmp_dashboard_configuration', []);
        }
    }
    
    public function actionInvoicingPreview()
    {
        $visitor = \XF::visitor();

        if (!\XF::options()->xfa_rmmp_activateInvoicing || !$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }
        
        $seller = \XF::finder('XFA\RMMarketplace:Seller')->where('user_id', \XF::visitor()->user_id)->fetchOne();
        
        if (!$seller)
        {
            throw $this->exception($this->noPermission());
        }
        
        $viewParams = ['seller' => $seller];
        
        $this->setResponseType('raw');

        return $this->view('XFA\RMMarketplace:InvoicePreview', '', $viewParams);        	    
    }

    public function actionProducts()
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        /** @var \XFRM\ControllerPlugin\Overview $overviewPlugin */
        $overviewPlugin = $this->plugin('XFRM:Overview');

        $categoryParams = $overviewPlugin->getCategoryListData();
        $viewableCategoryIds = $categoryParams['categories']->keys();

        $this->request()->set('creator_id', $visitor->user_id);
        $this->request()->set('order', 'xfa_rmmp_purchases');

        $listParams = $overviewPlugin->getCoreListData($viewableCategoryIds);

        $this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'resources');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-dashboard/products', null, ['page' => $listParams['page']]));

        $viewParams = $categoryParams + $listParams;

        return $this->view('', 'xfa_rmmp_dashboard_products', $viewParams);
    }

    public function actionViewSales()
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        if ($visitor->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $viewParams = $purchasePlugin->getListData('all', false);
        }
        else
        {
            $viewParams = $purchasePlugin->getListData('seller', false);
        }

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-dashboard/view-sales');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-dashboard/view-sales', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_dashboard_view_sales', $viewParams);
    }

    public function actionViewSalesFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        return $purchasePlugin->actionFilters('resources/market-place-dashboard/view-sales', true);
    }

    public function actionExportSalesXlsx()
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        if ($visitor->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $purchases = $purchasePlugin->getFullListData('all', false);
        }
        else
        {
            $purchases = $purchasePlugin->getFullListData('seller', false);
        }

        // Get user language
        $language = \XF::language(\XF::visitor()->language_id);

        // Create excel file
        $onesheet = new Writer('/optional/fonts/directory');

        // Enable autosizing of column widths and row heights
        $onesheet->enableCellAutosizing();

        // Create a header style
        $headerStyle = (new \OneSheet\Style\Style())
            ->setFontSize(13)
            ->setFontBold()
            ->setFontColor('FFFFFF')
            ->setFillColor('777777');

        // Header
        $onesheet->addRow([
            '#',
            \XF::phrase('xfa_rmmp_purchased_on')->render(),
            \XF::phrase('xfrm_resource')->render(),
            \XF::phrase('xfa_rmmp_buyer')->render(),
            \XF::phrase('xfa_rmmp_type')->render(),
            \XF::phrase('xfa_rmmp_quantity')->render(),
            \XF::phrase('xfa_rmmp_payment')->render(),
            \XF::phrase('xfa_rmmp_price')->render(),
            \XF::phrase('xfa_rmmp_shipment')->render()
        ], $headerStyle);

        // Freeze everything above cell A2 (the first row will be frozen)
        $onesheet->setFreezePaneCellId('A2');

        // Write data
        $phrases = [
            'purchase'          => \XF::phrase('xfa_rmmp_purchase')->render(),
            'renewal'           => \XF::phrase('xfa_rmmp_renewal')->render(),
            'payment_pending'   => \XF::phrase('xfa_rmmp_payment_pending')->render(),
            'unpaid'            => \XF::phrase('xfa_rmmp_unpaid')->render(),
            'validated'         => \XF::phrase('xfa_rmmp_validated')->render(),
            'refunded'          => \XF::phrase('xfa_rmmp_refunded')->render(),
            'inc'               => \XF::phrase('xfa_rmmp_inc')->render(),
            'shipping'          => \XF::phrase('xfa_rmmp_shipping')->render(),
            'pending'           => \XF::phrase('xfa_rmmp_pending')->render(),
            'in_preparation'    => \XF::phrase('xfa_rmmp_in_preparation')->render(),
            'sent'              => \XF::phrase('xfa_rmmp_sent')->render(),
            'delivered'         => \XF::phrase('xfa_rmmp_delivered')->render()
        ];

        foreach ($purchases AS $purchase)
        {
            // Build up status
            if ($purchase->status == 'pending')
            {
                if ($purchase->Transaction->status == 'pending')
                {
                    $paymentStatus = $phrases['payment_pending'];
                }
                else
                {
                    $paymentStatus = $phrases['unpaid'];
                }
            }
            else if ($purchase->status == 'validated')
            {
                $paymentStatus = $phrases['validated'];
            }
            else
            {
                $paymentStatus = $phrases['refunded'];
            }

            // Build up shipping
            if (isset($purchase->Transaction)
                    && $purchase->Transaction
                    && $purchase->Transaction->shipping > 0.0)
            {
                $shipping = $phrases['inc'] . ' ' . $purchase->Transaction->shipping . $purchase->currency . ' ' . $phrases['shipping'];
            }
            else
            {
                $shipping       = '';
            }

            // Build up shipment status
            if ($purchase->Resource->xfa_rmmp_type == 'physical')
            {
                switch($purchase->shipment_status)
                {
                    case 'pending':         $shipmentStatus = $phrases['pending']; break;
                    case 'in_preparation':  $shipmentStatus = $phrases['in_preparation']; break;
                    case 'sent':            $shipmentStatus = $phrases['sent']; break;
                    default:                $shipmentStatus = $phrases['delivered']; break;
                }
            }
            else
            {
                $shipmentStatus = 'N/A';
            }

            // Add row
            $onesheet->addRow([
                $purchase->purchase_id,
                $purchase->Buyer ? $purchase->Buyer->username : $purchase->username,
                $purchase->Resource->title,
                $language->date($purchase->purchase_date),
                $purchase->type == 'purchase' ? $phrases['purchase'] : $phrases['renewal'],
                $purchase->quantity,
                $paymentStatus,
                $purchase->price . $purchase->currency . $shipping,
                $shipmentStatus
            ]);
        }

        $onesheet->writeToBrowser(\XF::phrase('xfa_rmmp_purchases_log')->render() . ' ' . $language->date(\XF::$time) . '.xlsx');

        $this->setResponseType('raw');
        return $this->view('', '');
    }

    public function actionExportSalesCsv()
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        if ($visitor->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $purchases = $purchasePlugin->getFullListData('all', false);
        }
        else
        {
            $purchases = $purchasePlugin->getFullListData('seller', false);
        }

        $this->setResponseType('raw');
        return $this->view('XFA\RMMarketplace:ExportSalesCsv', '', ['purchases' => $purchases]);
    }

    public function actionManageLicenses()
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\ControllerPlugin\License $licensePlugin */
        $licensePlugin = $this->plugin('XFA\RMMarketplace:License');

        if ($visitor->canManageAllLicenses() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $viewParams = $licensePlugin->getListData('all', false);
        }
        else
        {
            $viewParams = $licensePlugin->getListData('seller', false);
        }

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-dashboard/manage-licenses');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-dashboard/manage-licenses', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_dashboard_manage_licenses', $viewParams);
    }

    public function actionManageLicensesFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\License $licensePlugin */
        $licensePlugin = $this->plugin('XFA\RMMarketplace:License');

        if (\XF::visitor()->canManageAllLicenses() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            return $licensePlugin->actionFilters('all', 'resources/market-place-dashboard/manage-licenses', true);
        }
        else
        {
            return $licensePlugin->actionFilters('own', 'resources/market-place-dashboard/manage-licenses', true);
        }
    }

    public function actionAddLicense()
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller() && !$visitor->canManageAllLicenses())
        {
            throw $this->exception($this->noPermission());
        }

        $redirect = $this->filter('redirect', 'str', 'dashboard');

        if ($this->isPost())
        {
            /* Check provided user exists */
            $user = $this->em()->findOne('XF:User', ['username' => $this->filter('username', 'str')]);
            if (!$user)
            {
                throw $this->exception($this->notFound(\XF::phrase('requested_user_not_found')));
            }

            /* Cleanup data */
            $inputData = [
                'start_date'    => $this->filter('start_date', 'datetime'),
                'end_date'      => $this->filter('end_date', 'datetime'),
            ];

            /* Get resource and check user is owner or can manage all licenses */
            /** @var \XFRM\Entity\ResourceItem $resource */
            $resource = $this->em()->find('XFRM:ResourceItem', $this->filter('resource_id', 'uint'));
            if (!$resource)
            {
                throw $this->exception($this->notFound(\XF::phrase('xfrm_requested_resource_not_found')));
            }

            if ($resource->user_id != $visitor->user_id && !$visitor->canManageAllLicenses())
            {
                throw $this->exception($this->noPermission());
            }

            $license = $this->setupLicenseCreate(
                $resource,
                $user,
                $inputData['start_date'],
                $inputData['end_date']
            )->save();

            $validator = $this->service('XFA\RMMarketplace:License\Activate', $license);
            $validator->activate();

            $purchase = $this->setupPurchaseWithoutTransactionCreate(
                $resource,
                $user,
                $license
            )->save();

            if ($redirect == 'dashboard')
            {
                return $this->redirect($this->buildLink('resources/market-place-dashboard/manage-licenses'));
            }
            else
            {
                return $this->redirect($this->buildLink('resources', $resource));
            }
        }
        else
        {
            /* Need to get the resources user can manage */
            /** @var \XFRM\Finder\ResourceItem $resourcesFinder */
            $resourcesFinder = $this->finder('XFRM:ResourceItem');
            $resourcesFinder->with('DigitalProduct', true);

            if (!$visitor->canManageAllLicenses())
            {
                $resourcesFinder->where('user_id', \XF::visitor()->user_id);
            }

            $resources = $resourcesFinder->fetch()->filterViewable();

            $resourcesOptions = [];
            foreach ($resources AS $resource)
            {
                $resourcesOptions[$resource->resource_id] = $resource->title . ($resource->DigitalProduct->duration == 0 ? ' - ' . \XF::phrase('xfa_rmmp_unlimited') : '');
            }

            $viewParams = [
                'resourcesOptions'  => $resourcesOptions,
                'redirect'          => $redirect
            ];
            return $this->view('', 'xfa_rmmp_license_add', $viewParams);
        }
    }

    public function actionStats()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Stats $statsPlugin */
        $statsPlugin = $this->plugin('XFA\RMMarketplace:Stats');

        if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $viewParams = $statsPlugin->getListData('all', false);
        }
        else
        {
            $viewParams = $statsPlugin->getListData('seller', false);
        }

        return $this->view('', 'xfa_rmmp_dashboard_stats', $viewParams);
    }

    public function actionStatsFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Stats $statsPlugin */
        $statsPlugin = $this->plugin('XFA\RMMarketplace:Stats');

        if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            return $statsPlugin->actionFilters('all');
        }
        else
        {
            return $statsPlugin->actionFilters('seller');
        }
    }
    
    public function actionPaymentData()
    {
        $visitor = \XF::visitor();
        $options = \XF::options();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }
        
		// Get authorized payment providers and user payment profiles info if in XF mode
        $providers  = null;
        $profiles   = null;
		if ($options->xfa_rmmp_mode == 'xf')
		{
			// Get providers
			$providers = $this->repository('XF:Payment')->findActivePaymentProviders()->whereId($options->xfa_rmmp_paymentProviderIds)->fetch();
			
			// Get user profiles
			$profiles = $this->finder('XF:PaymentProfile')->where('xfa_rmmp_user_id', \XF::visitor()->user_id)->keyedBy('provider_id')->fetch();
		} 

        if ($this->isPost())
        {
	        if ($options->xfa_rmmp_mode == 'custom')
	        {
	            $visitor->set('xfa_rmmp_user_paypal', $this->filter('paypal', 'str'));
	            $visitor->save();
	        }
	        else
	        {
		        $configuration = $this->filter('configuration', 'array-array-str');		       

				foreach ($providers AS $provider)
				{
					// Create or edit provider configuration
					if (isset($configuration[$provider->provider_id]['active']) && $configuration[$provider->provider_id]['active'])
					{
						if (isset($profiles[$provider->provider_id]))
						{
							$profile = $profiles[$provider->provider_id];
							$profile->active = 1;
						}
						else
						{
							$profile = $this->em()->create('XF:PaymentProfile');
				
							$profile->title = $provider->getTitle() . ' - ' . \XF::visitor()->username;
							$profile->xfa_rmmp_user_id = \XF::visitor()->user_id;
							$profile->active = 1;
							$profile->provider_id = $provider->provider_id;
						}				
						
						$this->userPaymentProfileSaveProcess($profile, $provider, $configuration[$provider->provider_id])->run();	
					}
					// Disable configuration if existing
					else if (isset($profiles[$provider->provider_id]))
					{
						$profiles[$provider->provider_id]->active = 0;
						$profiles[$provider->provider_id]->save();
					}
				}
	        }

            return $this->redirect($this->buildLink('resources/market-place-dashboard/payment-data'));
        }
        else
        {
	        $viewParams = [
	        	'providers' => $providers,
	        	'profiles'	=> $profiles
	        ];
	        
            return $this->view('', 'xfa_rmmp_dashboard_payment_data', $viewParams);
        }	    
    }

	protected function userPaymentProfileSaveProcess(\XF\Entity\PaymentProfile $profile, \XF\Entity\PaymentProvider $provider, $configuration)
	{
		$form = $this->formAction();

		$input = $this->filter('configuration[' . $provider->provider_id . ']', 'array-str');

		$form->validate(function(FormAction $form) use ($profile, $provider, $configuration)
		{
			$provider->getHandler()->verifyConfig($configuration, $errors);
			if ($errors)
			{
				$form->logErrors($errors);
			}
			
			$profile->options = $configuration;
		});

		$form->basicEntitySave($profile, []);

		return $form;
	}

    protected function setupSellerInfoCreate(\XF\Entity\User $user)
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\Service\Seller\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Seller\Create', $user);

        if (\XF::options()->xfa_rmmp_activateInvoicing)
        {
            $input = $this->filter([
                'activate_invoicing'    => 'bool',
                'invoice_prefix'        => 'str',
                'name'                  => 'str',
                'address'               => ['str','str','str','str'],
                'additional_info'       => 'str'
            ]);

            $input['address'] = implode('\n', $input['address']);

            $creator->configureInvoicing($input['activate_invoicing'], $input['invoice_prefix']);
            $creator->setSellerData($input['name'], $input['address'], $input['additional_info']);

            $logo = $this->request->getFile('logo', false, false);
            if ($logo)
            {
                $creator->addLogo($logo->getFileWrapper());
            }
        }

        if ($visitor->hasPermission('general', 'confMarketplaceCommType'))
        {
            $creator->setCommunicationType($this->filter('comm_type', 'str'));
        }

        if ($visitor->hasPermission('general', 'customizeMarketplaceComm'))
        {
            $communicationData = [
                'comm_title_purchase'           => $this->filter('comm_title_purchase', 'str'),
                'comm_text_purchase'            => $this->plugin('XF:Editor')->fromInput('comm_text_purchase'),
                'comm_title_purchase_refund'    => $this->filter('comm_title_purchase_refund', 'str'),
                'comm_text_purchase_refund'     => $this->plugin('XF:Editor')->fromInput('comm_text_purchase_refund'),
                'comm_title_renewal'            => $this->filter('comm_title_renewal', 'str'),
                'comm_text_renewal'             => $this->plugin('XF:Editor')->fromInput('comm_text_renewal'),
                'comm_title_renewal_refund'     => $this->filter('comm_title_renewal_refund', 'str'),
                'comm_text_renewal_refund'      =>  $this->plugin('XF:Editor')->fromInput('comm_text_renewal_refund')
            ];
            
            $creator->setCommunicationData($communicationData);
        }

        return $creator;
    }

    protected function setupSellerInfoEdit(\XFA\RMMarketplace\Entity\Seller $seller)
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\Service\Seller\Edit $editor */
        $editor = $this->service('XFA\RMMarketplace:Seller\Edit', $seller);

        if (\XF::options()->xfa_rmmp_activateInvoicing)
        {
            $input = $this->filter([
                'activate_invoicing' => 'bool',
                'invoice_prefix' => 'str',
                'name' => 'str',
                'address' => ['str', 'str', 'str', 'str'],
                'additional_info' => 'str',
                'delete' => 'bool'
            ]);

            $input['address'] = implode('\n', $input['address']);

            $editor->configureInvoicing($input['activate_invoicing'], $input['invoice_prefix']);
            $editor->setSellerData($input['name'], $input['address'], $input['additional_info']);

            if ($input['delete'])
            {
                $editor->deleteLogo();
            }

            $logo = $this->request->getFile('logo', false, false);
            if ($logo)
            {
                $editor->addLogo($logo->getFileWrapper());
            }
        }

        if ($visitor->hasPermission('general', 'confMarketplaceCommType'))
        {
            $editor->setCommunicationType($this->filter('comm_type', 'str'));
        }

        if ($visitor->hasPermission('general', 'customizeMarketplaceComm'))
        {
            $communicationData = [
                'comm_title_purchase'           => $this->filter('comm_title_purchase', 'str'),
                'comm_text_purchase'            => $this->plugin('XF:Editor')->fromInput('comm_text_purchase'),
                'comm_title_purchase_refund'    => $this->filter('comm_title_purchase_refund', 'str'),
                'comm_text_purchase_refund'     => $this->plugin('XF:Editor')->fromInput('comm_text_purchase_refund'),
                'comm_title_renewal'            => $this->filter('comm_title_renewal', 'str'),
                'comm_text_renewal'             => $this->plugin('XF:Editor')->fromInput('comm_text_renewal'),
                'comm_title_renewal_refund'     => $this->filter('comm_title_renewal_refund', 'str'),
                'comm_text_renewal_refund'      =>  $this->plugin('XF:Editor')->fromInput('comm_text_renewal_refund')
            ];

            $editor->setCommunicationData($communicationData);
        }


        return $editor;
    }

    protected function setupLicenseCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XF\Entity\User $user,
        $startDate, $endDate
    )
    {
        /** @var \XFA\RMMarketplace\Service\License\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:License\Create', $resource);

        $creator->setOwner($user);
        $creator->setStartDate($startDate);
        $creator->setEndDate($endDate);

        return $creator;
    }

    protected function setupPurchaseWithoutTransactionCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XF\Entity\User $user,
        \XFA\RMMarketplace\Entity\License $license
    )
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\CreateWithoutTransaction $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\CreateWithoutTransaction', $resource, $license);

        $creator->setPurchaser($user);
        $creator->setPrice(0);
        $creator->setValidated();

        return $creator;
    }

    public function setupDiscountAdd()
    {
        $discount = $this->em()->create('XFA\RMMarketplace:Discount');

        $form = $this->formAction();

        $input = $this->filter([
            'name'              => 'str',
            'type'              => 'str',
            'discount_amount'   => 'float',
            'from_date'         => 'datetime',
            'to_date'           => 'datetime',
            'applyto_renewal'   => 'bool'
        ]);

        // Fixed ?
        if ($this->filter('discount_type', 'str') == 'amount')
        {
            $input['is_fixed'] = 1;
        }
        else
        {
            $input['is_fixed'] = 0;
        }

        // Check end date is above start date
        if ($input['to_date'] < $input['from_date'])
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_must_be_after_start_date')));
        }

        // Check end date is not in the past
        if ($input['to_date'] < \XF::$time)
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_cant_be_in_past')));
        }

        // If all, check permissions are ok
        if ($input['type'] == 'all' && (!\XF::visitor()->canViewAllSales()
                || $this->request()->getCookie('xfa_rmmp_dashboard_conf') != 'all'))
        {
            throw $this->exception($this->noPermission());
        }

        // Validate discount
        switch($input['type'])
        {
            case 'all':
            case 'all_own':
                // Check there are resources without discount applied
                $resourceFinder = $this->finder('XFRM:ResourceItem')
                    ->where('xfa_rmmp_type', '<>', 'none')
                    ->where('xfa_rmmp_discount_id', '=', 0);

                if ($input['type'] == 'all_own')
                {
                    $resourceFinder->where('user_id', '=', \XF::visitor()->user_id);
                }

                if ($resourceFinder->total() == 0)
                {
                    throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_no_valid_resource_found_for_discount')));
                }

                break;
            case 'specifics':
                $resourceNames = $this->filter('resources', 'str');
                $resourceNames = preg_split('#\s*,\s*#', $resourceNames, -1, PREG_SPLIT_NO_EMPTY);

                /** @var \XFRM\Repository\ResourceItem $resourceRepo */
                $resourceRepo = $this->repository('XFRM:ResourceItem');
                $resources    = $resourceRepo->getNonRMDiscountedResourcesByName($resourceNames, $notFound)->toArray();

                if ($notFound)
                {
                    throw $this->exception(
                        $this->noPermission(
                            \XF::phrase(
                                'xfa_rmmp_the_following_resources_x_could_not_be_found_or_are_already_discounted',
                                ['names' => implode(', ', $notFound)]
                            )
                        )
                    );
                }

                $input['resource_ids'] = array_keys($resources);

                break;
        }

        $input['creator_id'] = \XF::visitor()->user_id;

        $form->basicEntitySave($discount, $input);

        return $form;
    }

    public function setupDiscountEdit($discount)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'name'              => 'str',
            'from_date'         => 'datetime',
            'to_date'           => 'datetime'
        ]);

        // Remove start date if current was in the past
        if ($discount->from_date < \XF::$time)
        {
            unset($input['from_date']);
        }
        else
        {
            // Check end date is above start date
            if ($input['to_date'] < $input['from_date'])
            {
                throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_must_be_after_start_date')));
            }
        }

        // Check end date is in the past
        if ($input['to_date'] < \XF::$time)
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_cant_be_in_past')));
        }

        $form->basicEntitySave($discount, $input);

        return $form;
    }
}