<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use Doctrine\ORM\Query\Parameter;
use XF\Mvc\ParameterBag;
use XF\Mvc\FormAction;
use XF\Pub\Controller\AbstractController;
use OneSheet\Writer;

class DashboardCoupon extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewResources($error)
            || !($visitor->isASeller()
                    || $visitor->canViewAllSales()
                    || $visitor->canManageAllLicenses()))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    public function actionIndex()
    {
        $visitor = \XF::visitor();
        $options = \XF::options();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        /** @var \XFA\RMMarketplace\ControllerPlugin\Coupon $couponPlugin */
        $couponPlugin = $this->plugin('XFA\RMMarketplace:Coupon');

        if ($visitor->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $viewParams = $couponPlugin->getListData('all', false);
        }
        else
        {
            $viewParams = $couponPlugin->getListData('seller', false);
        }

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-dashboard/coupon');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-dashboard/coupon', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_dashboard_coupons', $viewParams);
    }

    public function actionFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Coupon $couponPlugin */
        $couponPlugin = $this->plugin('XFA\RMMarketplace:Coupon');

        if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            return $couponPlugin->actionFilters('all');
        }
        else
        {
            return $couponPlugin->actionFilters('seller');
        }
    }

    public function actionAdd()
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $this->setupAddEdit($this->em()->create('XFA\RMMarketplace:Coupon'))->run();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/coupon'));
        }
        else
        {
            $viewParams = [
                'coupon'        => $this->em()->create('XFA\RMMarketplace:Coupon'),
                'usergroups'    => $this->em()->getRepository('XF:UserGroup')->getUserGroupTitlePairs()
            ];

            if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
            {
                $viewParams['hasAllRights'] = true;
            }
            else
            {
                $viewParams['hasAllRights'] = false;
            }

            return $this->view('', 'xfa_rmmp_dashboard_coupon_add', $viewParams);
        }
    }

    public function actionEdit(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $coupon = $this->finder('XFA\RMMarketplace:Coupon')->whereId($params->coupon_id)->fetchOne();
        if (!$coupon)
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $this->setupAddEdit($coupon)->run();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/coupon'));
        }
        else
        {
            // Get specific resources if any
            $resources = $this->finder('XFRM:ResourceItem')->whereId($coupon->resource_ids)->fetch();

            $specificResources  = "";
            if ($resources)
            {
                $resourcesList = [];
                foreach ($resources AS $resource)
                {
                    $resourcesList[] = $resource->title;
                }

                $specificResources = implode(',', $resourcesList);
            }

            $viewParams = [
                'coupon'            => $coupon,
                'usergroups'        => $this->em()->getRepository('XF:UserGroup')->getUserGroupTitlePairs(),
                'specificResources' => $specificResources
            ];

            if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
            {
                $viewParams['hasAllRights'] = true;
            }
            else
            {
                $viewParams['hasAllRights'] = false;
            }

            return $this->view('', 'xfa_rmmp_dashboard_coupon_add', $viewParams);
        }
    }

    public function actionDelete(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $coupon = $this->finder('XFA\RMMarketplace:Coupon')->whereId($params->coupon_id)->fetchOne();
        if (!$coupon)
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $coupon->delete();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/coupon'));
        }
        else
        {
            $viewParams = [
                'coupon' => $coupon
            ];

            return $this->view('', 'xfa_rmmp_dashboard_coupon_delete', $viewParams);
        }
    }

    public function actionGetResources()
    {
        $q = ltrim($this->filter('q', 'str', ['no-trim']));

        if ($q !== '' && utf8_strlen($q) >= 2)
        {
            /** @var \XFRM\Finder\ResourceItem $resourceFinder */
            $resourceFinder = $this->finder('XFRM:ResourceItem');
            $resourceFinder->where('title', 'like', $resourceFinder->escapeLike($q, '?%'));
            $resourceFinder->where('xfa_rmmp_type', '<>', 'none');

            if (!\XF::visitor()->canViewAllSales() || $this->request()->getCookie('xfa_rmmp_dashboard_conf') != 'all')
            {
                $resourceFinder->where('user_id', '=', \XF::visitor()->user_id);
            }

            $resources = $resourceFinder->fetch();
        }
        else
        {
            $resources    = [];
            $q          = '';
        }

        $viewParams = [
            'q'         => $q,
            'resources' => $resources
        ];

        return $this->view('XFA\RMMarketplace:GetDiscountResources', '', $viewParams);
    }

    public function actionResources(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $coupon = $this->finder('XFA\RMMarketplace:Coupon')->whereId($params->coupon_id)->fetchOne();
        if (!$coupon || $coupon->type != 'specifics')
        {
            throw $this->exception($this->noPermission());
        }

        $resources = $this->finder('XFRM:ResourceItem')->where('resource_id', '=', $coupon->resource_ids)->fetch();

        $viewParams = [
            'resources' => $resources
        ];

        return $this->view('', 'xfa_rmmp_dashboard_coupon_resources', $viewParams);
    }

    public function actionUsers(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $coupon = $this->finder('XFA\RMMarketplace:Coupon')->whereId($params->coupon_id)->fetchOne();
        if (!$coupon || !$coupon->user_ids)
        {
            throw $this->exception($this->noPermission());
        }

        $users = $this->finder('XF:User')->where('user_id', '=', $coupon->user_ids)->fetch();

        $viewParams = [
            'users' => $users
        ];

        return $this->view('', 'xfa_rmmp_dashboard_coupon_users', $viewParams);
    }

    public function actionUsages(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $coupon = $this->finder('XFA\RMMarketplace:Coupon')->whereId($params->coupon_id)->fetchOne();
        if (!$coupon || $coupon->total_usages == 0)
        {
            throw $this->exception($this->noPermission());
        }

        $usages = $this->finder('XFA\RMMarketplace:CouponUsage')
                       ->where('coupon_id', '=', $coupon->coupon_id)
                       ->with(['Purchase','Purchase.Resource'])->fetch();

        $viewParams = [
            'usages' => $usages
        ];

        return $this->view('', 'xfa_rmmp_dashboard_coupon_usages', $viewParams);
    }

    public function setupAddEdit($coupon)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'name'              => 'str',
            'code'              => 'str',
            'coupon_type'       => 'str',
            'apply_to'          => 'str',
            'type'              => 'str',
            'from_date'         => 'datetime',
            'to_date'           => 'datetime',
            'authorized_usages' => 'uint',
            'usergroups'        => 'array-uint'
        ]);

        // Check if coupon code is not already in use
        if ($coupon->isInsert())
        {
            $existing = $this->finder('XFA\RMMarketplace:Coupon')
                            ->where('code', $input['code'])
                            ->where('creator_id', \XF::visitor()->user_id);
        }
        else
        {
            $existing = $this->finder('XFA\RMMarketplace:Coupon')
                            ->where('code', $input['code'])
                            ->where('coupon_id', '<>', $coupon->coupon_id)
                            ->where('creator_id', \XF::visitor()->user_id);
        }

        if ($existing->total())
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_coupon_code_already_exists')));
        }

        // Check end date is above start date
        if ($input['to_date'] < $input['from_date'])
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_must_be_after_start_date')));
        }

        // Check end date is not in the past
        if ($input['to_date'] < \XF::$time)
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_cant_be_in_past')));
        }

        // If all, check permissions are ok
        if ($input['type'] == 'all' && (!\XF::visitor()->canViewAllSales()
                || $this->request()->getCookie('xfa_rmmp_dashboard_conf') != 'all'))
        {
            throw $this->exception($this->noPermission());
        }

        // Get amount depending on coupon type
        switch($input['coupon_type'])
        {
            case 'percentage':
                $input['coupon_amount'] = $this->filter('percentage_amount', 'float');
                break;
            case 'discount':
                $input['coupon_amount'] = $this->filter('fixed_amount', 'float');
                break;
            default:
                $input['coupon_amount'] = 0;
                break;
        }

        // If coupon applies to specific resources, get their ids
        if ($input['type'] == 'specifics')
        {
            $resourceNames = $this->filter('resources', 'str');
            $resourceNames = preg_split('#\s*,\s*#', $resourceNames, -1, PREG_SPLIT_NO_EMPTY);

            $resources     = $this->finder('XFRM:ResourceItem')->where('title', $resourceNames)->fetch()->toArray();

            $input['resource_ids'] = array_keys($resources);
        }

        $input['creator_id'] = \XF::visitor()->user_id;

        // Handle usernames
        $userNames = $this->filter('usernames', 'str');
        $userNames = preg_split('#\s*,\s*#', $userNames, -1, PREG_SPLIT_NO_EMPTY);

        $users     = $this->finder('XF:User')->where('username', $userNames)->fetch()->toArray();


        $input['user_ids'] = array_keys($users);

        $form->basicEntitySave($coupon, $input);

        return $form;
    }
}