<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2022
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\View;

class ExportSalesCsv extends \XF\Mvc\View
{
    public function renderRaw()
    {
        if (!headers_sent() && function_exists('header_remove'))
        {
            header_remove('Expires');
            header('Cache-control: private');
        }

        // Get user language and set content type/export file name
        $language = \XF::language(\XF::visitor()->language_id);

        $this->response
            ->contentType('text/csv', 'utf-8')
            ->setDownloadFileName(\XF::phrase('xfa_rmmp_purchases_log')->render() . ' ' . $language->date(\XF::$time) . '.csv');

        $fp = fopen('php://memory', 'r+');

        fputcsv($fp, [
            '#',
            \XF::phrase('xfa_rmmp_purchased_on')->render(),
            \XF::phrase('xfrm_resource')->render(),
            \XF::phrase('xfa_rmmp_buyer')->render(),
            \XF::phrase('xfa_rmmp_type')->render(),
            \XF::phrase('xfa_rmmp_quantity')->render(),
            \XF::phrase('xfa_rmmp_payment')->render(),
            \XF::phrase('xfa_rmmp_price')->render(),
            \XF::phrase('xfa_rmmp_shipment')->render()
        ]);

        // Write data
        $phrases = [
            'purchase'          => \XF::phrase('xfa_rmmp_purchase')->render(),
            'renewal'           => \XF::phrase('xfa_rmmp_renewal')->render(),
            'payment_pending'   => \XF::phrase('xfa_rmmp_payment_pending')->render(),
            'unpaid'            => \XF::phrase('xfa_rmmp_unpaid')->render(),
            'validated'         => \XF::phrase('xfa_rmmp_validated')->render(),
            'refunded'          => \XF::phrase('xfa_rmmp_refunded')->render(),
            'inc'               => \XF::phrase('xfa_rmmp_inc')->render(),
            'shipping'          => \XF::phrase('xfa_rmmp_shipping')->render(),
            'pending'           => \XF::phrase('xfa_rmmp_pending')->render(),
            'in_preparation'    => \XF::phrase('xfa_rmmp_in_preparation')->render(),
            'sent'              => \XF::phrase('xfa_rmmp_sent')->render(),
            'delivered'         => \XF::phrase('xfa_rmmp_delivered')->render()
        ];

        $purchases = $this->getParams()['purchases'];
        foreach ($purchases AS $purchase)
        {
            // Build up status
            if ($purchase->status == 'pending')
            {
                if ($purchase->Transaction->status == 'pending')
                {
                    $paymentStatus = $phrases['payment_pending'];
                }
                else
                {
                    $paymentStatus = $phrases['unpaid'];
                }
            }
            else if ($purchase->status == 'validated')
            {
                $paymentStatus = $phrases['validated'];
            }
            else
            {
                $paymentStatus = $phrases['refunded'];
            }

            // Build up shipping
            if (isset($purchase->Transaction)
                && $purchase->Transaction
                && $purchase->Transaction->shipping > 0.0)
            {
                $shipping = $phrases['inc'] . ' ' . $purchase->Transaction->shipping . $purchase->currency . ' ' . $phrases['shipping'];
            }
            else
            {
                $shipping       = '';
            }

            // Build up shipment status
            if ($purchase->Resource->xfa_rmmp_type == 'physical')
            {
                switch($purchase->shipment_status)
                {
                    case 'pending':         $shipmentStatus = $phrases['pending']; break;
                    case 'in_preparation':  $shipmentStatus = $phrases['in_preparation']; break;
                    case 'sent':            $shipmentStatus = $phrases['sent']; break;
                    default:                $shipmentStatus = $phrases['delivered']; break;
                }
            }
            else
            {
                $shipmentStatus = 'N/A';
            }

            // Add row
            fputcsv($fp, [
                $purchase->purchase_id,
                $purchase->Buyer ? $purchase->Buyer->username : $purchase->username,
                $purchase->Resource->title,
                $language->date($purchase->purchase_date),
                $purchase->type == 'purchase' ? $phrases['purchase'] : $phrases['renewal'],
                $purchase->quantity,
                $paymentStatus,
                $purchase->price . $purchase->currency . $shipping,
                $shipmentStatus
            ]);
        }

        rewind($fp);
        $csv = stream_get_contents($fp);
        fclose($fp);

        return $csv;
    }
}