<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\View;

use XFA\RMMarketplace\vendor\pdfinvoice\src\InvoicePrinter;

class Invoice extends \XF\Mvc\View
{
    public function renderRaw()
    {
        if (!headers_sent() && function_exists('header_remove'))
        {
            header_remove('Expires');
            header('Cache-control: private');
        }

        $this->response->header('Content-type', 'application/octet-stream', true);

        /* Get view data */
        $seller     = $this->params['purchase']->Resource->Seller;
        $purchase   = $this->params['purchase'];
        $options    = \XF::options();
        $visitor    = \XF::visitor();

        /* Generate invoice */
        $invoice = new InvoicePrinter('A4', $this->getCurrencySymbol(null, $purchase->currency), $options->xfa_rmmp_invoicesLanguage);

        /* Header Settings */
        if (!empty($seller->hasLogo()))
        {
            $invoice->setLogo($seller->getLogoUrl(false));
        }
        $invoice->setColor("#007fff");
        $invoice->setType("Sale Invoice");
        $invoice->setReference($seller->invoice_prefix . sprintf("%'.010d", $purchase->purchase_id));
        $invoice->setDate(date($options->xfa_rmmp_invoicesDateFormat, $purchase->validation_date));
        $invoice->setTime(date($options->xfa_rmmp_invoicesTimeFormat, $purchase->validation_date));
        if ($purchase->status == 'validated')
        {
            $invoice->setDue(date($options->xfa_rmmp_invoicesDateFormat, $purchase->validation_date));
        }
        else
        {
            $invoice->setDue(date($options->xfa_rmmp_invoicesDateFormat, strtotime('+1 months')));
        }

        $from = array($seller->name);
        if ($seller->address)
        {
            $fromAddress = explode('\n', $seller->address);
            $from = array_merge($from, $fromAddress);
        }

        if (count($from) < 5)
        {
            for ($i = count($from); $i < 5; $i++)
            {
                $from[] = "";
            }
        }

        $invoice->setFrom($from);

        if ($purchase->PurchaseInfo)
        {
            $address = preg_split('/\R/', $purchase->PurchaseInfo->address, -1, PREG_SPLIT_NO_EMPTY);

            if (count($address) < 4)
            {
                for ($i = count($address); $i < 4; $i++)
                {
                    $address[] = "";
                }
            }

            $invoice->setTo(array(
                $purchase->PurchaseInfo->name,
                $address[0],
                $address[1],
                $address[2],
                $address[3]
            ));
        }
        else
        {
            $invoice->setTo(array($visitor['email'],"","","",""));
        }

        /* Adding Items in table */
        if ($purchase->type == 'purchase')
        {
            $invoice->addItem($purchase->Resource->title, $purchase->Resource->tag_line, 1 , false, $purchase->price, false, $purchase->price);
        }
        else
        {
            $renewalPhrase = \XF::phrase('xfa_rmmp_renewal');
            $invoice->addItem($renewalPhrase->render() . ': ' . $purchase->Resource->title, $purchase->Resource->tag_line, 1 , false, $purchase->price, false, $purchase->price);
        }

        /* Add totals */
        $totalDuePhrase = \XF::phrase('xfa_rmmp_total_due');
        $invoice->addTotal($totalDuePhrase->render(), $purchase->price, true);

        /* Set badge */
        if ($purchase->status == 'validated')
        {
            $paymentStatusPhrase = \XF::phrase('xfa_rmmp_payment_paid');
        }
        else
        {
            $paymentStatusPhrase = \XF::phrase('xfa_rmmp_payment_due');
        }

        $invoice->addBadge($paymentStatusPhrase->render());

        /* Add Paragraph */
        if ($seller->additional_info)
        {
            $additionalInfoPhrase = \XF::phrase('xfa_rmmp_additional_info');
            $invoice->addTitle($additionalInfoPhrase->render());
            if ($purchase->Transaction
                && $purchase->Transaction->shipping)
            {
	            $shippingFees = $purchase->Transaction->shipping .  $this->getCurrencySymbol(null, $purchase->currency);
	            
	            $shipping = \XF::phrase('xfa_rmmp_x_shipping_fees_included', ['fees' => $shippingFees]);
	            $invoice->addParagraph($shipping->render());	            
            }
            $invoice->addParagraph($seller->additional_info);
        }
        /* Set footer note */
        $invoice->setFooternote($seller->name);

        return $invoice->render($seller->invoice_prefix . sprintf("%'.010d", $purchase->purchase_id) . '.pdf', 'I');
    }

    protected function getCurrencySymbol($locale, $currency)
    {
        // Create a NumberFormatter
        $formatter = new \NumberFormatter($locale, \NumberFormatter::CURRENCY);

        // Prevent any extra spaces, etc. in formatted currency
        $formatter->setPattern('¤');

        // Prevent significant digits (e.g. cents) in formatted currency
        $formatter->setAttribute(\NumberFormatter::MAX_SIGNIFICANT_DIGITS, 0);

        // Get the formatted price for '0'
        $formattedPrice = $formatter->formatCurrency(0, $currency);

        // Strip out the zero digit to get the currency symbol
        $zero = $formatter->getSymbol(\NumberFormatter::ZERO_DIGIT_SYMBOL);
        $currencySymbol = str_replace($zero, '', $formattedPrice);

        return $currencySymbol;
    }
}