<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\View;

use XFA\RMMarketplace\vendor\pdfinvoice\src\InvoicePrinter;

class InvoicePreview extends \XF\Mvc\View
{
    public function renderRaw()
    {
        if (!headers_sent() && function_exists('header_remove'))
        {
            header_remove('Expires');
            header('Cache-control: private');
        }

        $this->response->header('Content-type', 'application/octet-stream', true);

        /* Get view data */
        $seller     = $this->params['seller'];
        $options    = \XF::options();
        $visitor    = \XF::visitor();

// Generate invoice
        $invoice = new InvoicePrinter('A4', $this->getCurrencySymbol(null, 'EUR'), $options->xfa_rmmp_invoicesLanguage);

        /* Header Settings */
        if (!empty($seller->hasLogo()))
        {
            $invoice->setLogo($seller->getLogoUrl(false));
        }
        $invoice->setColor("#007fff");
        $invoice->setType("Sale Invoice");
        $invoice->setReference($seller->invoice_prefix . sprintf("%'.010d", 0));
        $invoice->setDate(date($options->xfa_rmmp_invoicesDateFormat, \XF::$time));
        $invoice->setTime(date($options->xfa_rmmp_invoicesTimeFormat, \XF::$time));
        $invoice->setDue(date($options->xfa_rmmp_invoicesDateFormat, strtotime('+1 months')));

        $from = array($seller->name);
        if ($seller->address)
        {
            $fromAddress = explode('\n', $seller->address);
            $from = array_merge($from, $fromAddress);
        }

        if (count($from) < 5)
        {
            for ($i = count($from); $i < 5; $i++)
            {
                $from[] = "";
            }
        }

        $invoice->setFrom($from);

		$invoice->setTo(array($visitor['email'],"","","",""));

        /* Adding Items in table */
        $invoice->addItem('RM Marketplace', 'Awesome product', 1 , false, 60, false, 60);

        /* Add totals */
        $totalDuePhrase = \XF::phrase('xfa_rmmp_total_due');
        $invoice->addTotal($totalDuePhrase->render(), 60, true);

        /* Set badge */
        $paymentStatusPhrase = \XF::phrase('xfa_rmmp_payment_due');

        $invoice->addBadge($paymentStatusPhrase->render());

        /* Add Paragraph */
        if ($seller->additional_info)
        {
            $additionalInfoPhrase = \XF::phrase('xfa_rmmp_additional_info');
            $invoice->addTitle($additionalInfoPhrase->render());
            $invoice->addParagraph($seller->additional_info);
        }
        
        /* Set footer note */
        $invoice->setFooternote($seller->name);

        return $invoice->render($seller->invoice_prefix . 'preview.pdf', 'I');
    }

    protected function getCurrencySymbol($locale, $currency)
    {
        // Create a NumberFormatter
        $formatter = new \NumberFormatter($locale, \NumberFormatter::CURRENCY);

        // Prevent any extra spaces, etc. in formatted currency
        $formatter->setPattern('¤');

        // Prevent significant digits (e.g. cents) in formatted currency
        $formatter->setAttribute(\NumberFormatter::MAX_SIGNIFICANT_DIGITS, 0);

        // Get the formatted price for '0'
        $formattedPrice = $formatter->formatCurrency(0, $currency);

        // Strip out the zero digit to get the currency symbol
        $zero = $formatter->getSymbol(\NumberFormatter::ZERO_DIGIT_SYMBOL);
        $currencySymbol = str_replace($zero, '', $formattedPrice);

        return $currencySymbol;
    }
}