<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Repository;

use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;
use XF\PrintableException;

class FeaturingPurchase extends Repository
{
    public function findLogsForList()
    {
        return $this->finder('XFA\RMMarketplace:FeaturingPurchase')
            ->with(['Transaction', 'Resource', 'Buyer'])
            ->setDefaultOrder('purchase_date', 'DESC');
    }

    public function getUsersWhoPurchased()
    {
        return $this->db()->fetchPairs("
			SELECT user.user_id, user.username
			FROM (
				SELECT DISTINCT user_id FROM xf_xfa_rmmp_featuring_purchase
			) AS purchase
			INNER JOIN xf_user AS user ON (purchase.user_id = user.user_id)
			ORDER BY user.username
		");
    }

    public function pruneUnpaidPurchases()
    {
        /** @var \XFA\RMMarketplace\Finder\FeaturingPurchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:FeaturingPurchase');
        $purchaseFinder->with(['Transaction', 'Resource']);

        /* Unpaid purchases */
        $purchaseFinder->where('status', 'pending');

        /* Older than a day */
        $purchaseFinder->where('purchase_date', '<', strtotime('-1 day'));

        $purchases = $purchaseFinder->fetch();

        if ($purchases)
        {
            foreach($purchases AS $purchase)
            {
                $purchase->delete();
            }
        }
    }

    public function expirePurchasedFeaturing()
    {
        /** @var \XFA\RMMarketplace\Finder\FeaturingPurchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:FeaturingPurchase');
        $purchaseFinder->with(['Transaction', 'Resource','Resource.Featured']);
        $purchaseFinder->where('status', 'validated');

        $purchases = $purchaseFinder->fetch();

        if ($purchases)
        {
            foreach($purchases AS $purchase)
            {
                if ($purchase->Resource->Featured)
                {
                    $dateExpire = (new \DateTime())->setTimestamp($purchase->Resource->Featured->feature_date)->modify('+' . $purchase->duration)->getTimestamp();

                    if ($dateExpire < \XF::$time)
                    {
                        /* Expire the purchase */
                        /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Expire $expirer */
                        $expirer = \XF::app()->service('XFA\RMMarketplace:FeaturingPurchase\Expire', $purchase);
                        $expirer->expire();
                    }
                }
            }
        }
    }
}