<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Repository;

use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;
use XF\PrintableException;

class Purchase extends Repository
{
    public function findLastSales($type = 'all', array $viewableCategoryIds = null)
    {
        /** @var \XFA\RMMarketplace\Finder\Purchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:Purchase');

        if (is_array($viewableCategoryIds))
        {
            $purchaseFinder->where('Resource.resource_category_id', $viewableCategoryIds);
        }
        else
        {
            $purchaseFinder->with('Resource.Category.Permissions|' . \XF::visitor()->permission_combination_id);
        }

        if ($type == 'own')
        {
            $purchaseFinder->where('Resource.user_id', \XF::visitor()->user_id);
        }

        $purchaseFinder
            ->where('status', 'validated')
            ->where('Resource.resource_state', 'visible')
            ->with('Resource', true)
            ->with(['Resource.Category', 'Buyer'])
            ->setDefaultOrder('purchase_date', 'desc');

        return $purchaseFinder;
    }

    public function pruneUnpaidPurchases()
    {
        /** @var \XFA\RMMarketplace\Finder\Purchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:Purchase');
        $purchaseFinder->with(['Transaction', 'Resource']);

        // Unpaid purchases
        $purchaseFinder->where('status', 'pending');
        $purchaseFinder->whereOr(['Transaction.status', 'initialized'], ['Transaction.status', 'pending']);

        // Older than X days
        $purchaseFinder->where('purchase_date', '<', strtotime('-' . \XF::options()->xfa_rmmp_unpaidPruneDelay . ' days'));

        $purchases = $purchaseFinder->fetch();

        if ($purchases)
        {
            foreach($purchases AS $purchase)
            {
                /** @var \XFA\RMMarketplace\Service\Purchase\Delete $deleter */
                \XF::app()->service('XFA\RMMarketplace:Purchase\Delete', $purchase)->delete();
            }
        }
    }
}