<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\FeaturingPurchase;

use XFA\RMMarketplace\Entity\FeaturingPurchase;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XFRM\Entity\ResourceItem $resource
     */
    protected $resource;

    /**
     * @var \XFA\RMMarketplace\Entity\Transaction $transaction
     */
    protected $transaction;

    /**
     * @var FeaturingPurchase
     */
    protected $purchase;

    public function __construct(
        \XF\App $app,
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction
    )
    {
        parent::__construct($app);

        $this->resource     = $resource;
        $this->transaction  = $transaction;

        $this->purchase     = $this->em()->create('XFA\RMMarketplace:FeaturingPurchase');

        $this->setPurchaseDefaults();
    }

    public function getPurchase()
    {
        return $this->purchase;
    }

    protected function setPurchaseDefaults()
    {
        $visitor = \XF::visitor();

        $this->purchase->resource_id    = $this->resource->resource_id;
        $this->purchase->transaction_id = $this->transaction->transaction_id;
        $this->purchase->user_id        = $visitor->user_id;
        $this->purchase->username       = $visitor->username;
        $this->purchase->status         = 'pending';
        $this->purchase->currency       = strtoupper(\XF::options()->xfa_rmmp_featuring_purchase_currency);
        $this->purchase->purchase_date  = \XF::$time;
    }

    public function setPrice($price)
    {
        $this->purchase->price  = $price;
    }

    public function setDuration(\XFA\RMMarketplace\Entity\FeaturingDuration $duration)
    {
        $this->purchase->duration = $duration->value . ' ' . $duration->type;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $purchase = $this->purchase;

        $purchase->save();

        return $purchase;
    }
}