<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\License;

use XFA\RMMarketplace\Entity\License;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XFRM\Entity\ResourceItem $resource
     */
    protected $resource;

    /**
     * @var License
     */
    protected $license;

    public function __construct(\XF\App $app, \XFRM\Entity\ResourceItem $resource)
    {
        parent::__construct($app);

        $this->resource     = $resource;

        $this->license = $this->em()->create('XFA\RMMarketplace:License');
        $this->setLicenseDefaults();
    }

    public function getLicense()
    {
        return $this->license;
    }

    protected function setLicenseDefaults()
    {
        $visitor = \XF::visitor();

        $this->license->resource_id         = $this->resource->resource_id;

        $this->license->validated           = 0;
        $this->license->active              = 0;
        $this->license->user_id             = $visitor->user_id;
        $this->license->username            = $visitor->username;
        $this->license->start_date          = \XF::$time;
        $this->license->end_date            = 0;
        $this->license->previous_end_date   = 0;
    }

    public function setOwner($user)
    {
        $this->license->user_id     = $user->user_id;
        $this->license->username    = $user->username;
    }

    public function setStartDate($start)
    {
        $this->license->start_date = $start;
    }

    public function setEndDate($end)
    {
        $this->license->end_date = $end;
    }

    public function setEndDateWithDuration($durationString)
    {
        $this->license->end_date = strtotime('+' . $durationString, $this->license->start_date);
    }

    public function setPurchaseId($purchaseId)
    {
        $this->license->purchase_id = $purchaseId;
    }

    protected function _validate()
    {
        return [];
    }

    public function generateKey($format = 'AAAAA-AAAAA-AAAAA-AAAAA-AAAAA')
    {
        $key = '';
        
        for ($i = 0; $i < \strlen($format); ++$i)
        {
            // Get current requested char type
            $charType = $format[$i];

            // If any - pick one random
            if ($charType == '#')
            {
                $randomType = \random_int(1, 5);

                switch ($randomType)
                {
                    case 1: $charType = '9'; break;
                    case 2: $charType = 'x'; break;
                    case 3: $charType = 'X'; break;
                    case 4: $charType = 'a'; break;
                    case 5: $charType = 'A'; break;
                }
            }

            // Update key
            switch($charType)
            {
                // 0-9
                case '9':
                    $key .= \random_int(0, 9);
                    break;
                // a-z
                case 'x':
                    $key .= \chr(\random_int(97, 122));
                    break;
                // A-Z
                case 'X':
                    $key .= \chr(\random_int(65, 90));
                    break;
                // a-z or 0-9
                case 'a':
                    if (\random_int(1, 2) == 1)
                    {
                        $key .= \chr(\random_int(97, 122));
                    }
                    else
                    {
                        $key .= \random_int(0, 9);
                    }
                    break;
                // A-Z or 0-9
                case 'A':
                    if (\random_int(1, 2) == 1)
                    {
                        $key .= \chr(\random_int(65, 90));
                    }
                    else
                    {
                        $key .= \random_int(0, 9);
                    }
                    break;
                // Keep input character (e.g. -)
                default:
                    $key .= $charType;
                    break;
            }
        }

        // If key already exist, regenerate recursively
        $license = \XF::finder('XFA\RMMarketplace:License')
                        ->where('resource_id', '=', $this->resource->resource_id)
                        ->where('lic_key', '=', $key)
                        ->fetchOne();
        if ($license)
        {
            $key = $this->generateKey($format);
        }

        return $key;
    }

    protected function _save()
    {
        $license = $this->license;

        // Generate key if required
        if (!empty($license->Resource->DigitalProduct->lic_key_format))
        {
            $license->lic_key = $this->generateKey($license->Resource->DigitalProduct->lic_key_format);
        }

        $license->save();

        return $license;
    }
}