<?php
/*************************************************************************
 * XFA Core - Xen Factory (c) 2017
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\License;

use XFA\RMMarketplace\Entity\License;

class NotifyExpiration extends \XF\Service\AbstractService
{
    /**
     * @var License
     */
    protected $license;

    public function __construct(\XF\App $app, License $license)
    {
        parent::__construct($app);

        $this->license = $license;
    }

    public function notify()
    {
        $options = \XF::options();

        // Sent notification
        if ($options->xfa_rmmp_licenseExpirationAlert == 'alert' || $options->xfa_rmmp_licenseExpirationAlert == 'both')
        {
            $this->alert();
        }

        if ($options->xfa_rmmp_licenseExpirationAlert == 'email' || $options->xfa_rmmp_licenseExpirationAlert == 'both')
        {
            $this->email();
        }

        // Mark notification sent
        $this->license->fastUpdate('expiration_sent', 1);
    }

    private function alert()
    {
        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');

        $alertRepo->alert(
            $this->license->Holder,
            $this->license->Resource->user_id,
            $this->license->Resource->username,
            'xfa_rmmp',
            $this->license->resource_id,
            'license_expired'
        );
    }

    private function email()
    {
        $this->app->mailer()->newMail()
            ->setToUser($this->license->Holder)
            ->setTemplate('xfa_rmmp_license_expiration', ['license' => $this->license])
            ->queue();
    }
}