<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\License;

use XFA\RMMarketplace\Entity\License;

class Renew extends \XF\Service\AbstractService
{
    /**
     * @var License
     */
    protected $license;

    public function __construct(\XF\App $app, License $license)
    {
        parent::__construct($app);

        $this->license = $license;
    }

    public function renew()
    {
        $license = $this->license;

        /* Compute new end date */
        if ($license->end_date < \XF::$time)
        {
            $newEndDate = strtotime('+' . $license->Resource->DigitalProduct->duration . ' ' . $license->Resource->DigitalProduct->duration_type, \XF::$time);
        }
        else
        {
            $newEndDate = strtotime('+' . $license->Resource->DigitalProduct->duration . ' ' . $license->Resource->DigitalProduct->duration_type, $license->end_date);
        }

        $previousEndDate = $license->end_date;

        /* Handle usergroup change */
        if (\XF::options()->xfa_rmmp_usergroup_onbuy)
        {
            $db = $this->db();
            $db->beginTransaction();

            /* Expire previous change if any and create new one */

            /** @var \XF\Service\User\TempChange $userGroupChange */
            $userGroupChange = $this->service('XF:User\TempChange');
            $userGroupChange->expireUserChangeByKey($license->Holder, 'xfa_rmmp_license_' . $license->license_id);
            $userGroupChange->applyGroupChange(
                $license->Holder,
                'xfa_rmmp_license_' . $license->license_id,
                [\XF::options()->xfa_rmmp_usergroup_onbuy],
                null,
                $newEndDate
            );

            /** @var \XF\Repository\UserAlert $alertRepo */
            $alertRepo = $this->repository('XF:UserAlert');
            $alertRepo->fastDeleteAlertsFromUser(
                $license->Holder->user_id,
                'user',
                $license->Holder->user_id,
                'upgrade_end'
            );

            $db->commit();
        }

        /* Save data */
        $license->end_date          = $newEndDate;
        $license->previous_end_date = $previousEndDate;
        $license->save();
    }
}