<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\License;

use XFA\RMMarketplace\Entity\License;

class UndoRenew extends \XF\Service\AbstractService
{
    /**
     * @var License
     */
    protected $license;

    public function __construct(\XF\App $app, License $license)
    {
        parent::__construct($app);

        $this->license = $license;
    }

    public function undoRenew()
    {
        $license = $this->license;

        /* Compute new end date */
        $newEndDate         = $license->previous_end_date;
        $previousEndDate    = $license->end_date;

        /* Handle usergroup change */
        if (\XF::options()->xfa_rmmp_usergroup_onbuy)
        {
            /* Expire change */
            /** @var \XF\Service\User\TempChange $userGroupChange */
            $userGroupChange = $this->service('XF:User\TempChange');
            $userGroupChange->expireUserChangeByKey($license->Holder, 'xfa_rmmp_license_' . $license->license_id);

            /* Remaining time in license before renew, recreate new change */
            if ($newEndDate > \XF::$time)
            {
                /** @var \XF\Service\User\TempChange $userGroupChange */
                $userGroupChange->applyGroupChange(
                    $license->Holder,
                    'xfa_rmmp_license_' . $license->license_id,
                    [\XF::options()->xfa_rmmp_usergroup_onbuy],
                    null,
                    $newEndDate
                );
            }
        }

        $license->end_date          = $newEndDate;
        $license->previous_end_date = $previousEndDate;
        $license->save();
    }
}