<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\PosInvoice;

use XFA\RMMarketplace\Entity\MonthlyInvoice;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XF\Entity\User $seller
     */
    protected $seller;

    /**
     * @var MonthlyInvoice $invoice
     */
    protected $invoice;

    public function __construct(\XF\App $app, \XF\Entity\User $seller)
    {
        parent::__construct($app);

        $this->seller  = $seller;

        $this->invoice = $this->em()->create('XFA\RMMarketplace:MonthlyInvoice');
        $this->setInvoiceDefaults();
    }

    protected function setInvoiceDefaults()
    {
        $visitor = \XF::visitor();

        $this->invoice->user_id 		= $this->seller->user_id;
        $this->invoice->username 		= $this->seller->username;
        $this->invoice->status			= 'pending';
        $this->invoice->invoice_date	= \XF::$time;
    }
    
    public function setAmount($price, $currency)
    {
	    $this->invoice->price 		= $price;
	    $this->invoice->currency	= $currency;
    }
    
    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $invoice = $this->invoice;

		// Save invoice
        $invoice->save();
        
        // Notify seller     
        /** @var \XFA\RMMarketplace\Service\PosInvoice\Notify $notifier */
        $notifier = \XF::service('XFA\RMMarketplace:PosInvoice\Notify', $invoice);
        
        $notifier->emailSeller('xfa_rmmp_new_invoice');

        $notifier->alertSeller('xfa_rmmp_new_invoice');        

        return $invoice;
    }
}