<?php
/*************************************************************************
 * XFA Core - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\PosInvoice;

use XFA\RMMarketplace\Entity\MonthlyInvoice;

class Notify extends \XF\Service\AbstractService
{
    /**
     * @var MonthlyInvoice $invoice
     */
    protected $invoice;

    public function __construct(\XF\App $app, MonthlyInvoice $invoice)
    {
        parent::__construct($app);

        $this->invoice = $invoice;
    }

    public function emailSeller($mailTemplate, $params = [])
    {
        $params['invoice'] = $this->invoice;

        $this->app->mailer()->newMail()
            ->setToUser($this->invoice->Seller)
            ->setTemplate($mailTemplate, $params)
            ->queue();
    }

    public function alertSeller($alertType)
    {
        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');

        $alertRepo->alert(
            $this->invoice->Seller,
            $this->invoice->Seller->user_id,
            $this->invoice->Seller->username,
            'user',
            $this->invoice->Seller->user_id,
            $alertType,
            ['price' => $this->invoice->price, 'currency' => $this->invoice->currency]
        );
    }
}