<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\PosInvoice;

use XFA\RMMarketplace\Entity\MonthlyInvoice;

class Validate extends \XF\Service\AbstractService
{
    /**
     * @var MonthlyInvoice $invoice
     */
    protected $invoice;

    public function __construct(\XF\App $app, MonthlyInvoice $invoice)
    {
        parent::__construct($app);

        $this->invoice = $invoice;
    }

    public function validate()
    {
	    $invoice = $this->invoice;
	    
	    // Validate the invoice
	    $invoice->fastUpdate('status', 'validated');

	    // If user was blocked, unblock him if no other unpaid invoice remains
        if ($invoice->Seller->xfa_rmmp_user_blocked)
        {
            $db = \XF::db();

            $invoiceTimeThreshold = \XF::$time - (\XF::options()->xfa_rmmp_pendingInvoiceBlockSalesDays * 86400);

            $invoices = $db->fetchAllColumn("
				SELECT invoice_id
				FROM xf_xfa_rmmp_invoices
				WHERE invoice_date < ? 
				    AND status = 'pending'
				    AND warned = '0'
				    AND user_id = ?
				ORDER BY invoice_id ASC
			", [$invoiceTimeThreshold, $invoice->user_id]);

            if (!$invoices)
            {
                $invoice->Seller->fastUpdate('xfa_rmmp_user_blocked', 0);
            }
        }
	    
	    // Notify seller     
        /** @var \XFA\RMMarketplace\Service\PosInvoice\Notify $notifier */
        $notifier = \XF::service('XFA\RMMarketplace:PosInvoice\Notify', $invoice);
        
        $notifier->emailSeller('xfa_rmmp_invoice_paid');
    }
}