<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;
use XFA\RMMarketplace\Entity\PurchaseInfo;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XFRM\Entity\ResourceItem $resource
     */
    protected $resource;

    /**
     * @var \XFA\RMMarketplace\Entity\Transaction $transaction
     */
    protected $transaction;

    /**
     * @var \XFA\RMMarketplace\Entity\License $license
     */
    protected $license;

    /**
     * @var \XFA\RMMarketplace\Entity\Coupon $coupon
     */
    protected $coupon = null;

    /**
     * @var Purchase
     */
    protected $purchase;

    public function __construct(
        \XF\App $app,
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction,
        \XFA\RMMarketplace\Entity\License $license = null
    )
    {
        parent::__construct($app);

        $this->resource     = $resource;
        $this->transaction  = $transaction;
        $this->license      = $license;

        $this->purchase     = $this->em()->create('XFA\RMMarketplace:Purchase');

        $this->setPurchaseDefaults();
    }

    public function getPurchase()
    {
        return $this->purchase;
    }

    protected function setPurchaseDefaults()
    {
        $visitor = \XF::visitor();

        $this->purchase->resource_id    = $this->resource->resource_id;
        $this->purchase->transaction_id = $this->transaction->transaction_id;
        $this->purchase->license_id     = 0;
        $this->purchase->quantity       = 1;
        $this->purchase->user_id        = $visitor->user_id;
        $this->purchase->username       = $visitor->username;
        $this->purchase->type           = 'purchase';
        $this->purchase->status         = 'pending';
        $this->purchase->price          = $this->resource->xfa_rmmp_amount;
        $this->purchase->currency       = strtoupper($this->resource->xfa_rmmp_currency);
        $this->purchase->purchase_date  = \XF::$time;
    }

    public function setType($type)
    {
        $this->purchase->type = $type;

        if ($type == 'renewal')
        {
            $this->purchase->license_id = $this->license->license_id;
        }
    }

    public function setPrice($price)
    {
        $this->purchase->price = max($price, 0.0);
    }

    public function setQuantity($quantity)
    {
        $this->purchase->quantity = $quantity;
    }
    
    public function setForUser($user)
    {
        $this->purchase->for_user_id    = $user->user_id;
        $this->purchase->for_username   = $user->username;
    }

    public function setCoupon($coupon)
    {
        $this->coupon = $coupon;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $purchase   = $this->purchase;
        $license    = $this->license;
        $coupon     = $this->coupon;

        // Save purchase
        $purchase->save();

        // Update license with purchase id
        if ($license)
        {
            $license->purchase_id = $purchase->purchase_id;
            $license->save();
        }

        // Register coupon usage if any
        if ($coupon)
        {
            $couponUsage = $this->em()->create('XFA\RMMarketplace:CouponUsage');
            $couponUsage->user_id       = $purchase->user_id;
            $couponUsage->coupon_id     = $coupon->coupon_id;
            $couponUsage->purchase_id   = $purchase->purchase_id;

            $couponUsage->save();
        }

        return $purchase;
    }
}