<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;
use XFA\RMMarketplace\Entity\PurchaseInfo;

class CreateWithInfo extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XFRM\Entity\ResourceItem $resource
     */
    protected $resource;

    /**
     * @var \XFA\RMMarketplace\Entity\Transaction $transaction
     */
    protected $transaction;

    /**
     * @var Purchase
     */
    protected $purchase;

    /**
     * @var PurchaseInfo
     */
    protected $purchaseInfo;

    /**
     * @var \XFA\RMMarketplace\Entity\Coupon $coupon
     */
    protected $coupon = null;

    public function __construct(
        \XF\App $app,
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction
    )
    {
        parent::__construct($app);

        $this->resource     = $resource;
        $this->transaction  = $transaction;

        $this->purchase     = $this->em()->create('XFA\RMMarketplace:Purchase');
        $this->purchaseInfo = $this->em()->create('XFA\RMMarketplace:PurchaseInfo');

        $this->setPurchaseDefaults();
    }

    public function getPurchase()
    {
        return $this->purchase;
    }

    protected function setPurchaseDefaults()
    {
        $visitor = \XF::visitor();

        $this->purchase->resource_id    = $this->resource->resource_id;
        $this->purchase->transaction_id = $this->transaction->transaction_id;
        $this->purchase->quantity       = 1;
        $this->purchase->user_id        = $visitor->user_id;
        $this->purchase->username       = $visitor->username;
        $this->purchase->type           = 'purchase';
        $this->purchase->status         = 'pending';
        $this->purchase->price          = $this->resource->xfa_rmmp_amount;
        $this->purchase->currency       = strtoupper($this->resource->xfa_rmmp_currency);
        $this->purchase->purchase_date  = \XF::$time;
    }

    public function setPrice($price)
    {
        $this->purchase->price  = $price;
    }

    public function setQuantity($quantity)
    {
        $this->purchase->quantity = $quantity;
    }

    public function setName($firstName, $lastName)
    {
        $this->purchaseInfo->name = $firstName . ' ' . $lastName;
    }

    public function setPhone($phone)
    {
        $this->purchaseInfo->phone = $phone;
    }

    public function setAddress($street, $street2, $zip, $city, $state, $country)
    {
        $this->purchaseInfo->address = $street . "\r\n"
                                        . ($street2 ? $street2 . "\r\n" : "")
                                        . $zip . ' ' . $city . "\r\n"
                                        . ($state && $state != 'None' ? $state . "\r\n" : "")
                                        . $country;
    }

    public function setAdditionalInfo($additionalInfo)
    {
        $this->purchaseInfo->additional_info = $additionalInfo;
    }
    
    public function setForUser($user)
    {
        $this->purchase->for_user_id    = $user->user_id;
        $this->purchase->for_username   = $user->username;
    }

    public function setCoupon($coupon)
    {
        $this->coupon = $coupon;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $purchase   = $this->purchase;
        $coupon     = $this->coupon;

        $purchase->save();

        $this->purchaseInfo->purchase_id = $purchase->purchase_id;
        $this->purchaseInfo->save();

        // Register coupon usage if any
        if ($coupon)
        {
            $couponUsage = $this->em()->create('XFA\RMMarketplace:CouponUsage');
            $couponUsage->user_id       = $purchase->user_id;
            $couponUsage->coupon_id     = $coupon->coupon_id;
            $couponUsage->purchase_id   = $purchase->purchase_id;

            $couponUsage->save();
        }

        return $purchase;
    }
}