<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;
use XFA\RMMarketplace\Entity\PurchaseInfo;

class CreateWithoutTransaction extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var \XFRM\Entity\ResourceItem $resource
     */
    protected $resource;

    /**
     * @var \XFA\RMMarketplace\Entity\License $license
     */
    protected $license;

    /**
     * @var Purchase
     */
    protected $purchase;

    public function __construct(
        \XF\App $app,
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\License $license = null
    )
    {
        parent::__construct($app);

        $this->resource     = $resource;
        $this->license      = $license;

        $this->purchase     = $this->em()->create('XFA\RMMarketplace:Purchase');

        $this->setPurchaseDefaults();
    }

    public function getPurchase()
    {
        return $this->purchase;
    }

    protected function setPurchaseDefaults()
    {
        $visitor = \XF::visitor();

        $this->purchase->resource_id    = $this->resource->resource_id;
        $this->purchase->transaction_id = 0;
        $this->purchase->quantity       = 1;
        $this->purchase->user_id        = $visitor->user_id;
        $this->purchase->username       = $visitor->username;
        $this->purchase->type           = 'purchase';
        $this->purchase->status         = 'pending';
        $this->purchase->price          = $this->resource->xfa_rmmp_amount;
        $this->purchase->currency       = strtoupper($this->resource->xfa_rmmp_currency);
        $this->purchase->purchase_date  = \XF::$time;
    }

    public function setType($type)
    {
        $this->purchase->type = $type;
    }

    public function setPrice($price)
    {
        $this->purchase->price  = $price;
    }

    public function setQuantity($quantity)
    {
        $this->purchase->quantity = $quantity;
    }

    public function setPurchaser($user)
    {
        $this->purchase->user_id    = $user->user_id;
        $this->purchase->username   = $user->username;
    }

    public function setValidated()
    {
        $this->purchase->status             = 'validated';
        $this->purchase->validation_date    = \XF::$time;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $purchase = $this->purchase;
        $resource = $this->resource;

        /* Save purchase */
        $purchase->save();

        // Increment purchases counts
        $resource->xfa_rmmp_purchases = $resource->xfa_rmmp_purchases + $purchase->quantity;
        $resource->save();

        return $purchase;
    }
}