<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;

class Delete extends \XF\Service\AbstractService
{
	/**
	 * @var \XFA\RMMarketplace\Entity\Purchase
	 */
	protected $purchase;

	public function __construct(\XF\App $app, Purchase $purchase)
	{
		parent::__construct($app);
		$this->purchase = $purchase;
	}


	public function delete()
	{
	    $purchase = $this->purchase;

        $db = $this->db();
        $db->beginTransaction();

        // If the transaction attached to this purchase was a valid one, we need to reset some data
        if ($purchase->status == 'validated' && $purchase->Transaction && $purchase->Transaction->status == 'completed')
        {
            // Digital product: if purchase delete the licence, otherwise undo the renew
            if ($purchase->Resource->xfa_rmmp_type == 'digital')
            {
                if ($purchase->type == 'purchase')
                {
                    $licenses = $this->finder('XFA\RMMarketplace:License')->where('purchase_id', $purchase->purchase_id)->fetch();

                    foreach ($licenses AS $license)
                    {
                        $license->delete();
                    }
                }
                else
                {
                    /** @var \XFA\RMMarketplace\Service\License\UndoRenew $unRenewer */
                    $unRenewer = $this->service('XFA\RMMarketplace:License\UndoRenew', $purchase->License);
                    $unRenewer->undoRenew();
                }
            }

            // Coupon was used, need to decrement its usage
            if ($purchase->CouponUsage)
            {
                $purchase->CouponUsage->Coupon->fastUpdate('total_usages', ($purchase->CouponUsage->Coupon->total_usages - 1));
            }

            // Additional actions only needed when purchase type is purchase
            if ($purchase->type == 'purchase')
            {
                // Decrement the resource/user purchases/user sales count in case of purchase type
                if (($purchase->Resource->xfa_rmmp_purchases - $purchase->quantity) > 0)
                {
                    $purchase->Resource->fastUpdate('xfa_rmmp_purchases', ($purchase->Resource->xfa_rmmp_purchases - $purchase->quantity));
                }
                else
                {
                    $purchase->Resource->fastUpdate('xfa_rmmp_purchases', 0);
                }

                if ($purchase->Buyer->xfa_rmmp_user_purchases > 0)
                {
                    $purchase->Buyer->fastUpdate('xfa_rmmp_user_purchases', ($purchase->Buyer->xfa_rmmp_user_purchases - 1));
                }

                if ($purchase->transaction_id)
                {
                    if ($purchase->Resource->User->xfa_rmmp_user_sales > 0)
                    {
                        $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales', ($purchase->Resource->User->xfa_rmmp_user_sales - 1));
                    }

                    if ($purchase->Resource->User->xfa_rmmp_user_sales_amount - $purchase->price < 0)
                    {
                        $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales_amount', 0);
                    }
                    else
                    {
                        $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales_amount', ($purchase->Resource->User->xfa_rmmp_user_sales_amount - $purchase->price));
                    }
                }

                // Physical product: increment quantity
                if ($purchase->Resource->PhysicalProduct)
                {
                    $purchase->Resource->PhysicalProduct->quantity++;
                    $purchase->Resource->PhysicalProduct->save();
                }

                // Delete usergroup change if any
                if (\XF::options()->xfa_rmmp_usergroup_onbuy)
                {
                    /** @var \XF\Service\User\UserGroupChange $userGroupChange */
                    $userGroupChange = $this->service('XF:User\UserGroupChange');
                    $userGroupChange->removeUserGroupChange(
                        $purchase->Buyer->user_id,
                        'xfa_rmmp_purchase_' . $purchase->purchase_id
                    );
                }
            }
        }

        // Delete the coupon usage if existing
        if ($purchase->CouponUsage)
        {
            $purchase->CouponUsage->delete();
        }

        // Delete the transaction if existing
        if ($purchase->Transaction)
        {
            $purchase->Transaction->delete();
        }

        // If resource is physical, delete purchase info
        if (isset($purchase->Resource->PhysicalProduct) && $purchase->PurchaseInfo)
        {
            $purchase->PurchaseInfo->delete();
        }

	    // And finally, delete the purchase itself
        $result = $purchase->delete();

        $db->commit();

        return $result;
	}
}