<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;

class EditShipmentStatus extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var Purchase
     */
    protected $purchase;

    public function __construct(\XF\App $app, Purchase $purchase)
    {
        parent::__construct($app);

        $this->purchase = $purchase;
    }

    public function setShipmentStatus($shipmentStatus, $shipmentData)
    {
        $this->purchase->shipment_status  = $shipmentStatus;
        $this->purchase->shipment_data    = $shipmentData;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $purchase = $this->purchase;

        /* Save the purchase info */
        $purchase->save();

        /* Warn the purchaser */
        switch($purchase->shipment_status)
        {
            case 'pending':
                $statusPhrase = \XF::Phrase('xfa_rmmp_pending');
                break;
            case 'in_preparation':
                $statusPhrase = \XF::Phrase('xfa_rmmp_in_preparation');
                break;
            case 'sent':
                $statusPhrase = \XF::Phrase('xfa_rmmp_sent');
                break;
            case 'delivered':
                $statusPhrase = \XF::Phrase('xfa_rmmp_delivered');
                break;
        }
        
        $notifier = $this->service('XFA\RMMarketplace:Purchase\Notify', $purchase);
        $notifier->emailBuyer('xfa_rmmp_order_update', ['shipment_status' => $statusPhrase]);

        return $purchase;
    }
}