<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;

class Refund extends \XF\Service\AbstractService
{
	/**
	 * @var \XFA\RMMarketplace\Entity\Purchase
	 */
	protected $purchase;

	public function __construct(\XF\App $app, Purchase $purchase)
	{
		parent::__construct($app);
		$this->purchase = $purchase;
	}


	public function refund()
	{
	    $purchase = $this->purchase;

        $db = $this->db();
        $db->beginTransaction();

        // Update transaction data (if existing)
        if ($purchase->Transaction)
        {
            $purchase->Transaction->fees = 0;
            $purchase->Transaction->txn_id_pos = 0;
            $purchase->Transaction->pos = 0;
            $purchase->Transaction->save();
        }

        // Update the purchase in the database
        $purchase->status           = 'refunded';
        $purchase->validation_date  = \XF::$time;
        $purchase->save();

        // Set the license inactive if digital product
        if ($purchase->Resource->xfa_rmmp_type == 'digital')
        {
            if ($purchase->type == 'purchase')
            {
                $licenses = $this->finder('XFA\RMMarketplace:License')->where('purchase_id', $purchase->purchase_id)->fetch();

                foreach ($licenses AS $license)
                {
                    /** @var \XFA\RMMarketplace\Service\License\UndoRenew $unRenewer */
                    $unRenewer = $this->service('XFA\RMMarketplace:License\UndoRenew', $license);
                    $unRenewer->undoRenew();
                }
            }
            else
            {
                /** @var \XFA\RMMarketplace\Service\License\UndoRenew $unRenewer */
                $unRenewer = $this->service('XFA\RMMarketplace:License\UndoRenew', $purchase->License);
                $unRenewer->undoRenew();
            }
        }

        // Decrement coupon usage if any
        if ($purchase->CouponUsage)
        {
            $purchase->CouponUsage->Coupon->fastUpdate('total_usages', ($purchase->CouponUsage->Coupon->total_usages - 1));
        }

        // Additional actions only needed when purchase type is purchase
        if ($purchase->type == 'purchase')
        {
            // Decrement the resource/user purchases/user sales count in case of purchase type
            if (($purchase->Resource->xfa_rmmp_purchases - $purchase->quantity) > 0)
            {
                $purchase->Resource->fastUpdate('xfa_rmmp_purchases', ($purchase->Resource->xfa_rmmp_purchases - $purchase->quantity));
            }
            else
            {
                $purchase->Resource->fastUpdate('xfa_rmmp_purchases', 0);
            }

            if ($purchase->Buyer->xfa_rmmp_user_purchases > 0)
            {
                $purchase->Buyer->fastUpdate('xfa_rmmp_user_purchases', ($purchase->Buyer->xfa_rmmp_user_purchases - 1));
            }

            if ($purchase->transaction_id)
            {
                if ($purchase->Resource->User->xfa_rmmp_user_sales > 0)
                {
                    $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales', ($purchase->Resource->User->xfa_rmmp_user_sales - 1));
                }

                if ($purchase->Resource->User->xfa_rmmp_user_sales_amount - $purchase->price < 0)
                {
                    $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales_amount', 0);
                } else
                {
                    $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales_amount', ($purchase->Resource->User->xfa_rmmp_user_sales_amount - $purchase->price));
                }
            }

            // Physical product: increment quantity
            if ($purchase->Resource->PhysicalProduct)
            {
                $purchase->Resource->PhysicalProduct->quantity++;
                $purchase->Resource->PhysicalProduct->save();
            }

            $notifier = $this->service('XFA\RMMarketplace:Purchase\Notify', $purchase);

            // Communicate with the buyer
            $notifier->communicate('purchase_refund');

            // Alert seller
            $notifier->alertSeller('refund_complete');

            // If required delete usergroup change
            if (\XF::options()->xfa_rmmp_usergroup_onbuy)
            {
                /** @var \XF\Service\User\UserGroupChange $userGroupChange */
                $userGroupChange = $this->service('XF:User\UserGroupChange');
                $userGroupChange->removeUserGroupChangeLogByKey(
                    'xfa_rmmp_purchase_' . $purchase->purchase_id
                );
            }
        }
        else
        {
            $notifier = $this->service('XFA\RMMarketplace:Purchase\Notify', $purchase);

            // Communicate with the buyer
            $notifier->communicate('renewal_refund');

            // Alert seller
            $notifier->alertSeller('refund_renewal_complete');
        }

        $db->commit();
    }
}