<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;

class Validate extends \XF\Service\AbstractService
{
    /**
     * @var Purchase
     */
    protected $purchase;

    public function __construct(\XF\App $app, Purchase $purchase)
    {
        parent::__construct($app);

        $this->purchase = $purchase;
    }

    public function validate()
    {
        $db = $this->db();
        $db->beginTransaction();

        $purchase = $this->purchase;
        $resource = $purchase->Resource;

        /* Validate the purchase */
        $purchase->status             = 'validated';
        $purchase->validation_date    = \XF::$time;
        $purchase->save();

        /* Set the license(s) active if digital product */
        if ($resource->xfa_rmmp_type == 'digital')
        {
            if ($purchase->type == 'purchase')
            {
                $licenses = $this->finder('XFA\RMMarketplace:License')->where('purchase_id', $purchase->purchase_id)->fetch();

                foreach ($licenses AS $license)
                {
                    /** @var \XFA\RMMarketplace\Service\License\Activate $activator */
                    $activator = $this->service('XFA\RMMarketplace:License\Activate', $license);
                    $activator->activate();
                }
            }
            else
            {
                /** @var \XFA\RMMarketplace\Service\License\Renew $renewer */
                $renewer = $this->service('XFA\RMMarketplace:License\Renew', $purchase->License);
                $renewer->renew();
            }
        }

        // Increment coupon usage if any
        if ($purchase->CouponUsage)
        {
            $purchase->CouponUsage->Coupon->fastUpdate('total_usages', ($purchase->CouponUsage->Coupon->total_usages + 1));
        }

        if ($purchase->type == 'purchase')
        {
            // Increment resources purchases
            $resource->fastUpdate('xfa_rmmp_purchases', ($resource->xfa_rmmp_purchases + $purchase->quantity));

            // Increment buyer purchases
            $purchase->Buyer->fastUpdate('xfa_rmmp_user_purchases', ($purchase->Buyer->xfa_rmmp_user_purchases + 1));

            // Increment seller sales
            $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales', ($purchase->Resource->User->xfa_rmmp_user_sales + 1));
            $purchase->Resource->User->fastUpdate('xfa_rmmp_user_sales_amount', ($purchase->Resource->User->xfa_rmmp_user_sales_amount + $purchase->price));

            // Warn out of stock if reached
            if ($resource->xfa_rmmp_type == 'physical')
            {
                if ($resource->PhysicalProduct->quantity == 0)
                {
                    $this->repository('XFRM:ResourceItem')->sendSellerOutOfStockAlert($resource);
                }
            }

            $notifier = $this->service('XFA\RMMarketplace:Purchase\Notify', $purchase);

            // Communicate with the buyer
            $notifier->communicate('purchase');

            // Alert seller
            $notifier->alertSeller('purchase_complete');
            
            // Alert friend if bought for friend
	        if ($this->purchase->ForUser)
	        {
	            // If purchased for user send email
	            $params = [
	                'purchase' => $this->purchase
	            ];
	
	            $this->app->mailer()->newMail()
	                ->setToUser($this->purchase->ForUser)
	                ->setTemplate('xfa_rmmp_x_purchased_for_you', $params)
	                ->queue();
	        }

            // If required change usergroup
            if (\XF::options()->xfa_rmmp_usergroup_onbuy)
            {
                /** @var \XF\Service\User\UserGroupChange $userGroupChange */
                $userGroupChange = $this->service('XF:User\UserGroupChange');
                $userGroupChange->addUserGroupChange(
                    $purchase->Buyer->user_id,
                    'xfa_rmmp_purchase_' . $purchase->purchase_id,
                    [\XF::options()->xfa_rmmp_usergroup_onbuy]
                );

                /** @var \XF\Repository\UserAlert $alertRepo */
                $alertRepo = $this->repository('XF:UserAlert');
                $alertRepo->fastDeleteAlertsFromUser(
                    $purchase->Buyer->user_id,
                    'user',
                    $purchase->Buyer->user_id,
                    'upgrade_end'
                );
            }
        }
        else
        {
            $notifier = $this->service('XFA\RMMarketplace:Purchase\Notify', $purchase);

            // Communicate with the buyer
            $notifier->communicate('renewal');

            // Alert seller
            $notifier->alertSeller('purchase_renewal_complete');
        }

        $db->commit();
    }
}