<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Seller;

use XFA\RMMarketplace\Entity\Seller;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var Seller
     */
    protected $seller;

    public function __construct(\XF\App $app, \XF\Entity\User $user)
    {
        parent::__construct($app);

        $this->seller = $this->em()->create('XFA\RMMarketplace:Seller');
        $this->seller->user_id = $user->user_id;
    }

    public function configureInvoicing($activate, $prefix)
    {
        $this->seller->activate_invoicing = $activate;
        $this->seller->invoice_prefix     = $prefix;
    }

    public function setSellerData($name, $address, $additionalInfo)
    {
        $this->seller->name             = $name;
        $this->seller->address          = $address;
        $this->seller->additional_info  = $additionalInfo;
    }

    public function addLogo(\XF\FileWrapper $logo)
    {
        $maxWidth       = 400;
        $maxHeight      = 250;

        $image = $this->app->imageManager()->imageFromFile($logo->getFilePath());
        if (!$image)
        {
            return null;
        }

        $image->resize($maxWidth, $maxHeight);

        $newTempFile = \XF\Util\File::getTempFile();
        if ($newTempFile && $image->save($newTempFile))
        {
            $filePath = sprintf('data://xfa/rmmarketplace/logo_images/%d/%d.png',
                floor($this->seller->user_id / 1000),
                $this->seller->user_id
            );

            try
            {
                \XF\Util\File::copyFileToAbstractedPath($newTempFile, $filePath);
                $this->seller->logo_date = \XF::$time;
            }
            catch (\Exception $e)
            {
                \XF\Util\File::deleteFromAbstractedPath($filePath);

                throw $e;
            }
        }
    }

    public function setCommunicationType($commType)
    {
        $this->seller->comm_type = $commType;
    }

    public function setCommunicationData($confArray)
    {
        $this->seller->comm_title_purchase          = $confArray['comm_title_purchase'];
        $this->seller->comm_text_purchase           = $confArray['comm_text_purchase'];
        $this->seller->comm_title_purchase_refund   = $confArray['comm_title_purchase_refund'];
        $this->seller->comm_text_purchase_refund    = $confArray['comm_text_purchase_refund'];
        $this->seller->comm_title_renewal           = $confArray['comm_title_renewal'];
        $this->seller->comm_text_renewal            = $confArray['comm_text_renewal'];
        $this->seller->comm_title_renewal_refund    = $confArray['comm_title_renewal_refund'];
        $this->seller->comm_text_renewal_refund     = $confArray['comm_text_renewal_refund'];
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $seller = $this->seller;

        $seller->save();

        return $seller;
    }
}