<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Stats;

class Grapher extends \XF\Service\Stats\Grapher
{
    protected $currency     = 'USD';
    protected $userId       = 0;
    protected $resourceId   = 0;
    protected $saleType     = 'all';

    public function filterByCurrency($currency)
    {
        $this->currency = $currency;
    }

    public function filterByUser(\XF\Entity\User $user)
    {
        $this->userId = $user->user_id;
    }

    public function filterByResource($resourceId)
    {
        $this->resourceId = $resourceId;
    }

    public function filterBySaleType($saleType)
    {
        $this->saleType = $saleType;
    }

    protected function getRawDataWhereClause()
    {
        $db = $this->db();

        $where  = 'purchase.purchase_date BETWEEN ' . $db->quote($this->start) . ' AND ' . $db->quote($this->end) . '
    			    AND purchase.status=\'validated\'';

        if($this->currency != 'all')
        {
            $where .= ' AND purchase.currency=' . $db->quote($this->currency);
        }

        if ($this->userId != 0)
        {
            $where = 'resource.user_id=' . $db->quote($this->userId) . ' AND ' . $where;
        }

        if ($this->resourceId != 0)
        {
            $where = 'resource.resource_id=' . $db->quote($this->resourceId) . ' AND ' . $where;
        }

        if ($this->saleType != 'all')
        {
            if ($this->saleType == 'purchase')
            {
                $where .= ' AND purchase.type=\'purchase\'';
            }
            else
            {
                $where .= ' AND purchase.type=\'renewal\'';
            }
        }

        return $where;
    }

    protected function getRawData()
    {
        if (!$this->types)
        {
            throw new \LogicException("Must have at least one type selected");
        }

        $output = [];
        $db = $this->db();

        $where = $this->getRawDataWhereClause();

        $stats = $db->query('
			SELECT COUNT(purchase_id) AS nb_sales,ROUND(AVG(purchase.purchase_date)) AS avg_purchase_date, SUM(purchase.price) AS total_price, SUM(transaction.fees) AS total_fees, SUM(transaction.pos) AS total_pos, SUM(transaction.shipping) AS total_shipping
			FROM xf_xfa_rmmp_purchase AS purchase
			LEFT JOIN xf_rm_resource AS resource ON (resource.resource_id=purchase.resource_id)
			LEFT JOIN xf_xfa_rmmp_transaction AS transaction ON (transaction.transaction_id=purchase.transaction_id)
			WHERE ' . $where . '
			GROUP BY YEAR(FROM_UNIXTIME(purchase.purchase_date)), DAYOFYEAR(FROM_UNIXTIME(purchase.purchase_date))
			ORDER BY purchase_date
		');

        while ($stat = $stats->fetch())
        {
            if (in_array('nb_sales', $this->types))
            {
                $output[$stat['avg_purchase_date']]['nb_sales'] = $stat['nb_sales'];
            }

            if (in_array('gross', $this->types))
            {
                $output[$stat['avg_purchase_date']]['gross'] = $stat['total_price'] + 0;
            }

            if (in_array('fees', $this->types))
            {
                $output[$stat['avg_purchase_date']]['fees'] = $stat['total_fees'] + 0;
            }

            if (in_array('pos', $this->types))
            {
                $output[$stat['avg_purchase_date']]['pos'] = $stat['total_pos'] + 0;
            }

            if (in_array('net', $this->types))
            {
                $output[$stat['avg_purchase_date']]['net'] = $stat['total_price'] - $stat['total_fees'] - $stat['total_pos'] - $stat['total_shipping'];
            }
        }

        /* If we filter by resource, we must get the updates if updates in the types */
        if ($this->resourceId && in_array('updates', $this->types))
        {
            $updates = $db->query('
                SELECT COUNT(resource_update_id) AS nb_updates, AVG(post_date) AS avg_post_date
                FROM xf_rm_resource_update
                WHERE resource_id=' . $db->quote($this->resourceId) . ' AND post_date BETWEEN ' . $this->start . ' AND ' . $this->end . '
			    GROUP BY YEAR(FROM_UNIXTIME(post_date)), DAYOFYEAR(FROM_UNIXTIME(post_date))
                ORDER BY post_date            
            ');

            while ($update = $updates->fetch())
            {
                $output[$update['avg_post_date']]['updates'] = $update['nb_updates'];
            }
        }

        return $output;
    }

    public function getSalesTotal()
    {
        $db = $this->db();

        $where = $this->getRawDataWhereClause();

        /* Get the sales data */
        $data = $db->fetchRow('
			SELECT COUNT(purchase_id) AS nb_sales,AVG(purchase.purchase_date) AS avg_purchase_date, SUM(purchase.price) AS total_price, SUM(transaction.fees) AS total_fees, SUM(transaction.pos) AS total_pos, SUM(transaction.shipping) AS total_shipping
			FROM xf_xfa_rmmp_purchase AS purchase
			LEFT JOIN xf_rm_resource AS resource ON (resource.resource_id=purchase.resource_id)
			LEFT JOIN xf_xfa_rmmp_transaction AS transaction ON (transaction.transaction_id=purchase.transaction_id)
			WHERE ' . $where
        );

        if (!$data)
        {
            return [];
        }

        /* Prepare stats */
        return [
            'nb_sales'  => $data['nb_sales'],
            'gross'     => $data['total_price'] + 0,
            'fees'      => $data['total_fees'] + 0,
            'shipping'	=> $data['total_shipping'] + 0,
            'pos'       => $data['total_pos'] + 0,
            'net'       => ($data['total_price'] - $data['total_fees'] - $data['total_pos'] - $data['total_shipping'])
        ];
    }

    public function getGroupedData(\XF\Stats\Grouper\AbstractGrouper $grouper)
    {
        $data = parent::getGroupedData($grouper);

        /* xF averages, we need to undo that */
        foreach($data AS &$_data)
        {
            foreach($_data['values'] AS $key => $value)
            {
                $_data['averages'][$key] = $value;
            }
        }

        return $data;
    }
}