<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Transaction;

use XFA\RMMarketplace\Entity\Transaction;

class Create extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var Transaction
     */
    protected $transaction;

    public function __construct(\XF\App $app, \XFRM\Entity\ResourceItem $resource)
    {
        parent::__construct($app);

        $this->transaction = $this->em()->create('XFA\RMMarketplace:Transaction');
        $this->setTransactionDefaults();
    }

    public function getTransaction()
    {
        return $this->transaction;
    }

    public function setTransactionDefaults()
    {
        $this->transaction->status = 'initialized';
    }

    public function setPaypalEmail($email)
    {
        $this->transaction->business = $email;
    }

    public function setPendingManualStatus()
    {
        $this->transaction->status = 'pending_manual';
    }

    public function setCompletedStatus()
    {
        $this->transaction->status = 'completed';
    }

    public function setPercentageOnSaleData($txnId, $amount, $setPendingPosPaid = false)
    {
        $this->transaction->txn_id_pos  = $txnId;
        $this->transaction->pos         = $amount;

        if ($setPendingPosPaid)
        {
            $this->transaction->pos_paid = 'pending';
        }
    }
    
    public function setShipping($shipping)
    {
	    $this->transaction->shipping = $shipping;
    }

    protected function _validate()
    {
        return [];
    }

    protected function _save()
    {
        $transaction = $this->transaction;

        $transaction->save();

        return $transaction;
    }
}