<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	// Create tables
    public function installStep1()
    {
        $sm = $this->schemaManager();

        $sm->createTable('xf_xfa_rmmp_transaction', function(Create $table)
        {
            $table->addColumn('transaction_id', 'int')->autoIncrement();
            $table->addColumn('business', 'varchar', 250)->setDefault('');
            $table->addColumn('txn_id', 'varchar', 30)->setDefault('');
            $table->addColumn('txn_id_pos', 'varchar', 30)->setDefault('');
            $table->addColumn('token', 'varchar', 20)->nullable()->setDefault('NULL');
            $table->addColumn('fees', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('pos', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('shipping', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('status', 'enum')->values(['initialized', 'pending', 'completed', 'expired', 'pending_manual'])->setDefault('completed');
            $table->addColumn('pos_paid', 'enum')->values(['na', 'pending', 'completed'])->setDefault('na');
            $table->addColumn('pos_invoice_id', 'int')->setDefault(0);
        });

        $sm->createTable('xf_xfa_rmmp_purchase', function(Create $table)
        {
            $table->addColumn('purchase_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('license_id', 'int')->setDefault(0);
            $table->addColumn('quantity', 'int')->setDefault(1);
            $table->addColumn('transaction_id', 'int');
            $table->addColumn('type', 'enum')->values(['purchase', 'renewal'])->setDefault('purchase');
            $table->addColumn('status', 'enum')->values(['pending', 'validated', 'refunded'])->setDefault('pending');
            $table->addColumn('shipment_status', 'enum')->values(['pending', 'in_preparation', 'sent', 'delivered'])->setDefault('pending');
            $table->addColumn('shipment_data', 'varchar', 500)->setDefault('');
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('for_user_id', 'int');
            $table->addColumn('for_username', 'varchar', 100)->setDefault('');
            $table->addColumn('price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('currency', 'varchar', 3)->setDefault('');
            $table->addColumn('purchase_date', 'int');
            $table->addColumn('validation_date', 'int');
            $table->addKey(['purchase_id','resource_id'], 'purchase_resource');
            $table->addKey(['purchase_id','transaction_id'], 'purchase_transaction');
            $table->addKey(['purchase_id','license_id'], 'purchase_license');
            $table->addKey(['purchase_id','user_id'], 'purchase_user');
        });

        $sm->createTable('xf_xfa_rmmp_purchase_add_info', function(Create $table)
        {
            $table->addColumn('purchase_id', 'int');
            $table->addColumn('name', 'varchar', 200)->setDefault('');
            $table->addColumn('phone', 'varchar', 100)->setDefault('');
            $table->addColumn('address', 'text');
            $table->addColumn('additional_info', 'text');
            $table->addPrimaryKey('purchase_id');
        });

        $sm->createTable('xf_xfa_rmmp_digital_product', function(Create $table)
        {
            $table->addColumn('digitalp_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('duration', 'int');
            $table->addColumn('duration_type', 'enum')->values(['day', 'week', 'month', 'year'])->setDefault('day');
            $table->addColumn('has_restriction', 'tinyint')->setDefault(0);
            $table->addColumn('require_url', 'tinyint')->setDefault(0);
            $table->addColumn('renewal_price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('base_renewal_price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('delay_renewal', 'int');
            $table->addColumn('allow_expired_renewal', 'tinyint')->setDefault(1);
            $table->addColumn('lic_key_format', 'mediumtext')->nullable()->setDefault(null);
            $table->addKey(['digitalp_id','resource_id'], 'digitalp_resource');
        });

        $sm->createTable('xf_xfa_rmmp_physical_product', function(Create $table)
        {
            $table->addColumn('physicalp_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('quantity', 'int')->setDefault(0);
            $table->addColumn('sell_on_stockout', 'tinyint')->setDefault(0);
            $table->addColumn('shipping', 'text')->nullable()->setDefault(null);
            $table->addKey(['physicalp_id','resource_id'], 'physicalp_resource');
        });

        $sm->createTable('xf_xfa_rmmp_license', function(Create $table)
        {
            $table->addColumn('license_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('purchase_id', 'int')->setDefault(0);
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('validated', 'tinyint')->setDefault(0);
            $table->addColumn('active', 'tinyint')->setDefault(0);
            $table->addColumn('siteurl', 'varchar', 200)->setDefault('');
            $table->addColumn('start_date', 'int');
            $table->addColumn('end_date', 'int');
            $table->addColumn('previous_end_date', 'int');
            $table->addColumn('expiration_sent', 'tinyint')->setDefault(0);
            $table->addColumn('lic_key', 'mediumtext')->nullable()->setDefault(null);
            $table->addKey(['license_id','resource_id'], 'license_resource');
            $table->addKey(['license_id','user_id'], 'license_user');
        });

        $sm->createTable('xf_xfa_rmmp_seller_info', function(Create $table)
        {
            $table->addColumn('user_id', 'int');
            $table->addColumn('activate_invoicing', 'tinyint')->setDefault(0);
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('invoice_prefix', 'varchar', 5)->setDefault('');
            $table->addColumn('address', 'mediumtext')->nullable();
            $table->addColumn('additional_info', 'mediumtext')->nullable();
            $table->addColumn('logo_date', 'int');
            $table->addColumn('comm_type', 'enum')->values(['email', 'pm', 'both'])->setDefault('email');
            $table->addColumn('comm_title_purchase', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_purchase', 'mediumtext');
            $table->addColumn('comm_title_renewal', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_renewal', 'mediumtext');
            $table->addColumn('comm_title_purchase_refund', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_purchase_refund', 'mediumtext');
            $table->addColumn('comm_title_renewal_refund', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_renewal_refund', 'mediumtext');
            $table->addUniqueKey('user_id', 'user_id');
        });

        $sm->createTable('xf_xfa_rmmp_featuring_purchase', function(Create $table)
        {
            $table->addColumn('purchase_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('transaction_id', 'int');
            $table->addColumn('status', 'enum')->values(['pending', 'validated', 'refunded', 'expired'])->setDefault('pending');
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('currency', 'varchar', 3)->setDefault('');
            $table->addColumn('duration', 'varchar', 100);
            $table->addColumn('purchase_date', 'int');
            $table->addColumn('validated', 'tinyint')->setDefault(0);
            $table->addColumn('validation_date', 'int');
            $table->addColumn('refund_txn_id', 'varchar', 30);
        });

        $sm->createTable('xf_xfa_rmmp_featuring_duration', function(Create $table)
        {
            $table->addColumn('duration_id', 'int')->autoIncrement();
            $table->addColumn('value', 'int');
            $table->addColumn('type', 'varchar', 10);
            $table->addColumn('amount', 'decimal', '10,2');
            $table->addColumn('active', 'tinyint')->setDefault(1);
            $table->addColumn('display_order', 'int');
        });
        
        $sm->createTable('xf_xfa_rmmp_invoices', function(Create $table)
        {
            $table->addColumn('invoice_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('currency', 'varchar', 3)->setDefault('');
            $table->addColumn('status', 'enum')->values(['pending', 'validated', 'refunded', 'expired'])->setDefault('pending');
            $table->addColumn('invoice_date', 'int');
            $table->addColumn('warned', 'tinyint')->setDefault(0);
        });

        $sm->createTable('xf_xfa_rmmp_discount', function(Create $table)
        {
            $table->addColumn('discount_id', 'int')->autoIncrement();
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('creator_id', 'int')->setDefault(0);
            $table->addColumn('discount_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('is_fixed', 'tinyint')->setDefault(0);
            $table->addColumn('applyto_renewal', 'tinyint')->setDefault(0);
            $table->addColumn('type', 'enum')->values(['all', 'all_own', 'specifics'])->setDefault('all_own');
            $table->addColumn('resource_ids', 'mediumtext');
            $table->addColumn('from_date', 'int')->setDefault(0);
            $table->addColumn('to_date', 'int')->setDefault(0);
            $table->addColumn('applied', 'tinyint')->setDefault(0);
        });

        $sm->createTable('xf_xfa_rmmp_coupon', function(Create $table)
        {
            $table->addColumn('coupon_id', 'int')->autoIncrement();
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('code', 'varchar', 100)->setDefault('');
            $table->addColumn('creator_id', 'int')->setDefault(0);
            $table->addColumn('coupon_type', 'enum')->values(['free', 'discount', 'percentage'])->setDefault('percentage');
            $table->addColumn('coupon_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('apply_to', 'enum')->values(['purchase', 'renewal', 'both'])->setDefault('purchase');
            $table->addColumn('type', 'enum')->values(['all', 'all_own', 'specifics'])->setDefault('all_own');
            $table->addColumn('resource_ids', 'mediumtext');
            $table->addColumn('from_date', 'int')->setDefault(0);
            $table->addColumn('to_date', 'int')->setDefault(0);
            $table->addColumn('usergroups', 'varchar', 500)->setDefault('');
            $table->addColumn('user_ids', 'varchar', 500)->setDefault('');
            $table->addColumn('authorized_usages', 'int')->unsigned(false)->setDefault(-1);
            $table->addColumn('total_usages', 'int')->setDefault(0);
        });

        $sm->createTable('xf_xfa_rmmp_coupon_usage', function(Create $table)
        {
            $table->addColumn('usage_id', 'int')->autoIncrement();
            $table->addColumn('coupon_id', 'int')->setDefault(0);
            $table->addColumn('purchase_id', 'int')->setDefault(0);
            $table->addColumn('user_id', 'int')->setDefault(0);
            $table->addKey(['coupon_id','user_id'], 'coupon_user');
        });
    }

    // Alter tables
    public function installStep2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_conf', 'tinyint', 3)->setDefault(0);
            $table->addColumn('xfa_rmmp_types', 'varchar', 255)->setDefault('a:4:{s:4:"free";s:1:"1";s:7:"digital";a:4:{s:6:"active";s:1:"1";s:7:"renewal";s:1:"1";s:11:"restriction";s:1:"1";s:12:"url_required";s:1:"1";}s:8:"physical";s:1:"1";s:7:"service";s:1:"1";}');
            $table->addColumn('xfa_rmmp_payment_data', 'varchar', 250)->setDefault('');
            $table->addColumn('xfa_rmmp_sale_percentage', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('xfa_rmmp_sale_percentage_paypal', 'varchar', 250)->setDefault('');
            $table->addColumn('xfa_rmmp_alt_display', 'tinyint', 1)->setDefault(0);
            $table->addColumn('xfa_rmmp_restrict_support', 'tinyint', 1)->setDefault(0);
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_parent_id', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_type', 'enum')->values(['none', 'digital', 'physical', 'service'])->setDefault('none');
            $table->addColumn('xfa_rmmp_max_qty', 'int')->setDefault(1);
            $table->addColumn('xfa_rmmp_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('xfa_rmmp_base_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('xfa_rmmp_discount_id', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_currency', 'varchar', 3)->setDefault('');
            $table->addColumn('xfa_rmmp_paypal', 'varchar', 250)->setDefault('');
            $table->addColumn('xfa_rmmp_accept_other', 'tinyint', 1)->setDefault(0);
            $table->addColumn('xfa_rmmp_purchases', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_cgv', 'mediumtext')->nullable()->setDefault(null);
            $table->addColumn('xfa_rmmp_img_date', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_sales_disabled', 'tinyint', 1)->setDefault(0);
        });

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_purchases', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_user_sales', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_user_sales_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('xfa_rmmp_user_cgv', 'mediumtext')->nullable()->setDefault(null);
            $table->addColumn('xfa_rmmp_user_paypal', 'varchar', 250)->setDefault('');
            $table->addColumn('xfa_rmmp_user_blocked', 'tinyint')->setDefault(0);
        });

        $sm->alterTable('xf_payment_profile', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_id', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_active', 'int')->setDefault(0);
        });
    }

    // Add db content
    public function installStep3()
    {
        $this->db()->query("
			REPLACE INTO xf_purchasable
				(purchasable_type_id, purchasable_class, addon_id)
			VALUES
				('xfa_rmmp_purchase', 'XFA\\\\RMMarketplace:MarketplacePurchase', 'XFA/RMMarketplace'),
				('xfa_rmmp_renew', 'XFA\\\\RMMarketplace:MarketplaceRenew', 'XFA/RMMarketplace'),
				('xfa_rmmp_feature', 'XFA\\\\RMMarketplace:MarketplaceFeature', 'XFA/RMMarketplace'),
				('xfa_rmmp_invoice', 'XFA\\\\RMMarketplace:MarketplaceInvoice', 'XFA/RMMarketplace')
		");
    }

    // Add widgets
    public function installStep4()
    {
        $this->_addWidgets();
    }

    // Upgrade from 3.0.0/3.1.0 to 3.2.0
    public function upgrade903020090Step1()
    {
        $sm = $this->schemaManager();

        $sm->createTable('xfa_rmmp_featuring_purchase', function(Create $table)
        {
            $table->addColumn('purchase_id', 'int')->autoIncrement();
            $table->addColumn('resource_id', 'int');
            $table->addColumn('transaction_id', 'int');
            $table->addColumn('status', 'enum')->values(['pending', 'validated', 'refunded', 'expired'])->setDefault('pending');
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('currency', 'varchar', 3)->setDefault('');
            $table->addColumn('duration', 'varchar', 100);
            $table->addColumn('purchase_date', 'int');
            $table->addColumn('validated', 'tinyint')->setDefault(0);
            $table->addColumn('validation_date', 'int');
            $table->addColumn('refund_txn_id', 'varchar', 30);
        });

        $sm->createTable('xfa_rmmp_featuring_duration', function(Create $table)
        {
            $table->addColumn('duration_id', 'int')->autoIncrement();
            $table->addColumn('value', 'int');
            $table->addColumn('type', 'varchar', 10);
            $table->addColumn('amount', 'decimal', '10,2');
            $table->addColumn('active', 'tinyint')->setDefault(1);
            $table->addColumn('display_order', 'int');
        });

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_types', 'varchar', 255)->setDefault('a:4:{s:4:"free";s:1:"1";s:7:"digital";a:4:{s:6:"active";s:1:"1";s:7:"renewal";s:1:"1";s:11:"restriction";s:1:"1";s:12:"url_required";s:1:"1";}s:8:"physical";s:1:"1";s:7:"service";s:1:"1";}');
        });
    }

    // Upgrade to 3.3.0
    public function upgrade903030090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_physical_product', function(Alter $table)
        {
            $table->addColumn('sell_on_stockout', 'tinyint')->setDefault(0);
        });
    }

    // Upgrade to 3.4.0
    public function upgrade903040090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_alt_display', 'tinyint', 1)->setDefault(0);
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_img_date', 'int')->setDefault(0);
        });
    }

    // Upgrade to 3.4.1
    public function upgrade903040190Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_cgv', 'mediumtext')->nullable()->setDefault(null);
        });
    }

    // Upgrade to 3.5.0
    public function upgrade903050090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_transaction', function(Alter $table)
        {
            $table->addColumn('fees', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('pos', 'decimal', '10,2')->setDefault('0.00');
        });
    }

    // Upgrade to 3.6.0
    public function upgrade903060090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_transaction', function(Alter $table)
        {
            $table->addColumn('txn_id_pos', 'varchar', 30)->setDefault('');
            $table->addColumn('token', 'varchar', 20)->nullable()->setDefault('NULL');
            $table->addColumn('status', 'enum')->values(['initialized', 'pending', 'completed', 'expired'])->setDefault('completed');
        });
    }

    // Upgrade to 3.8.0
    public function upgrade903080090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_purchase_add_info', function(Alter $table)
        {
            $table->addColumn('name', 'varchar', 200)->setDefault('');
        });

        $this->query("UPDATE xfa_rmmp_purchase_add_info SET name=CONCAT_WS(' ',first_name,last_name)");

        $sm->alterTable('xfa_rmmp_purchase_add_info', function(Alter $table)
        {
            $table->dropColumns(['first_name', 'last_name']);
        });

        $sm->createTable('xfa_rmmp_seller_info', function(Create $table)
        {
            $table->addColumn('user_id', 'int');
            $table->addColumn('activate_invoicing', 'tinyint')->setDefault(0);
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('invoice_prefix', 'varchar', 5);
            $table->addColumn('address', 'mediumtext');
            $table->addColumn('additional_info', 'mediumtext');
            $table->addColumn('logo_date', 'int');
            $table->addUniqueKey('user_id', 'user_id');
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_accept_other', 'tinyint', 1)->setDefault(0);
        });

        $sm->alterTable('xfa_rmmp_transaction', function(Alter $table)
        {
            $table->changeColumn('status', 'enum')->values(['initialized', 'pending', 'completed', 'expired','pending_manual'])->setDefault('completed');
        });
    }

    // Upgrade to 3.9.0
    public function upgrade903090090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_purchase', function(Alter $table)
        {
            $table->addColumn('shipment_data', 'varchar', 500)->setDefault('');
        });
    }

    // Upgrade to 3.9.2
    public function upgrade903090290Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->changeColumn('xfa_rmmp_cgv', 'mediumtext')->nullable()->setDefault(null);
        });
    }

    // Upgrade 4.0.0
    public function upgrade904000090Step1()
    {
        $this->_addWidgets();
    }

    // Upgrade 4.1.0
    public function upgrade904010095Step1()
    {
        \XF::app()->jobManager()->enqueueUnique(
            'XFARMMPUpgrade904010090Resource',
            'XFA\RMMarketplace:Upgrade\Resource904010090',
            [],
            false
        );
    }

    // Upgrade 4.2.0
    public function upgrade904020090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_restrict_support', 'tinyint', 1)->setDefault(0);
        });
    }

    // Upgrade 4.3.0
    public function upgrade904030090Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_license', function(Alter $table)
        {
            $table->addColumn('purchase_id', 'int')->setDefault(0);
            $table->addColumn('expiration_sent', 'tinyint')->setDefault(0);
        });

        $sm->alterTable('xfa_rmmp_purchase', function(Alter $table)
        {
            $table->addColumn('quantity', 'int')->setDefault(1);
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_max_qty', 'int')->setDefault(1);
        });

        $sm->alterTable('xf_user', function (Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_sales', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_user_sales_amount', 'decimal', '10,2')->setDefault('0.00');
        });
    }

    public function upgrade904030090Step2()
    {
        // Flag already expired license as sent to avoid alerts
        $this->db()->query('
            UPDATE xfa_rmmp_license
            SET expiration_sent=1
            WHERE validated = 1 AND end_date > 0 AND end_date < ' . \XF::$time . '
        ');
    }

    public function upgrade904030090Step3()
    {
        // Reconstruct purchases data after removal of license id in purchase table and add of purchase_id in license table
        \XF::app()->jobManager()->enqueueUnique(
            'XFARMMPUpgrade904030090Purchase',
            'XFA\RMMarketplace:Upgrade\Purchase904030090',
            [],
            false
        );

        // Rebuild user data (sales number and sales total amount)
        \XF::app()->jobManager()->enqueueUnique(
            'XFARMMPUpgrade904030090User',
            'XFA\RMMarketplace:Upgrade\User904030090',
            [],
            false
        );
    }

    // Upgrade to 4.3.2
    public function upgrade904030290Step1()
    {
        // Rebuild resources purchases count
        \XF::app()->jobManager()->enqueueUnique(
            'XFARMMPUpgrade904030290Resource',
            'XFA\RMMarketplace:Upgrade\Resource904030290',
            [],
            false
        );
    }

    // Upgrade to 5.0.0
    public function upgrade905000090Step1()
    {
        $this->db()->query("
			REPLACE INTO xf_purchasable
				(purchasable_type_id, purchasable_class, addon_id)
			VALUES
				('xfa_rmmp_purchase', 'XFA\\\\RMMarketplace:MarketplacePurchase', 'XFA/RMMarketplace'),
				('xfa_rmmp_renew', 'XFA\\\\RMMarketplace:MarketplaceRenew', 'XFA/RMMarketplace'),
				('xfa_rmmp_feature', 'XFA\\\\RMMarketplace:MarketplaceFeature', 'XFA/RMMarketplace'),
				('xfa_rmmp_invoice', 'XFA\\\\RMMarketplace:MarketplaceInvoice', 'XFA/RMMarketplace')
		");
    }
    
    public function upgrade905000090Step2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->renameColumn('xfa_rmmp_paypal_cat', 'xfa_rmmp_payment_data');
        });

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_paypal', 'varchar', 250)->setDefault('');
        });

        $sm->alterTable('xf_payment_profile', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_id', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_active', 'int')->setDefault(0);
        });

        $sm->alterTable('xfa_rmmp_purchase', function(Alter $table)
        {        
			$table->addColumn('for_user_id', 'int');
			$table->addColumn('for_username', 'varchar', 100)->setDefault('');
        });
        
        $sm->alterTable('xfa_rmmp_transaction', function(Alter $table)
        {        
        	$table->addColumn('pos_paid', 'enum')->values(['na', 'pending', 'completed'])->setDefault('na');
            $table->addColumn('pos_invoice_id', 'int')->setDefault(0);
            $table->addColumn('shipping', 'decimal', '10,2')->setDefault('0.00');
        });
    }     

    public function upgrade905000090Step3()
    {
        $sm = $this->schemaManager();
        
        $sm->createTable('xfa_rmmp_invoices', function(Create $table)
        {
            $table->addColumn('invoice_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 100)->setDefault('');
            $table->addColumn('price', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('currency', 'varchar', 3)->setDefault('');
            $table->addColumn('status', 'enum')->values(['pending', 'validated', 'refunded', 'expired'])->setDefault('pending');
            $table->addColumn('invoice_date', 'int');            
        });
    }

    // 5.0.2
    public function upgrade905000293Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_physical_product', function(Alter $table)
        {
            $table->addColumn('shipping', 'text')->nullable()->setDefault(null);
        });
    }

    // 5.1.0
    public function upgrade905010000Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_user_blocked', 'tinyint')->setDefault(0);
        });

        $sm->alterTable('xfa_rmmp_digital_product', function(Alter $table)
        {
            $table->addColumn('allow_expired_renewal', 'tinyint')->setDefault(1);
            $table->addColumn('base_renewal_price', 'decimal', '10,2')->setDefault('0.00');
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_base_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('xfa_rmmp_discount_id', 'int')->setDefault(0);
            $table->addColumn('xfa_rmmp_sales_disabled', 'tinyint', 1)->setDefault(0);
        });

        $sm->alterTable('xfa_rmmp_invoices', function(Alter $table)
        {
            $table->addColumn('warned', 'tinyint')->setDefault(1);
        });

        $sm->alterTable('xfa_rmmp_seller_info', function(Alter $table)
        {
            $table->addColumn('comm_type', 'enum')->values(['email', 'pm', 'both'])->setDefault('email');
            $table->addColumn('comm_title_purchase', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_purchase', 'mediumtext');
            $table->addColumn('comm_title_renewal', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_renewal', 'mediumtext');
            $table->addColumn('comm_title_purchase_refund', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_purchase_refund', 'mediumtext');
            $table->addColumn('comm_title_renewal_refund', 'varchar', 500)->setDefault('');
            $table->addColumn('comm_text_renewal_refund', 'mediumtext');
        });
    }

    public function upgrade905010000Step2()
    {
        $sm = $this->schemaManager();

        $sm->createTable('xfa_rmmp_discount', function(Create $table)
        {
            $table->addColumn('discount_id', 'int')->autoIncrement();
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('creator_id', 'int')->setDefault(0);
            $table->addColumn('discount_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('is_fixed', 'tinyint')->setDefault(0);
            $table->addColumn('applyto_renewal', 'tinyint')->setDefault(0);
            $table->addColumn('type', 'enum')->values(['all', 'all_own', 'specifics'])->setDefault('all_own');
            $table->addColumn('resource_ids', 'mediumtext');
            $table->addColumn('from_date', 'int')->setDefault(0);
            $table->addColumn('to_date', 'int')->setDefault(0);
            $table->addColumn('applied', 'tinyint')->setDefault(0);
        });
    }

    // 5.1.1
    public function upgrade905010100Step1()
    {
        // Rebuild resources purchases count
        \XF::app()->jobManager()->enqueueUnique(
            'XFARMMPUpgrade905010100Resource',
            'XFA\RMMarketplace:Upgrade\Resource905010100',
            [],
            false
        );
    }

    // 5.2.0
    public function upgrade905020000Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_digital_product', function(Alter $table)
        {
            $table->addColumn('lic_key_format', 'mediumtext')->nullable()->setDefault(null);
        });

        $sm->alterTable('xfa_rmmp_license', function(Alter $table)
        {
            $table->addColumn('lic_key', 'mediumtext')->nullable()->setDefault(null);
        });
    }

    // 5.2.1
    public function upgrade905020100Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xfa_rmmp_seller_info', function(Alter $table)
        {
            $table->changeColumn('invoice_prefix', 'varchar', 5)->setDefault('');
            $table->changeColumn('address', 'mediumtext')->nullable();
            $table->changeColumn('additional_info', 'mediumtext')->nullable();
        });
    }

    // 5.3.0
    public function upgrade905030000Step1()
    {
        $sm = $this->schemaManager();

        $sm->renameTable('xfa_rmmp_transaction',        'xf_xfa_rmmp_transaction');
        $sm->renameTable('xfa_rmmp_purchase',           'xf_xfa_rmmp_purchase');
        $sm->renameTable('xfa_rmmp_purchase_add_info',  'xf_xfa_rmmp_purchase_add_info');
        $sm->renameTable('xfa_rmmp_digital_product',    'xf_xfa_rmmp_digital_product');
        $sm->renameTable('xfa_rmmp_physical_product',   'xf_xfa_rmmp_physical_product');
        $sm->renameTable('xfa_rmmp_license',            'xf_xfa_rmmp_license');
        $sm->renameTable('xfa_rmmp_seller_info',        'xf_xfa_rmmp_seller_info');
        $sm->renameTable('xfa_rmmp_featuring_purchase', 'xf_xfa_rmmp_featuring_purchase');
        $sm->renameTable('xfa_rmmp_featuring_duration', 'xf_xfa_rmmp_featuring_duration');
        $sm->renameTable('xfa_rmmp_invoices',           'xf_xfa_rmmp_invoices');
        $sm->renameTable('xfa_rmmp_discount',           'xf_xfa_rmmp_discount');
    }

    public function upgrade905030000Step2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->addColumn('xfa_rmmp_parent_id', 'int')->setDefault(0);
        });
    }

    public function upgrade905030000Step3()
    {
        $sm = $this->schemaManager();

        $sm->createTable('xf_xfa_rmmp_coupon', function(Create $table)
        {
            $table->addColumn('coupon_id', 'int')->autoIncrement();
            $table->addColumn('name', 'varchar', 100)->setDefault('');
            $table->addColumn('code', 'varchar', 100)->setDefault('');
            $table->addColumn('creator_id', 'int')->setDefault(0);
            $table->addColumn('coupon_type', 'enum')->values(['free', 'discount', 'percentage'])->setDefault('percentage');
            $table->addColumn('coupon_amount', 'decimal', '10,2')->setDefault('0.00');
            $table->addColumn('apply_to', 'enum')->values(['purchase', 'renewal', 'both'])->setDefault('purchase');
            $table->addColumn('type', 'enum')->values(['all', 'all_own', 'specifics'])->setDefault('all_own');
            $table->addColumn('resource_ids', 'mediumtext');
            $table->addColumn('from_date', 'int')->setDefault(0);
            $table->addColumn('to_date', 'int')->setDefault(0);
            $table->addColumn('usergroups', 'varchar', 500)->setDefault('');
            $table->addColumn('user_ids', 'varchar', 500)->setDefault('');
            $table->addColumn('authorized_usages', 'int')->unsigned(false)->setDefault(-1);
            $table->addColumn('total_usages', 'int')->setDefault(0);
        });

        $sm->createTable('xf_xfa_rmmp_coupon_usage', function(Create $table)
        {
            $table->addColumn('usage_id', 'int')->autoIncrement();
            $table->addColumn('coupon_id', 'int')->setDefault(0);
            $table->addColumn('purchase_id', 'int')->setDefault(0);
            $table->addColumn('user_id', 'int')->setDefault(0);
            $table->addKey(['coupon_id','user_id'], 'coupon_user');
        });
    }

    // Delete tables
    public function uninstallStep1(array $stepParams = [])
    {
        $sm = $this->schemaManager();

        $sm->dropTable('xf_xfa_rmmp_transaction');
        $sm->dropTable('xf_xfa_rmmp_purchase');
        $sm->dropTable('xf_xfa_rmmp_purchase_add_info');
        $sm->dropTable('xf_xfa_rmmp_digital_product');
        $sm->dropTable('xf_xfa_rmmp_physical_product');
        $sm->dropTable('xf_xfa_rmmp_license');
        $sm->dropTable('xf_xfa_rmmp_seller_info');
        $sm->dropTable('xf_xfa_rmmp_featuring_purchase');
        $sm->dropTable('xf_xfa_rmmp_featuring_duration');
        $sm->dropTable('xf_xfa_rmmp_invoices');
        $sm->dropTable('xf_xfa_rmmp_discount');
        $sm->dropTable('xf_xfa_rmmp_coupon');
        $sm->dropTable('xf_xfa_rmmp_coupon_usage');
    }

    // Undo alter tables
    public function uninstallStep2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_rm_category', function(Alter $table)
        {
            $table->dropColumns(['xfa_rmmp_conf', 'xfa_rmmp_types', 'xfa_rmmp_payment_data', 'xfa_rmmp_sale_percentage', 'xfa_rmmp_sale_percentage_paypal', 'xfa_rmmp_alt_display', 'xfa_rmmp_restrict_support']);
        });

        $sm->alterTable('xf_rm_resource', function(Alter $table)
        {
            $table->dropColumns(['xfa_rmmp_type', 'xfa_rmmp_max_qty', 'xfa_rmmp_amount', 'xfa_rmmp_currency', 'xfa_rmmp_paypal', 'xfa_rmmp_accept_other', 'xfa_rmmp_purchases', 'xfa_rmmp_cgv', 'xfa_rmmp_img_date','xfa_rmmp_base_amount', 'xfa_rmmp_discount_id','xfa_rmmp_sales_disabled','xfa_rmmp_parent_id']);
        });

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->dropColumns(['xfa_rmmp_user_purchases', 'xfa_rmmp_cgv', 'xfa_rmmp_user_sales', 'xfa_rmmp_user_sales_amount','xfa_rmmp_user_paypal','xfa_rmmp_user_blocked']);
        });

        $sm->alterTable('xf_payment_profile', function(Alter $table)
        {
            $table->dropColumns(['xfa_rmmp_user_id', 'xfa_rmmp_active']);
        });
    }

    // Delete widget
    public function uninstallStep3()
    {
        $this->deleteWidget('xfa_rmmp_top_sales_overview');
        $this->deleteWidget('xfa_rmmp_last_sales_own_dashboard');
        $this->deleteWidget('xfa_rmmp_top_sales_own_dashboard');
        $this->deleteWidget('xfa_rmmp_last_sales_all_dashboard');
        $this->deleteWidget('xfa_rmmp_top_sales_all_dashboard');
    }

    protected function _addWidgets()
    {
        $this->createWidget(
            'xfa_rmmp_top_sales_overview',
            'xfa_rmmp_top_sales',
            [
                'positions' => [
                    'xfrm_overview_sidenav' => 150,
                    'xfrm_category_sidenav' => 150,
                ],
                'options' => [
                    'type'   => 'all',
                    'items' => 10
                ]
            ],
            'Top purchases'
        );

        $this->createWidget(
            'xfa_rmmp_last_sales_own_dashboard',
            'xfa_rmmp_last_sales',
            [
                'positions' => [
                    'xfa_rmmp_sidenav' => 10
                ],
                'options' => [
                    'type'  => 'own',
                    'items' => 15
                ]
            ],
            'Your last sales'
        );

        $this->createWidget(
            'xfa_rmmp_top_sales_own_dashboard',
            'xfa_rmmp_top_sales',
            [
                'positions' => [
                    'xfa_rmmp_sidenav' => 20
                ],
                'options' => [
                    'type'  => 'own',
                    'items' => 15
                ]
            ],
            'Your top sales'
        );

        $this->createWidget(
            'xfa_rmmp_last_sales_all_dashboard',
            'xfa_rmmp_last_sales',
            [
                'positions' => [
                    'xfa_rmmp_sidenav' => 30
                ],
                'options' => [
                    'type'  => 'all',
                    'items' => 15
                ]
            ],
            'Overall last sales'
        );

        $this->createWidget(
            'xfa_rmmp_top_sales_all_dashboard',
            'xfa_rmmp_top_sales',
            [
                'positions' => [
                    'xfa_rmmp_sidenav' => 40
                ],
                'options' => [
                    'type'  => 'all',
                    'items' => 15
                ]
            ],
            'Overall top sales'
        );
    }
}