<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Util;

class DomainValidator
{
    public static function checkValidity($url, $verificationMethod, $blacklistedUrls)
    {
        if($verificationMethod)
        {
            /* Check valid url provided */
            if (!filter_var('http://' . $url, FILTER_VALIDATE_URL))
            {
                return false;
            }

            /* Check url is not blacklisted if licence checking is active */
            if ($blacklistedUrls)
            {
                $blacklistedDomains = preg_split('/[\r\n]+/', $blacklistedUrls, -1, PREG_SPLIT_NO_EMPTY);

                if (in_array($url, $blacklistedDomains))
                {
                    return false;
                }
            }
        }

        $url = 'http://' . $url;

        /* Reach the site to check it exists */
        switch($verificationMethod)
        {
            case 'disable':
                return true;
                break;
            case 'method_1':
                /* Get Answer with CURL */
                $curlInit = curl_init($url);
                curl_setopt($curlInit, CURLOPT_CONNECTTIMEOUT, 10);
                curl_setopt($curlInit, CURLOPT_HEADER, true);
                curl_setopt($curlInit, CURLOPT_NOBODY, true);
                curl_setopt($curlInit, CURLOPT_RETURNTRANSFER, true);

                $response = curl_exec($curlInit);

                curl_close($curlInit);

                /* Return status */
                if ($response)
                {
                    return true;
                }
                else
                {
                    return false;
                }
                break;
            case 'method_2':
                $cookie = '';

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (compatible; CrawlBot/1.0.0)');
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_COOKIEJAR, $cookie);
                curl_setopt($ch, CURLOPT_HEADER, true);
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT , 5);
                curl_setopt($ch, CURLOPT_TIMEOUT, 5);
                curl_setopt($ch, CURLOPT_ENCODING, "");
                curl_setopt($ch, CURLOPT_AUTOREFERER, true);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);    # required for https urls
                curl_setopt($ch, CURLOPT_MAXREDIRS, 15);
                $page=curl_exec($ch);

                $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                curl_close($ch);

                if($httpcode>=200 && $httpcode<300)
                {
                    return true;
                }
                else
                {
                    return false;
                }
                break;
            case 'method_3':
                $cookie = '';

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (compatible; CrawlBot/1.0.0)');
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_COOKIEJAR, $cookie);
                curl_setopt($ch, CURLOPT_HEADER, true);
                curl_setopt($ch, CURLOPT_CONNECTTIMEOUT , 5);
                curl_setopt($ch, CURLOPT_TIMEOUT, 5);
                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
                curl_setopt($ch, CURLOPT_ENCODING, "");
                curl_setopt($ch, CURLOPT_AUTOREFERER, true);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);    # required for https urls
                curl_setopt($ch, CURLOPT_MAXREDIRS, 15);
                $page=curl_exec($ch);

                $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                curl_close($ch);

                if($httpcode>=200 && $httpcode<300)
                {
                    return true;
                }
                else
                {
                    return false;
                }
                break;
            case 'method_4':
                ini_set("default_socket_timeout","05");
                set_time_limit(5);

                $f = fopen($url, "r");
                $r = fread($f,1000);
                fclose($f);

                if(strlen($r)>1)
                {
                    return true;
                }
                else
                {
                    return false;
                }
                break;
        }

    }
}