<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Util;

class UserDataEncrypter
{	
    public static function getEncryptionKeyAndCreateIfNotExists($userId)
    {
        // Construct path
        $path = sprintf('/xfa/rmmarketplace/keys/%d/%d.data',
            floor($userId / 1000),
            $userId
        );
        $path = \XF::getRootDirectory() . '/' . \XF::app()->config()['internalDataPath'] . $path;

        // Key not yet created, create it
        if (!file_exists($path) || !is_file($path))
        {
            $cypherKeyLen   = openssl_cipher_iv_length("AES-128-CBC");
            $cipherKey   	= openssl_random_pseudo_bytes($cypherKeyLen);
            
            \XF\Util\File::writeFile($path, $cipherKey);            
        }
        else
        {
            $cipherKey 		= file_get_contents($path);
        }
        
        return $cipherKey;
    }

    public static function decrypt($userId, $data, $cipherKey)
    {
        $c = base64_decode($data);

        $initializationVectorLen    = openssl_cipher_iv_length("AES-128-CBC");
        $initializationVector       = substr($c, 0, $initializationVectorLen);

        $cipheredTextRaw    = substr($c, $initializationVectorLen+32); // 32 = sha256 length

        return openssl_decrypt($cipheredTextRaw, "AES-128-CBC", $cipherKey, $options=OPENSSL_RAW_DATA, $initializationVector);
    }

    public static function encrypt($userId, $data, $cipherKey)
    {
        $initializationVectorLen  = openssl_cipher_iv_length("AES-128-CBC");
        $initializationVector     = openssl_random_pseudo_bytes($initializationVectorLen);

        $cipheredTextRaw    = openssl_encrypt($data, "AES-128-CBC", $cipherKey, $options=OPENSSL_RAW_DATA, $initializationVector);
        $hmac               = hash_hmac('sha256', $cipheredTextRaw, $cipherKey, $as_binary=true);
        $cipheredText       = base64_encode( $initializationVector.$hmac.$cipheredTextRaw );

        return $cipheredText;
    }
}