<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Widget;

use XF\Widget\AbstractWidget;

class LastSales extends AbstractWidget
{
    public function render()
    {
        /* Check user can view resources */
        $visitor = \XF::visitor();
        if (!method_exists($visitor, 'canViewResources') || !$visitor->canViewResources())
        {
            return '';
        }

        /* Get options */
        $options    = $this->options;
        $nbItems    = $options['items'];
        $type       = $options['type'];

        // Check user can view all purchases if not own
        if ($type != 'own' && (!method_exists($visitor, 'canViewAllSales') || !$visitor->canViewAllSales()))
        {
            return '';
        }

        /* Check we are in a category, in which case get the descendants categories */
        $hasCategoryContext = (
            isset($this->contextParams['category'])
            && $this->contextParams['category'] instanceof \XFRM\Entity\Category
        );
        $useContext = false;

        if ($hasCategoryContext)
        {
            $category               = $this->contextParams['category'];
            $viewableDescendants    = $category->getViewableDescendants();
            $sourceCategoryIds      = array_keys($viewableDescendants);
            $sourceCategoryIds[]    = $category->resource_category_id;

            $useContext = true;
        }
        else
        {
            $sourceCategoryIds = null;
        }

        /* Get resources and filter viewable resources */
        $finder = $this->repository('XFA\RMMarketplace:Purchase')->findLastSales($type, $sourceCategoryIds);

        if (!$useContext)
        {
            // with the context, we already fetched the category and permissions
            $finder->with('Resource.Category.Permissions|' . $visitor->permission_combination_id);
        }

        // We get twice more purchases as we are filtering afterwards, this should be sufficient
        $purchases = $finder->fetch(max($nbItems * 2, 10));

        /** @var \XFRM\Entity\ResourceItem $resource */
        foreach ($purchases AS $purchaseId => $purchase)
        {
            if (!$purchase->Resource->canView())
            {
                unset($purchases[$purchaseId]);
            }
        }

        $purchases  = $purchases->slice(0, $nbItems, true);

        $viewParams = [
            'title'     => $this->getTitle(),
            'purchases' => $purchases
        ];
        return $this->renderer('xfa_rmmp_widget_last_sales', $viewParams);
    }
}