<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XF\Entity;

use XFA\RMMarketplace\Util\UserDataEncrypter;

class PaymentProfile extends XFCP_PaymentProfile
{	
	protected function _saveToSource()
	{				
		if ($this->xfa_rmmp_user_id && array_key_exists('options', $this->_newValues))
		{
			$structure 	= $this->_structure;
			$columns 	= $structure->columns;
		
			$data = $this->_em->encodeValueForSource($columns['options']['type'], $this->_newValues['options']);
			
			$userId = \XF::visitor()->user_id;
			
			// Get key or create if not existing
			$cipherKey = UserDataEncrypter::getEncryptionKeyAndCreateIfNotExists($userId);
			
			// Encrypt and change value
			$this->_newValues['options'] = [UserDataEncrypter::encrypt($userId, $data, $cipherKey)];
		}
		
		return parent::_saveToSource();
	}	
	
	public function getValue($key)
	{
		$value = parent::getValue($key);

		if (isset($this->_values['xfa_rmmp_user_id']) 
				&& $this->_values['xfa_rmmp_user_id'] 
				&& $key == 'options'
				&& !array_key_exists('options', $this->_newValues))
		{
			// Get initial data
			$columns = $this->_structure->columns;
			$column = $columns[$key];
			
			$value = $this->_em->decodeValueFromSourceExtended($column['type'], $this->_values[$key], $column);

			$userId = $this->_values['xfa_rmmp_user_id'];
			
			// Get key or create if not existing
			$cipherKey = UserDataEncrypter::getEncryptionKeyAndCreateIfNotExists($userId);

			// Decrypt and update cache
			$value = UserDataEncrypter::decrypt($userId, $value[0], $cipherKey);			
	
			$value = $this->_em->decodeValueFromSourceExtended($column['type'], $value, $column);

			$this->_valueCache['options'] = $value;
		}

		return $value;
	}
}