<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Admin\Controller;

class Category extends XFCP_Category
{
    protected function categorySaveProcess(\XFRM\Entity\Category $category)
    {
        $form = parent::categorySaveProcess($category);

        $inputData = $this->filter([
            'xfa_rmmp_conf'                     => 'uint',
            'xfa_rmmp_sale_percentage'          => 'float',
            'xfa_rmmp_sale_percentage_paypal'   => 'str',
            'xfa_rmmp_types'                    => [
                'free'      => 'uint',
                'digital'   => ['active' => 'uint', 'renewal' => 'uint', 'restriction' => 'uint', 'url_required' => 'uint'],
                'physical'  => 'uint',
                'service'   => 'uint'
            ],
            'xfa_rmmp_alt_display'              => 'uint',
            'xfa_rmmp_restrict_support'         => 'uint'
        ]);

        // Clean payment data if conf != 2
        if ($inputData['xfa_rmmp_conf'] != 2)
        {
            $inputData['xfa_rmmp_payment_data'] = "";
        }
        else
        {
			// Mode custom
			if (\XF::options()->xfa_rmmp_mode == 'custom')
			{
				$inputData['xfa_rmmp_payment_data'] = $this->filter('xfa_rmmp_payment_data', 'str');
			}	        
			else
			{
				$inputData['xfa_rmmp_payment_data'] = implode(',', $this->filter('xfa_rmmp_payment_data', 'array-uint'));
			}
        }

        /* Set 0 values to types */
        if (!isset($inputData['xfa_rmmp_types']['digital']['active']))
        {
            $inputData['xfa_rmmp_types']['digital']['active'] = 0;
        }

        if (!isset($inputData['xfa_rmmp_types']['digital']['renewal']))
        {
            $inputData['xfa_rmmp_types']['digital']['renewal'] = 0;
        }

        if (!isset($inputData['xfa_rmmp_types']['digital']['restriction']))
        {
            $inputData['xfa_rmmp_types']['digital']['restriction'] = 0;
        }

        if (!isset($inputData['xfa_rmmp_types']['digital']['url_required']))
        {
            $inputData['xfa_rmmp_types']['digital']['url_required'] = 0;
        }

        if (!isset($inputData['xfa_rmmp_types']['physical']))
        {
            $inputData['xfa_rmmp_types']['physical'] = 0;
        }

        if (!isset($inputData['xfa_rmmp_types']['service']))
        {
            $inputData['xfa_rmmp_types']['service'] = 0;
        }

        $form->setup(function() use ($category, $inputData)
        {
            $category->bulkSet($inputData);
        });

        return $form;
    }
}