<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Entity;

class Category extends XFCP_Category
{
    public function canAskForPayment()
    {
        if (!\XF::visitor()->user_id || $this->xfa_rmmp_conf == 0)
        {
            return false;
        }

        return $this->canSellResource();
    }

    public function canSellResource()
    {
        return $this->hasPermission('canAskForPayment');
    }

    public function canSellChildResource()
    {
        return $this->hasPermission('sellChildResources');
    }

    public function canBuyResource()
    {
        return $this->hasPermission('canBuy');
    }
    
    public function canBuyResourceForFriend()
    {
	    return $this->hasPermission('canBuyForFriend');
    }

    public function canAcceptOtherPaymentMeans()
    {
        return $this->hasPermission('acceptOtherPaymentMeans');
    }

    public function canDownloadWithoutBuying()
    {
        return $this->hasPermission('canDownloadWithoutBuying');
    }

    public function canViewAllPurchasesLog()
    {
        return $this->hasPermission('canViewAllPurchasesLog');
    }

    public function canPurchaseFeaturing()
    {
        return $this->hasPermission('canPurchaseFeaturing');
    }

    public function canActivateAutoSerialKeyGen()
    {
        if (\XF::options()->xfa_rmmp_allowLicKeyGen
                && $this->hasPermission('activateAutoSerialKeyGen'))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function hasOnlyOneMarketplaceAllowedType()
    {
        if ($this->xfa_rmmp_conf == 0)
        {
            return false;
        }

        $count = 0;
        if ($this->xfa_rmmp_types['digital']['active'])
        {
            $count++;
        }

        if ($this->xfa_rmmp_types['physical'])
        {
            $count++;
        }

        if ($this->xfa_rmmp_types['service'])
        {
            $count++;
        }

        if ($count > 1)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
    
    public function hasReachedMaxNumberOfSales()
    {
	    $maxAllowedNumberOfSales = $this->hasPermission('maxNumberResourcesForSale');
	    
	    // No limit
	    if ($maxAllowedNumberOfSales <= 0)
	    {
		    return false;
	    }
	    
	    $rssFinder = \XF::finder('XFRM:ResourceItem');
	    $rssFinder->where('user_id', \XF::visitor()->user_id);
	    $rssFinder->where('resource_state', 'visible');
	    $rssFinder->where('xfa_rmmp_type', '<>', 'none');

	    if ($rssFinder->total() >= $maxAllowedNumberOfSales)
	    {
		    return true;
	    }
	    
	    return false;
    }
    
    public function getCategoryPaymentData()
    {
	    if (\XF::options()->xfa_rmmp_mode == 'custom')
	    {
		    return $this->xfa_rmmp_payment_data;
	    }
	    else
	    {
		    $paymentData = explode(',', $this->xfa_rmmp_payment_data);

		    // Return nothing is the only value of the retrieved array is not numeric (could be a paypal address from a previous setup, we ignore)
		    if (count($paymentData) == 1 && !is_numeric($paymentData[0]))
		    {
			    return "";
		    }
		    else
		    {
			    return $paymentData;
		    }
	    }
    }
}