<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Entity;

class ResourceItem extends XFCP_ResourceItem
{
    public function canAskForPayment()
    {
        return $this->Category->canAskForPayment();
    }

    public function canDownload(&$error = null)
    {
        $visitor = \XF::visitor();

        $canDownload = parent::canDownload($error);

        /* Default rights not met */
        if (!$canDownload)
        {
            return false;
        }

        /* If user is author or user can download without buying download is authorized */
        if ($this->Category->canDownloadWithoutBuying($error)
            || ($visitor->user_id && $this->user_id == $visitor->user_id))
        {
            return true;
        }

        /* Check if valid license */
        if ($this->xfa_rmmp_type == 'digital' && !$this->DigitalProduct->hasValidLicense())
        {
            return false;
        }

        return true;
    }

    public function canBuy()
    {
        // Can't buy at all
        if (!$this->Category->canBuyResource())
        {
            return false;
        }

        // In case a parent is set, we need to check if the parent was purchased
        if ($this->canSellChildResource()
            && $this->xfa_rmmp_parent_id
            && !$this->ValidParentPurchase)
        {
            return false;
        }

        return true;
    }

    public function canActivateAutoSerialKeyGen()
    {
        if (!\XF::options()->xfa_rmmp_allowLicKeyGen)
        {
            return false;
        }

        if ($this->user_id == \XF::visitor()->user_id)
        {
            return $this->Category->canActivateAutoSerialKeyGen();
        }
        else
        {
            $canActivate = \XF::asVisitor($this->User, function()
            {
                return $this->Category->canActivateAutoSerialKeyGen();
            });

            return $canActivate;
        }
    }

    public function canAcceptOtherPaymentMeans()
    {
        if ($this->user_id == \XF::visitor()->user_id)
        {
            return $this->Category->canAcceptOtherPaymentMeans();
        }
        else
        {
            $canAccept = \XF::asVisitor($this->User, function()
            {
                return $this->Category->canAcceptOtherPaymentMeans();
            });

            return $canAccept;
        }
    }

    public function canSellChildResource()
    {
        if ($this->user_id == \XF::visitor()->user_id)
        {
            return $this->Category->canSellChildResource();
        }
        else
        {
            $canAccept = \XF::asVisitor($this->User, function()
            {
                return $this->Category->canSellChildResource();
            });

            return $canAccept;
        }
    }

    public function isForSaleInMarketplace()
    {
        if (($this->xfa_rmmp_type == 'none') 
        		|| ($this->Category->xfa_rmmp_conf == 0))
        {
            return false;
        }
        
        if ($this->xfa_rmmp_amount > 0.00)
        {
	        return true;
        }
        
        return false;
    }

    public function canEditResourceImage()
    {
        if ($this->canEdit()
            && (
                \XF::options()->xfa_rmmp_activateAltResourceDisplay
                || \XF::options()->xfa_rmmp_activateAltResourceDisplayOnIndex
                || $this->Category->xfa_rmmp_alt_display
            )
        )
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function canBuyFeaturing()
    {
        if ($this->Category->canPurchaseFeaturing()
                && $this->isForSaleInMarketplace()
                && (!$this->Featured || !$this->Featured->feature_date)
                && $this->user_id == \XF::visitor()->user_id)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function canEditAnySale()
    {
        return $this->hasPermission('canEditAnySale');
    }

    public function canBePurchasedInMultipleQty()
    {
        // Can't purchase more than 1
        if ($this->xfa_rmmp_max_qty == 1)
        {
            return false;
        }

        // Physical product and remaining not enough
        if ($this->xfa_rmmp_type == 'physical'
            && $this->PhysicalProduct->quantity <= 1
            && !$this->PhysicalProduct->sell_on_stockout)
        {
            return false;
        }

        return true;
    }

    public function getPurchasableQty()
    {
        // Only physical products specific case to handle as it depends of sell on stockout
        if ($this->xfa_rmmp_type == 'physical'
            && !$this->PhysicalProduct->sell_on_stockout)
        {
            return min($this->PhysicalProduct->quantity, $this->xfa_rmmp_max_qty);
        }

        return ($this->xfa_rmmp_max_qty ? $this->xfa_rmmp_max_qty : -1);
    }

    public function isRequiringCGVValidation()
    {
        if ($this->xfa_rmmp_cgv != '' || $this->User->xfa_rmmp_user_cgv != '')
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function isPurchasable()
    {
        if (!$this->isForSaleInMarketplace())
        {
            return false;
        }

        if (!$this->canBuy())
        {
            return false;
        }

        if ($this->xfa_rmmp_type == 'digital' && !$this->DigitalProduct->isPurchasable())
        {
            return false;
        }

        if ($this->xfa_rmmp_type == 'physical' && !$this->PhysicalProduct->isPurchasable())
        {
            return false;
        }

        return true;
    }
    
    public function isPurchasableForFriend()
    {
	    if ($this->xfa_rmmp_type != 'digital')
	    {
		    return false;
	    }
	    
	    return $this->Category->canBuyResourceForFriend();
    }
    
    public function bypassPercentageOnSale()
    {
	    return $this->User->hasResourceCategoryPermission($this->resource_category_id, 'bypassPercentageOnSale');
    }

    public function removeFromSale()
    {
        $db = $this->db();

        $idQuoted = $db->quote($this->resource_id);

        $transactionIds =  $db->fetchAllColumn("
			SELECT transaction_id
			FROM xf_xfa_rmmp_purchase
			WHERE resource_id = ?
		", $this->resource_id);

        if ($transactionIds)
        {
            $db->delete('xf_xfa_rmmp_transaction', 'transaction_id IN(' . $db->quote($transactionIds) . ')');
        }

        $purchasesId = $db->fetchAllColumn("
			SELECT purchase_id
			FROM xf_xfa_rmmp_purchase
			WHERE resource_id = ?
		", $this->resource_id);

        if ($purchasesId)
        {
            $db->delete('xf_xfa_rmmp_purchase_add_info', 'purchase_id IN(' . $db->quote($purchasesId) . ')');
        }

        $db->delete('xf_xfa_rmmp_purchase', 'resource_id = ' . $idQuoted);
        $db->delete('xf_xfa_rmmp_digital_product', 'resource_id = ' . $idQuoted);
        $db->delete('xf_xfa_rmmp_physical_product', 'resource_id = ' . $idQuoted);
        $db->delete('xf_xfa_rmmp_license', 'resource_id = ' . $idQuoted);
        $db->delete('xf_xfa_rmmp_featuring_purchase', 'resource_id = ' . $idQuoted);

        $this->xfa_rmmp_type = 'none';
        $this->save();
    }

    public function applyDiscount($discount)
    {
        // Register discount on resource
        $this->xfa_rmmp_discount_id = $discount->discount_id;

        // Backup current price
        $this->xfa_rmmp_base_amount = $this->xfa_rmmp_amount;

        // Apply discount
        if ($discount->is_fixed)
        {
            $this->xfa_rmmp_amount = $this->xfa_rmmp_amount - $discount->discount_amount;
        }
        else
        {
            $this->xfa_rmmp_amount = $this->xfa_rmmp_amount * (1.0 - ($discount->discount_amount / 100.0));
        }

        // Apply to renewal if digital product and required
        if ($discount->applyto_renewal
            && $this->xfa_rmmp_type == 'digital')
        {
            $this->DigitalProduct->applyDiscount($discount);
            $this->addCascadedSave($this->DigitalProduct);
        }
    }

    public function removeDiscount()
    {
        // Reset discount
        $this->xfa_rmmp_discount_id = 0;

        // Restore price
        $this->xfa_rmmp_amount = $this->xfa_rmmp_base_amount;

        // Apply to renewal if digital product and required
        if ($this->xfa_rmmp_type == 'digital' && $this->DigitalProduct)
        {
            $this->DigitalProduct->removeDiscount();
            $this->addCascadedSave($this->DigitalProduct);
        }
    }

    public function getAbstractedImagePath($sizeCode = null)
    {
        $resourceId = $this->resource_id;

        return sprintf('data://xfa/rmmarketplace/resource_images/%d/%d.jpg',
            floor($resourceId / 1000),
            $resourceId
        );
    }

    public function getImageUrl($canonical = false)
    {
        $app = $this->app();

        if ($this->xfa_rmmp_img_date)
        {
            $group = floor($this->resource_id / 1000);
            return $app->applyExternalDataUrl(
                "xfa/rmmarketplace/resource_images/{$group}/{$this->resource_id}.jpg?{$this->xfa_rmmp_img_date}",
                $canonical
            );
        }
        else
        {
            return null;
        }
    }

    public function getMarketplacePaymentEmail()
    {
        $email = '';

        switch($this->Category->xfa_rmmp_conf)
        {
            case 1:
                $email = $this->xfa_rmmp_paypal;
                break;
            case 2:
                $email = $this->Category->xfa_rmmp_payment_data;
                break;
            case 3:
            default:
                $email = \XF::options()->xfa_rmmp_paypal;
                break;
        }

        return $email;
    }
    
    public function getDigitalPrefixId()
    {
	    return intval(\XF::options()->xfa_rmmp_digitalResourcePrefix);
    }
    
    public function getPhysicalPrefixId()
    {
	    return intval(\XF::options()->xfa_rmmp_physicalResourcePrefix);
    }
    
    public function getServicePrefixId()
    {
	    return intval(\XF::options()->xfa_rmmp_serviceResourcePrefix);
    }
    
    public function getOutOfStockPrefixId()
    {
	    return intval(\XF::options()->xfa_rmmp_outOfStockResourcePrefix);
    }
    
    public function getInitialPurchasePriceWithShippingIfAny()
    {
	    if ($this->xfa_rmmp_type == 'physical' && $this->PhysicalProduct->shipping)
	    {
		 	$price = $this->xfa_rmmp_amount + $this->PhysicalProduct->shipping[0]['cost'];   
	    }
	    else
	    {
		    $price = $this->xfa_rmmp_amount;
	    }
	    
	    return number_format((float)$price, 2, '.', '');
    }

    public function getComputedResourcePrice($quantity, $coupon)
    {
        $price = $this->xfa_rmmp_amount * $quantity;

        if ($coupon)
        {
            switch($coupon->coupon_type)
            {
                case 'discount':
                    $price = $price - $coupon->coupon_amount;
                    break;
                case 'percentage':
                    $price = $price - ($price * ($coupon->coupon_amount / 100));
                    break;
            }
        }

        return $price;
    }

    protected function _postDelete()
    {
        parent::_postDelete();

        /** @var \XFA\RMMarketplace\Service\ResourceItem\Image $imageService */
        $imageService = $this->app()->service('XFA\RMMarketplace:ResourceItem\Image', $this);
        $imageService->deleteImageForResourceDelete();
    }
}