<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Pub\Controller;

use XF\Mvc\ParameterBag;

class ResourceItem extends XFCP_ResourceItem
{
    public function actionView(ParameterBag $params)
    {
        $reply = parent::actionView($params);

        if ($reply instanceof \XF\Mvc\Reply\View)
        {
            $resource = $reply->getParam('resource');

            if ($resource->canSellChildResource())
            {
                $childResources = $this->finder('XFRM:ResourceItem')->where('xfa_rmmp_parent_id', $resource->resource_id)->fetch();

                $reply->setParam('childResources', $childResources);
            }
        }

        return $reply;
    }

    /*
    public function actionValidate()
    {
        \XF::logError(json_encode($_POST));
    }
    */

    protected function setupResourceChangeType(\XFRM\Entity\ResourceItem $resource)
    {
        $typeChanger = parent::setupResourceChangeType($resource);

        if ($resource->xfa_rmmp_type != 'none')
        {
            $resourceType = $this->filter('resource_type', 'str');

            switch($resource->xfa_rmmp_type)
            {
                case 'digital':
                    if ($resourceType == 'external_purchase' || $resourceType == 'fileless')
                    {
                        $typeChanger->getResource()->error(\XF::phrase('xfa_rmmp_resource_type_change_not_allowed'));
                    }
                    break;
                case 'physical':
                    if ($resourceType == 'download_local' || $resourceType == 'download_external' || $resourceType == 'external_purchase')
                    {
                        $typeChanger->getResource()->error(\XF::phrase('xfa_rmmp_resource_type_change_not_allowed'));
                    }
                    break;
                case 'service':
                    if ($resourceType == 'download_local' || $resourceType == 'download_external' || $resourceType == 'external_purchase')
                    {
                        $typeChanger->getResource()->error(\XF::phrase('xfa_rmmp_resource_type_change_not_allowed'));
                    }
                    break;
            }
        }

        return $typeChanger;
    }

    protected function setupResourceEdit(\XFRM\Entity\ResourceItem $resource)
    {
        $editor     = parent::setupResourceEdit($resource);

        $category   = $editor->getResource()->Category;

        if ($category->canAskForPayment() && $this->filter('resource_payment', 'str') == 'resource_for_sale')
        {
            // Cleanup
            $saleData = $this->filter([
                'xfa_rmmp_parent_id'    => 'uint',
                'xfa_rmmp_max_qty'      => 'uint',
                'xfa_rmmp_type'         => 'str',
                'xfa_rmmp_amount'       => 'str',
                'xfa_rmmp_accept_other' => 'bool',
                'xfa_rmmp_currency'     => 'str'
            ]);

            /* If resource sale is authorized, get the selected product typ (only if not already for sale) */
            $previousProductType = $resource->xfa_rmmp_type;
            if ($previousProductType == 'none')
            {
                $productType = $this->filter('xfa_rmmp_type', 'str');

                /* Check product type is authorized for the category */
                switch ($productType)
                {
                    case 'physical':
                        if ($category->xfa_rmmp_types['physical'] == 0)
                        {
                            $editor->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                            return $editor;
                        }
                        break;
                    case 'digital':
                        if ($category->xfa_rmmp_types['digital']['active'] == 0)
                        {
                            $editor->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                            return $editor;
                        }
                        break;
                    case 'service':
                        if ($category->xfa_rmmp_types['service'] == 0)
                        {
                            $editor->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                            return $editor;
                        }
                        break;
                }
            }
            else
            {
                $productType = $resource->xfa_rmmp_type;

                /* Upon edit, change of product type not allowed */
                unset($saleData['xfa_rmmp_type']);
            }

            $saleData['xfa_rmmp_currency'] = strtoupper($saleData['xfa_rmmp_currency']);

            /* Check minimum price */
            if ((\XF::options()->xfa_rmmp_minimumPrice && $saleData['xfa_rmmp_amount'] < \XF::options()->xfa_rmmp_minimumPrice)
                || $saleData['xfa_rmmp_amount'] == 0)
            {
                $editor->getResource()->error(\XF::phrase('xfa_rmmp_price_below_minimum_required_value'));
                return $editor;
            }

            /* Check if price is input with decimal as coma instead of point */
            if (strpos($saleData['xfa_rmmp_amount'], ',') !== false)
            {
                $saleData['xfa_rmmp_amount'] = str_replace(',', '.', $saleData['xfa_rmmp_amount']);
            }

            /* Handle terms and conditions */
            $saleData['xfa_rmmp_cgv'] = $this->plugin('XF:Editor')->fromInput('xfa_rmmp_cgv');

            /* Get email address only if not set globally */
            if ($category->xfa_rmmp_conf == 1)
            {
                $saleData['xfa_rmmp_paypal'] = $this->filter('xfa_rmmp_paypal', 'str');
            }

            /* Add marketplace data to resource */
            $editor->getResource()->bulkSet($saleData);

            /* Only perform the rest if no errors in the datawriter */
            if ($editor->validate($errors))
            {
                /* Reset validation */
                $editor->resetValidation();

                /* If resource save completed, create product depending on its type, expect for service where nothing needs to be done */
                switch ($productType)
                {
                    /* Physical Product */
                    case 'physical':
                        $inputData = $this->filter([
                            'quantity'          => 'uint',
                            'sell_on_stockout'  => 'uint'
                        ]);

                        $inputData['resource_id'] = $resource->resource_id;

                        if ($previousProductType == 'none')
                        {
                            $physicalProduct = $this->em()->create('XFA\RMMarketplace:PhysicalProduct');
                        }
                        else
                        {
                            $physicalProduct = $resource->PhysicalProduct;
                        }
			                        
			            // Handle shipping costs
			            $filteredShippingCosts = $this->filter('shipping', 'array');
			
			            $shippingCosts = [];
			            foreach ($filteredShippingCosts AS $shippingUnfiltered)
			            {
			                $shipping = $this->filterArray($shippingUnfiltered, ['location' => 'str', 'cost' => 'float']);
			
			                if (!empty($shipping['location']))
			                {
			                    $shippingCosts[] = $shipping;
			                }
			            }
		
		                $inputData['shipping'] = $shippingCosts;
                        
                        $physicalProduct->bulkSet($inputData);
                        $physicalProduct->save();

                        break;

                    /* Digital product */
                    case 'digital':
                        $inputData = [];

                        $inputData['resource_id'] = $resource->resource_id;

                        if ($category->xfa_rmmp_types['digital']['renewal'])
                        {
                            $inputData['duration']              = $this->filter('duration', 'uint');
                            $inputData['duration_type']         = $this->filter('duration_type', 'str');
                            $inputData['delay_renewal']         = $this->filter('delay_renewal', 'uint');
                            $inputData['allow_expired_renewal'] = $this->filter('allow_expired_renewal', 'uint');
                            $inputData['renewal_price']         = $this->filter('renewal_price', 'float');
                        } else
                        {
                            $inputData['duration']              = 0;
                            $inputData['delay_renewal']         = 0;
                            $inputData['allow_expired_renewal'] = 1;
                            $inputData['renewal_price']         = 0;
                        }

                        if ($category->xfa_rmmp_types['digital']['restriction'])
                        {
                            $inputData['has_restriction'] = $this->filter('has_restriction', 'bool');
                        } else
                        {
                            $inputData['has_restriction'] = false;
                        }

                        if ($category->xfa_rmmp_types['digital']['url_required'])
                        {
                            $inputData['require_url'] = $this->filter('require_url', 'bool');
                        } else
                        {
                            $inputData['require_url'] = false;
                        }

                        if ($category->canActivateAutoSerialKeyGen())
                        {
                            $inputData['lic_key_format'] = $this->filter('lic_key_format', 'str');
                        }
                        else
                        {
                            $inputData['lic_key_format'] = "";
                        }

                        if ($previousProductType != 'digital')
                        {
                            $digitalProduct = $this->em()->create('XFA\RMMarketplace:DigitalProduct');
                        }
                        else
                        {
                            $digitalProduct = $resource->DigitalProduct;
                        }
                        $digitalProduct->bulkSet($inputData);
                        $digitalProduct->save();

                        break;
                }
            }
        }

        return $editor;
    }

    public function actionEdit(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);

        $reply = parent::actionEdit($params);

        if ($this->isPost() && $reply instanceof \XF\Mvc\Reply\Redirect)
        {
            if ($resource->canEditResourceImage())
            {
                try
                {
                    /** @var \XFA\RMMarketplace\ControllerPlugin\ResourceImage $plugin */
                    $plugin = $this->plugin('XFA\RMMarketplace:ResourceImage');
                    $plugin->actionUpload($resource);
                }
                catch (\XF\Mvc\Reply\Exception $e)
                {
                }
            }
        }

        if (!$this->isPost() && $reply instanceof \XF\Mvc\Reply\View)
        {
            // If user can send child resources get user resources
            if ($resource->Category->canSellChildResource())
            {
                $reply->setParam(
                    'userResources',
                    $this->repository('XFRM:ResourceItem')->findPaidResourcesByUserOptions(\XF::visitor()->user_id)
                );
            }
        }

        return $reply;
    }

    public function actionMove(ParameterBag $params)
    {
        $response = parent::actionMove($params);

        if ($response instanceof \XF\Mvc\Reply\View)
        {
            $resource = $response->getParam('resource');

            /* Need to reconstruct category list if the resource is paid */
            if ($resource->isForSaleInMarketplace())
            {
                $categoryRepo   = $this->getCategoryRepo();
                $categories     = $categoryRepo->getViewableCategories();

                foreach($categories AS $idx => $category)
                {
                    if ($resource->xfa_rmmp_type == 'digital')
                    {
                        if ($category->xfa_rmmp_types['digital']['active'] == 0)
                        {
                            unset($categories[$idx]);
                        }
                    }
                    else
                    {
                        if ($category->xfa_rmmp_types[$resource->xfa_rmmp_type] == 0)
                        {
                            unset($categories[$idx]);
                        }
                    }
                }

                $response->setParam('categoryTree', $categoryRepo->createCategoryTree($categories));
            }
        }

        return $response;
    }

    public function actionDownload(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        $version = $resource->CurrentVersion;

        if ($resource->xfa_rmmp_type == 'digital' && \XF::visitor()->user_id)
        {
            /** @var \XFA\RMMarketplace\Finder\License $licenseFinder */
            $licenseFinder = $this->finder('XFA\RMMarketplace:License');

            if ($resource->DigitalProduct->duration)
            {
                $license = $licenseFinder->where('resource_id', $version->resource_id)
                    ->where('end_date', '>=', $version->release_date)
                    ->where('user_id', \XF::visitor()->user_id)
                    ->where('active', 1)
                    ->fetchOne();
            }
            else
            {
                $license = $licenseFinder->where('resource_id', $version->resource_id)
                    ->where('user_id', \XF::visitor()->user_id)
                    ->where('active', 1)
                    ->fetchOne();
            }

            if ($license)
            {
                /* Register capability */
                $GLOBALS['xfa_rmmp_dl_authorized'] = true;
            }
        }

        return parent::actionDownload($params);
    }

    public function actionMarketPlaceViewTnc(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);

        $viewParams = [
            'resource' => $resource
        ];
        return $this->view('', 'xfa_rmmp_tnc', $viewParams);
    }

    public function actionImage(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canEdit($error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            /** @var \XFA\RMMarketplace\ResourceItem\Image $imageService */
            $imageService = $this->service('XFA\RMMarketplace:ResourceItem\Image', $resource);

            $action = $this->filter('image_action', 'str');

            if ($action == 'delete')
            {
                $imageService->deleteImage();
            }
            else if ($action == 'custom')
            {
                $upload = $this->request->getFile('image_upload', false, false);
                if ($upload)
                {
                    if (!$imageService->setImageFromUpload($upload))
                    {
                        return $this->error($imageService->getError());
                    }

                    if (!$imageService->updateImage())
                    {
                        return $this->error(\XF::phrase('xfa_new_image_could_not_be_applied_try_later'));
                    }
                }
            }

            return $this->redirect($this->buildLink('resources', $resource));
        }
        else
        {
            $viewParams = [
                'resource' => $resource,
                'category' => $resource->Category
            ];
            return $this->view('', 'xfa_rmmp_resource_edit_image', $viewParams);
        }
    }

    public function actionMarketPlaceRemoveFromSale(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canEdit($error) || !$resource->isForSaleInMarketplace())
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $resource->removeFromSale();

            return $this->redirect($this->buildLink('resources', $resource));
        }
        else
        {
            $viewParams = [
                'resource' => $resource,
                'category' => $resource->Category
            ];
            return $this->view('', 'xfa_rmmp_resource_remove_from_sale', $viewParams);
        }
    }

    public function actionMarketPlaceEnableSale(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canEdit($error) || !$resource->isForSaleInMarketplace())
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $resource->fastUpdate('xfa_rmmp_sales_disabled', 0);

            return $this->redirect($this->buildLink('resources', $resource));
        }
        else
        {
            $viewParams = [
                'resource' => $resource,
                'category' => $resource->Category
            ];

                return $this->view('', 'xfa_rmmp_resource_enable_sale', $viewParams);
        }
    }

    public function actionMarketPlaceDisableSale(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canEdit($error) || !$resource->isForSaleInMarketplace())
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $resource->fastUpdate('xfa_rmmp_sales_disabled', 1);

            return $this->redirect($this->buildLink('resources', $resource));
        }
        else
        {
            $viewParams = [
                'resource' => $resource,
                'category' => $resource->Category
            ];

            return $this->view('', 'xfa_rmmp_resource_disable_sale', $viewParams);
        }
    }

    public function actionMarketPlacePurchases(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);

        /* Check resource is for sale and purchasable */
        if (!$resource->isForSaleInMarketplace())
        {
            return $this->error(\XF::phrase('xfa_rmmp_resource_is_not_paid'));
        }

        /* Check user can view logs */
        if (!$resource->Category->canViewAllPurchasesLog() && $resource->user_id != \XF::visitor()->user_id)
        {
            return $this->error(\XF::phrase('xfa_rmmp_you_are_not_resource_owner'));
        }

        $finder = $this->finder('XFA\RMMarketplace:Purchase');
        $finder->where('resource_id', $resource->resource_id);
        $finder->order('purchase_date', 'desc');

        $total      = $finder->total();
        $page       = $this->filterPage();
        $perPage    = $this->options()->xfa_rmmp_purchasePerPage;

        $this->assertValidPage($page, $perPage, $total, 'resources/market-place-purchases');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-purchases', $resource, ['page' => $page]));

        $purchases = $finder->limitByPage($page, $perPage)->fetch();

        $viewParams = [
            'resource'  => $resource,
            'purchases' => $purchases,
            'page'      => $page,
            'perPage'   => $perPage,
            'total'     => $total
        ];
        return $this->view('', 'xfa_rmmp_purchases', $viewParams);
    }

    public function actionFeaturingPurchase(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canBuyFeaturing())
        {
            return $this->noPermission();
        }

        $durations = $this->finder('XFA\RMMarketplace:FeaturingDuration')->where('active', 1)->order(['display_order'])->fetch();
        if (!$durations)
        {
            return $this->error(\XF::phrase('xfa_rmmp_purchase_unavailable_no_durations_contact_admin'));
        }

        if ($this->isPost())
        {
            return $this->rerouteController(__CLASS__, 'featuring-purchase-create', $params);
        }
        else
        {
            $viewParams = [
                'resource'  => $resource,
                'durations' => $durations
            ];
            return $this->view('', 'xfa_rmmp_featuring_purchase', $viewParams);
        }
    }
    
    public function actionFeaturingPurchaseXf(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canBuyFeaturing())
        {
            return $this->noPermission();
        }

        $durations = $this->finder('XFA\RMMarketplace:FeaturingDuration')->where('active', 1)->order(['display_order'])->fetch();
        if (!$durations)
        {
            return $this->error(\XF::phrase('xfa_rmmp_purchase_unavailable_no_durations_contact_admin'));
        }
        
        
        // Get payment profiles
		$profiles = $this->finder('XF:PaymentProfile')
							->where('payment_profile_id', \XF::options()->xfa_rmmp_paymentProfileIds)
							->where('active', 1)
							->fetch();
        
        if (!count($profiles))
        {
            throw $this->exception($this->noPermission());
        }
        
        
        $viewParams = [
            'resource'  => $resource,
            'durations' => $durations,
            'profiles'	=> $profiles
        ];
        return $this->view('', 'xfa_rmmp_featuring_purchase_xf', $viewParams);        	    
    }

    public function actionFeaturingPurchaseCreate(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);

        $duration = $this->em()->find('XFA\RMMarketplace:FeaturingDuration', $this->filter('duration_id', 'uint'));
        if (!$duration)
        {
            return $this->error(\XF::phrase('xfa_rmmp_requested_featuring_duration_not_found'));
        }

        $transaction = $this->setupFeaturingPurchaseTransactionCreate($resource)->save();

        /* Create the purchase */
        $purchase = $this->setupFeaturingPurchaseCreate($resource, $transaction, $duration)->save();

        /* Configure set express checkout call */
        $setExpressCheckout = $this->service('XFA\Core:PayPal\SetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $setExpressCheckout->setUrls(
            $this->buildLink('canonical:resources/featuring-purchase-done', $purchase),
            $this->buildLink('canonical:resources', $resource)
        );

        $setExpressCheckout->setBuyerEmail(\XF::visitor()->email);

        $setExpressCheckout->setItem(
            $transaction->transaction_id,
            \XF::phrase('xfa_rmmp_purchase_resource_featuring') . ' - ' . $resource->title .' - '. $duration->value . ' ' . $duration->type,
            $duration->amount,
            \XF::options()->xfa_rmmp_featuring_purchase_currency,
            \XF::options()->xfa_rmmp_paypal,
            $this->buildLink('canonical:resources', $resource)
        );

        /* Perform the call and check for errors */
        $setExpressCheckout->run();

        if ($setExpressCheckout->hasErrors())
        {
            return $this->error($setExpressCheckout->getErrors());
        }

        /* Update transaction and redirect to paypal */
        $transaction->token = $setExpressCheckout->getToken();
        $transaction->save();

        $viewParams = [
            'resource'  => $resource,
            'redirect'  => $setExpressCheckout->getRedirectUrl()
        ];
        return $this->view('', 'xfa_core_paypal_express_checkout_redirect', $viewParams);
    }

    public function actionFeaturingPurchaseDone(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);
        if (!$resource->canBuyFeaturing())
        {
            return $this->noPermission();
        }

        /* Check provided token correspond to an existing transaction */
        $token = $this->filter('token', 'str');

        $transaction = \XF::finder('XFA\RMMarketplace:Transaction')->where('token', $token)->fetchOne();
        if (!$transaction)
        {
            return $this->error(\XF::phrase('xfa_rmmp_transaction_doesnt_exist'));
        }

        /* Check transaction and resource are matching */
        if (!$transaction->FeaturingPurchase || $transaction->FeaturingPurchase->resource_id != $resource->resource_id)
        {
            return $this->error(\XF::phrase('xfa_rmmp_purchase_and_resource_do_not_match'));
        }

        /* Configure get express checkout details call */
        $getExpressCheckoutDetails = $this->service('XFA\Core:PayPal\GetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $getExpressCheckoutDetails->setToken($token);

        /* Perform the call and check for errors */
        $getExpressCheckoutDetails->run();

        if ($getExpressCheckoutDetails->hasErrors())
        {
            return $this->error($getExpressCheckoutDetails->getErrors());
        }

        /* Configure get express checkout details call */
        $doExpressCheckout = $this->service('XFA\Core:PayPal\DoExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $doExpressCheckout->setExpressCheckoutDetails($getExpressCheckoutDetails->getResult());
        $doExpressCheckout->setToken($token);

        $doExpressCheckout->setItem(
            $transaction->transaction_id,
            \XF::phrase('xfa_rmmp_purchase_resource_featuring') . ' - ' . $resource->title .' - '. $transaction->FeaturingPurchase->duration,
            $transaction->FeaturingPurchase->price,
            \XF::options()->xfa_rmmp_featuring_purchase_currency,
            \XF::options()->xfa_rmmp_paypal,
            $this->buildLink('canonical:resources', $resource)
        );

        /* Perform the call and check for errors */
        $doExpressCheckout->run();

        if ($doExpressCheckout->hasErrors())
        {
            return $this->error($doExpressCheckout->getErrors());
        }

        $paymentInfo = $doExpressCheckout->getPaymentInfo();

        switch ($paymentInfo['status'])
        {
            case 'Pending':
                /* Update the transaction status */
                $transaction->status = 'pending';
                $transaction->save();

                /* Show pending error */
                $viewParams = [
                    'message' => \XF::phrase('xfa_rmmp_payment_pending_error')->render()
                ];

                return $this->view('', 'message_page', $viewParams);
                break;
            case 'Completed':
            case 'Processed':
                /* Update the transaction data in the database */
                $transaction = $this->setupFeaturingPurchaseTransactionEdit($transaction, $paymentInfo)->save();

                /* Validate the purchase */
                $validator = $this->service('XFA\RMMarketplace:FeaturingPurchase\Validate', $transaction->FeaturingPurchase);
                $validator->validate();

                /* Create the view params array */
                $viewParams = [
                    'resource'  => $resource,
                    'type'      => 'purchase'
                ];

                /* Return success view */
                return $this->view('', 'xfa_rmmp_payment_redirect', $viewParams);
                break;
            default:
                /* Other status shouldn't happen */
                throw $this->exception($this->noPermission());
                break;
        }
    }

    public function actionMarketPlaceEditSale(ParameterBag $params)
    {
        $resource = $this->assertViewableResource($params->resource_id);

        if (!$resource->isForSaleInMarketplace()
                || ($resource->user_id != \XF::visitor()->user_id && !$resource->canEditAnySale()))
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            /** @var \XFRM\Service\ResourceItem\Edit $editor */
		    $editor = $this->service('XFRM:ResourceItem\Edit', $resource);

            /* Cleanup */
            $saleData = $this->filter([
                'xfa_rmmp_max_qty'      => 'uint',
                'xfa_rmmp_amount'       => 'str',
                'xfa_rmmp_accept_other' => 'bool',
                'xfa_rmmp_currency'     => 'str'
            ]);

            $saleData['xfa_rmmp_currency'] = strtoupper($saleData['xfa_rmmp_currency']);

            /* Check minimum price */
            if ((\XF::options()->xfa_rmmp_minimumPrice && $saleData['xfa_rmmp_amount'] < \XF::options()->xfa_rmmp_minimumPrice)
                || $saleData['xfa_rmmp_amount'] == 0)
            {
                $editor->getResource()->error(\XF::phrase('xfa_rmmp_price_below_minimum_required_value'));
                return $editor;
            }

            /* Check if price is input with decimal as coma instead of point */
            if (strpos($saleData['xfa_rmmp_amount'], ',') !== false)
            {
                $saleData['xfa_rmmp_amount'] = str_replace(',', '.', $saleData['xfa_rmmp_amount']);
            }

            /* Handle terms and conditions */
            $saleData['xfa_rmmp_cgv'] = $this->plugin('XF:Editor')->fromInput('xfa_rmmp_cgv');

            /* Get email address only if not set globally */
            if ($editor->getResource()->Category->xfa_rmmp_conf == 1)
            {
                $saleData['xfa_rmmp_paypal'] = $this->filter('xfa_rmmp_paypal', 'str');
            }

            /* Add marketplace data to resource */
            $editor->getResource()->bulkSet($saleData);

            /* Only perform the rest if no errors in the datawriter */
            if ($editor->validate($errors))
            {
                /* Reset validation */
                $editor->resetValidation();

                /* If resource save completed, create product depending on its type, expect for service where nothing needs to be done */
                switch ($editor->getResource()->xfa_rmmp_type)
                {
                    /* Physical Product */
                    case 'physical':
                        $inputData = $this->filter([
                            'quantity' => 'uint',
                            'sell_on_stockout' => 'uint'
                        ]);

                        $physicalProduct = $resource->PhysicalProduct;
                        $physicalProduct->bulkSet($inputData);
                        $physicalProduct->save();

                        break;

                    /* Digital product */
                    case 'digital':
                        $inputData = [];

                        if ($editor->getResource()->Category->xfa_rmmp_types['digital']['renewal'])
                        {
                            $inputData['duration'] = $this->filter('duration', 'uint');
                            $inputData['duration_type']         = $this->filter('duration_type', 'str');
                            $inputData['delay_renewal']         = $this->filter('delay_renewal', 'uint');
                            $inputData['allow_expired_renewal'] = $this->filter('allow_expired_renewal', 'uint');
                            $inputData['renewal_price']         = $this->filter('renewal_price', 'float');
                        } else
                        {
                            $inputData['duration']              = 0;
                            $inputData['delay_renewal']         = 0;
                            $inputData['allow_expired_renewal'] = 1;
                            $inputData['renewal_price']         = 0;
                        }

                        if ($editor->getResource()->Category->xfa_rmmp_types['digital']['restriction'])
                        {
                            $inputData['has_restriction'] = $this->filter('has_restriction', 'bool');
                        } else
                        {
                            $inputData['has_restriction'] = false;
                        }

                        if ($editor->getResource()->Category->xfa_rmmp_types['digital']['url_required'])
                        {
                            $inputData['require_url'] = $this->filter('require_url', 'bool');
                        } else
                        {
                            $inputData['require_url'] = false;
                        }

                        if ($editor->getResource()->canActivateAutoSerialKeyGen())
                        {
                            $inputData['lic_key_format'] = $this->filter('lic_key_format', 'str');
                        }
                        else
                        {
                            $inputData['lic_key_format'] = "";
                        }

                        $digitalProduct = $resource->DigitalProduct;
                        $digitalProduct->bulkSet($inputData);
                        $digitalProduct->save();

                        break;
                }

                $editor->save();

                return $this->redirect($this->buildLink('resources', $resource));
            }
        }
        else
        {
            $viewParams = [
                'resource'  => $resource,
            ];
            return $this->view('', 'xfa_rmmp_edit_sale', $viewParams);
        }
    }

    protected function setupFeaturingPurchaseTransactionCreate(\XFRM\Entity\ResourceItem $resource)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Transaction\Create', $resource);

        $creator->setPaypalEmail(\XF::options()->xfa_rmmp_paypal);

        return $creator;
    }

    protected function setupFeaturingPurchaseTransactionEdit(\XFA\RMMarketplace\Entity\Transaction $transaction, $paymentInfo)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Edit $editor */
        $editor = $this->service('XFA\RMMarketplace:Transaction\Edit', $transaction);

        $editor->setStatus('completed');
        $editor->setTxnId($paymentInfo['transaction_id']);
        $editor->setFeesAmount($paymentInfo['fees_amount']);

        return $editor;
    }

    protected function setupFeaturingPurchaseCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction,
        \XFA\RMMarketplace\Entity\FeaturingDuration $duration
    )
    {
        /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:FeaturingPurchase\Create', $resource, $transaction);

        $creator->setPrice($duration->amount);
        $creator->setDuration($duration);

        return $creator;
    }
}