<?php
/*************************************************************************
 * XFA Core - Xen Factory (c) 2017
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Repository;

class ResourceItem extends XFCP_ResourceItem
{
    public function sendSellerOutOfStockAlert(\XFRM\Entity\ResourceItem $resource)
    {
        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');

        return $alertRepo->alert(
            $resource->User, $resource->user_id, $resource->username, 'xfa_rmmp', $resource->resource_id, 'out_of_stock'
        );
    }

    public function findPaidResourcesByUserOptions($userId)
    {
        $userResources = $this->findResourcesByUser(\XF::visitor()->user_id)
                                ->where('xfa_rmmp_type', '<>', 'none')
                                ->order('title', 'asc')
                                ->fetch();

        $userResourcesList = [];
        foreach ($userResources AS $userResource)
        {
            $userResourcesList[$userResource['resource_id']] = $userResource['title'];
        }

        return $userResourcesList;
    }

    public function findTopSales($type = 'all', array $viewableCategoryIds = null)
    {
        /** @var \XFRM\Finder\ResourceItem $resourceFinder */
        $resourceFinder = $this->finder('XFRM:ResourceItem');

        if (is_array($viewableCategoryIds))
        {
            $resourceFinder->where('resource_category_id', $viewableCategoryIds);
        }
        else
        {
            $resourceFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
        }

        if ($type == 'own')
        {
            $resourceFinder->where('user_id', \XF::visitor()->user_id);
        }

        $resourceFinder
            ->where('resource_state', 'visible')
            ->where('xfa_rmmp_purchases', '>', 0)
            ->with(['User'])
            ->setDefaultOrder('xfa_rmmp_purchases', 'desc');

        return $resourceFinder;
    }

    public function getNonRMDiscountedResourcesByName(array $resourceNames, &$notFound = [])
    {
        $resourceNames = array_map('trim', $resourceNames);
        foreach ($resourceNames AS $key => $resourceName)
        {
            if ($resourceName === '')
            {
                unset($resourceNames[$key]);
            }
        }

        $notFound = [];

        if (!$resourceNames)
        {
            return $this->em->getEmptyCollection();
        }

        $finder = $this->finder('XFRM:ResourceItem')
            ->where('title', $resourceNames)
            ->where('xfa_rmmp_type', '<>', 'none')
            ->where('xfa_rmmp_discount_id', '=', 0);

        $resources = $finder->fetch();
        if ($resources->count() != count($resourceNames))
        {
            $resourceNamesLower = array_map('strtolower', $resourceNames);
            $notFound = $resourceNames;

            foreach ($resources AS $resource)
            {
                do
                {
                    $foundKey = array_search(strtolower($resource['title']), $resourceNamesLower, true);
                    if ($foundKey !== false)
                    {
                        unset($notFound[$foundKey]);
                        unset($resourceNamesLower[$foundKey]);
                    }
                }
                while ($foundKey !== false);
            }
        }

        $orderedResources = [];
        foreach ($resourceNames AS $searchResourceName)
        {
            $searchResourceName = utf8_deaccent(utf8_strtolower($searchResourceName));
            foreach ($resources AS $id => $resource)
            {
                $testResourceName = utf8_deaccent(utf8_strtolower($resource->title));
                if ($searchResourceName == $testResourceName && !isset($orderedResources[$id]))
                {
                    $orderedResources[$id] = $resource;
                }
            }
        }
        foreach ($resources AS $id => $resource)
        {
            if (!isset($orderedResources[$id]))
            {
                $orderedResources[$id] = $resource;
            }
        }

        return $this->em->getBasicCollection($orderedResources);
    }
}